/* Copyright (C) 2003 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "myx_admin_library.h"

#include <myx_aux_functions.h>

/*
 * public functions definitions
 */

MYX_PROCESS_LIST * myx_get_process_list(MYSQL *mysql)
{
  MYX_PROCESS_LIST *process_list= malloc(sizeof(MYX_PROCESS_LIST));
  MYX_PROCESS_INFO *process_info;
  unsigned int i, j;
  MYSQL_RES* res;
  MYSQL_ROW row;
  MYSQL_FIELD *fields;
  unsigned int num_fields;

  process_list->process_infos_num= 0;

  if (!(myx_mysql_query(mysql, "SHOW FULL PROCESSLIST")))
  {
    if ( (res= mysql_store_result(mysql)) )
    {
      process_list->process_infos_num= (int)mysql_num_rows(res);
      process_list->process_infos= malloc(sizeof(MYX_PROCESS_INFO)*process_list->process_infos_num);

      i=0;
      while( (row= mysql_fetch_row(res)) )
      {
        process_info= process_list->process_infos+i;
        process_info->id= NULL;
        process_info->user= NULL;
        process_info->host= NULL;
        process_info->db= NULL;
        process_info->command= NULL;
        process_info->time= NULL;
        process_info->state= NULL;
        process_info->info= NULL;

        //Depending on the version of the server there might be different columns
        num_fields = mysql_num_fields(res);
        fields = mysql_fetch_fields(res);

        for(j=0;j<num_fields;j++)
        {
            if (!(strcmp2(fields[j].name, "Id")))
            {
              process_info->id= malloc(sizeof(char)*((unsigned int)strlen2(row[j])+1));
              strcpy2(process_info->id, row[j]);
            }
            else if (!(strcmp2(fields[j].name, "User")))
            {
              process_info->user= malloc(sizeof(char)*((unsigned int)strlen2(row[j])+1));
              strcpy2(process_info->user, row[j]);
            }
            else if (!(strcmp2(fields[j].name, "Host")))
            {
              process_info->host= malloc(sizeof(char)*((unsigned int)strlen2(row[j])+1));
              strcpy2(process_info->host, row[j]);
            }
            else if (!(strcmp2(fields[j].name, "db")))
            {
              process_info->db= malloc(sizeof(char)*((unsigned int)strlen2(row[j])+1));
              strcpy2(process_info->db, row[j]);
            }
            else if (!(strcmp2(fields[j].name, "Command")))
            {
              process_info->command= malloc(sizeof(char)*((unsigned int)strlen2(row[j])+1));
              strcpy2(process_info->command, row[j]);
            }
            else if (!(strcmp2(fields[j].name, "Time")))
            {
              process_info->time= malloc(sizeof(char)*((unsigned int)strlen2(row[j])+1));
              strcpy2(process_info->time, row[j]);
            }
            else if (!(strcmp2(fields[j].name, "State")))
            {
              process_info->state= malloc(sizeof(char)*((unsigned int)strlen2(row[j])+1));
              strcpy2(process_info->state, row[j]);
            }
            else if (!(strcmp2(fields[j].name, "Info")))
            {
              process_info->info= malloc(sizeof(char)*((unsigned int)strlen2(row[j])+1));
              strcpy2(process_info->info, row[j]);
            }
        }

        i++;
      }
    }
    else
    {
      free(process_list);
      process_list= NULL;
    }

    mysql_free_result(res);
  }
  else
  {
    free(process_list);
    process_list= NULL;
  }


  return process_list;
}

int myx_free_process_list(MYX_PROCESS_LIST *process_list)
{
  unsigned int i;

  for(i=0;i<process_list->process_infos_num;i++)
  {
    free(((MYX_PROCESS_INFO *)(process_list->process_infos+i))->id);
    free(((MYX_PROCESS_INFO *)(process_list->process_infos+i))->user);
    free(((MYX_PROCESS_INFO *)(process_list->process_infos+i))->host);
    free(((MYX_PROCESS_INFO *)(process_list->process_infos+i))->db);
    free(((MYX_PROCESS_INFO *)(process_list->process_infos+i))->command);
    free(((MYX_PROCESS_INFO *)(process_list->process_infos+i))->time);
    free(((MYX_PROCESS_INFO *)(process_list->process_infos+i))->state);
    free(((MYX_PROCESS_INFO *)(process_list->process_infos+i))->info);
  }

  free(process_list->process_infos);
  free(process_list);

  return 0;
}

int myx_kill_thread(MYSQL *mysql, unsigned long pid)
{
  return mysql_kill(mysql, pid);
}

unsigned long myx_get_thread_id(MYSQL *mysql)
{
  return mysql_thread_id(mysql);
}

