/* Copyright (C) 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include "myx_admin_library.h"
#include <myx_xml_aux_functions.h>
#include <myx_aux_functions.h>


/* forward definitions */
static void free_repl_host_content(MYX_REPL_HOST *host);
static void free_repl_user_host_content(MYX_USER_REPL_HOST *rephost);
static int in_sql_query_list(const char *name, MYX_REPL_HOSTS *list);
static int in_userlist(const char *name, MYX_USER_REPL_HOSTS *replist);

/*
 * Public functions
 */

/* replist may be NULL */
MYX_REPL_HOSTS *myx_show_repl_hosts_status(MYSQL *mysql, MYX_USER_REPL_HOSTS *replist, MYX_ADMIN_LIB_ERROR *error_code)
{
  char *stmt;
  MYSQL_RES *result;
  MYSQL_ROW row;
  MYX_REPL_HOSTS *list;
  unsigned int i, j;
  MYX_REPL_HOST *master_host= g_malloc0(sizeof(MYX_REPL_HOST));
  MYSQL_FIELD *fields;
  unsigned int num_fields;

  *error_code= 0;

  //-------------------------------------------------------
  //get master info

  //server_id
  stmt= "show variables like 'server_id'";
  if ( myx_mysql_query(mysql, stmt) || !(result= mysql_store_result(mysql)) )
  {
    *error_code =  MYX_ADMIN_SQL_ERROR;
    g_free(master_host);
    return NULL;
  }
  if(!(row= mysql_fetch_row(result)))
  {
    return NULL;
  }  
  master_host->server_id= atoi(row[1]);
  mysql_free_result(result);

  //Port
  stmt= "show variables like 'port'";
  if ( myx_mysql_query(mysql, stmt) || !(result= mysql_store_result(mysql)) )
  {
    *error_code =  MYX_ADMIN_SQL_ERROR;
    free_repl_host_content(master_host);
    g_free(master_host);
    return NULL;
  }
  if(!(row= mysql_fetch_row(result)))
  {
    free_repl_host_content(master_host);
    g_free(master_host);
    return NULL;
  }  
  master_host->port= atoi(row[1]);
  mysql_free_result(result);

  //
  stmt= "show master status";
  if ( myx_mysql_query(mysql, stmt) || !(result= mysql_store_result(mysql)) )
  {
    *error_code =  MYX_ADMIN_SQL_ERROR;
    free_repl_host_content(master_host);
    g_free(master_host);
    return NULL;
  }
  if(!(row= mysql_fetch_row(result)))
  {
    free_repl_host_content(master_host);
    g_free(master_host);
    return NULL;
  }  
  master_host->binlog_file= g_strdup(row[0]);
  master_host->binlog_pos= g_strdup(row[1]);
  mysql_free_result(result);

  master_host->is_master= 1;
  master_host->host= g_strdup(mysql->host);

  //-------------------------------------------------------
  //add slaves
  list= g_malloc0(sizeof(MYX_REPL_HOSTS));

  stmt= "show slave hosts";
  if ( myx_mysql_query(mysql, stmt) || !(result= mysql_store_result(mysql)) )
  {
    *error_code =  MYX_ADMIN_SQL_ERROR;
    free_repl_host_content(master_host);
    g_free(master_host);
    g_free(list);
    return NULL;
  }

  list->hosts_num= (unsigned int)mysql_num_rows(result)+1;
  list->hosts= g_malloc0(sizeof(MYX_REPL_HOST) * (list->hosts_num));

  num_fields = mysql_num_fields(result);
  fields = mysql_fetch_fields(result);

  //Copy master as first host
  list->hosts[0].server_id= master_host->server_id;
  list->hosts[0].host= g_strdup(master_host->host);
  list->hosts[0].port= master_host->port;
  list->hosts[0].rpl_recovery_rank= master_host->rpl_recovery_rank;
  list->hosts[0].master_id= 0;
  list->hosts[0].is_master= 1;
  list->hosts[0].status= MYX_RHS_NEW_HOST;
  list->hosts[0].binlog_file= g_strdup(master_host->binlog_file);
  list->hosts[0].binlog_pos= g_strdup(master_host->binlog_pos);
  free_repl_host_content(master_host);
  g_free(master_host);

  i= 1;
  while ( (row= mysql_fetch_row(result)) )
  {
    for(j=0;j<num_fields;j++)
    {
      //if the column is Host, set host
      if(strcmp(fields[j].name, "Server_id")==0)
      {
        list->hosts[i].server_id= atoi(row[j]);
      }
      else if(strcmp(fields[j].name, "Host")==0)
      {
        list->hosts[i].host= g_strdup(row[j]);
      }
      else if(strcmp(fields[j].name, "Port")==0)
      {
        list->hosts[i].port= atoi(row[4]);
      }
      else if(strcmp(fields[j].name, "Rpl_recovery_rank")==0)
      {
        list->hosts[i].rpl_recovery_rank= atoi(row[5]);
      }
      else if(strcmp(fields[j].name, "Master_id")==0)
      {
        list->hosts[i].master_id= atoi(row[6]);
      }
    }

    list->hosts[i].is_master= 0;
    list->hosts[i].status= MYX_RHS_NEW_HOST;
    list->hosts[i].binlog_file= NULL;
    list->hosts[i].binlog_pos= NULL;

    i++;
  }

  /* compare with replist */
  if (replist)
  {
    for (i=0; i < list->hosts_num; i++)
    {
      list->hosts[i].status= (in_userlist(list->hosts[i].host, replist))? MYX_RHS_AVAILABLE : MYX_RHS_NEW_HOST ;
    }

    //now traverse the userlist and search the red ones
    for (i=0; i < replist->hosts_num; i++)
    {
      if ( !in_sql_query_list(replist->hosts[i].name, list) )
      {
	      //add this host as a red one
	      MYX_REPL_HOST *slave;

	      list->hosts_num++;
	      list->hosts= g_realloc(list->hosts, sizeof(MYX_REPL_HOST) * list->hosts_num);
	      slave= list->hosts + list->hosts_num-1;

	      slave->server_id= 0;
        slave->host= g_strdup(replist->hosts[i].name);
	      slave->port= 0;
	      slave->rpl_recovery_rank= 0;
	      slave->master_id= 0;
	      slave->status= MYX_RHS_NOT_AVAILABLE;
        slave->binlog_file= NULL;
        slave->binlog_pos= NULL;
        slave->is_master= 0;
      }
    }
  }

  mysql_free_result(result);

  return list;
}


int myx_free_repl_hosts_status(MYX_REPL_HOSTS *list)
{
  unsigned int i;

  if (list)
  {
    for (i=0; i < list->hosts_num; i++)
    {
      free_repl_host_content(list->hosts+i);
    }
    g_free(list->hosts);
    g_free(list);
  }

  return 0;
}


MYX_USER_REPL_HOSTS *myx_read_repl_user_hosts(const char *filename, MYX_ADMIN_LIB_ERROR *error_code)
{
  MYX_USER_REPL_HOSTS *user_replist;
  xmlDocPtr doc;
  xmlParserCtxtPtr ctxt;
  xmlNodePtr root;

  *error_code = MYX_ADMIN_NO_ERROR;

  if (! file_exists(filename) )
  {
    *error_code = MYX_ADMIN_ERROR_CANT_OPEN_FILE;
    return NULL;
  }

  /* create a parser context */
  ctxt = xmlNewParserCtxt();
  if (ctxt == NULL) 
  {
    return NULL;
  }
  /* parse the file, activating the DTD validation option */
  doc = xmlCtxtReadFile(ctxt, filename, NULL, 0);

  /* check if parsing suceeded */
  if (doc == NULL) 
  {
    *error_code = MYX_ADMIN_XML_PARSE_ERROR;
    xmlFreeParserCtxt(ctxt);
    return NULL;
  } 
  else 
  {
    /* check if validation suceeded */
    if (ctxt->valid == 0)
    {
      *error_code = MYX_ADMIN_XML_NO_VALID_DOCUMENT;
      xmlFreeParserCtxt(ctxt);	    
      xmlFreeDoc(doc);
      return NULL;
    }
  }
  /* free up the parser context */
  xmlFreeParserCtxt(ctxt);

      
  root = xmlDocGetRootElement(doc);
    
  if (root == NULL) {
    *error_code = MYX_ADMIN_XML_EMPTY_DOCUMENT;
    xmlFreeDoc(doc);
    return NULL;
  }

  user_replist= g_malloc0(sizeof(MYX_USER_REPL_HOSTS));

  user_replist->hosts_num= get_child_count(root, "rephost");

  if (user_replist->hosts_num  > 0)
  {
    int i;
    xmlNodePtr cur;

    user_replist->hosts= g_malloc0(sizeof(MYX_USER_REPL_HOST) * user_replist->hosts_num);

    i=0;
    cur = root->children;
    while (cur != NULL) 
    {
      if (!xmlStrcmp(cur->name, "rephost"))
      {
	      user_replist->hosts[i].name=  xmlGetProp(cur, "name");
	      i++;
      }

      cur = cur->next;
    }
  }

  xmlFreeDoc(doc);
    
  return user_replist;
}

int myx_save_repl_user_hosts(const MYX_USER_REPL_HOSTS *replist, const char *filename)
{
  xmlDocPtr doc;
  xmlNodePtr root_node, node;
  int i;

  doc = xmlNewDoc("1.0");

  root_node = doc->children = xmlNewDocRawNode(doc,NULL, "user_replist", NULL);  

    
  for (i=0; i < (int)replist->hosts_num; i++)
  {
    node= xmlNewTextChild(root_node, NULL, "rephost", NULL);  
    xmlSetProp(node, "name", replist->hosts[i].name); 
  }

  if (xmlSaveFormatFile(filename, doc, 1) == -1)
  {
    xmlFreeDoc(doc);
    return -1;
  }

  xmlFreeDoc(doc);

  return 0;
}


int myx_free_repl_user_hosts(MYX_USER_REPL_HOSTS *replist)
{
  unsigned int i;

  if (replist)
  {
    for (i=0; i < replist->hosts_num; i++)
    {
      free_repl_user_host_content(replist->hosts+i);
    }
    g_free(replist->hosts);
    g_free(replist);
  }
  return 0;
}





/*
 * Private functions
 */

static int in_sql_query_list(const char *name, MYX_REPL_HOSTS *list)
{
  unsigned int i;

  for (i=0; i < list->hosts_num; i++)
  {
    if (!xmlStrcmp(name, list->hosts[i].host))
      return 1;
  }

  return 0;
}

static int in_userlist(const char *name, MYX_USER_REPL_HOSTS *replist)
{
  unsigned int i;

  for (i=0; i< replist->hosts_num; i++)
  {
    if (!xmlStrcmp(name, replist->hosts[i].name))
    {
      return 1;
    }
  }

  return 0;
}

static void free_repl_host_content(MYX_REPL_HOST *host)
{
  if(host->host)
    g_free(host->host);
  if(host->binlog_file)
    g_free(host->binlog_file);
  if(host->binlog_pos)
    g_free(host->binlog_pos);
}


static void free_repl_user_host_content(MYX_USER_REPL_HOST *rephost)
{
  xmlFree(rephost->name);
}
