/* Copyright (C) 2003 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "myx_admin_library.h"

#define USER_INFO_TABLENAME "user_info"

/*
 * public functions definitions
 */

MYX_USER_NAMES * myx_get_user_names(MYSQL *mysql)
{
    MYX_USER_NAMES *user_names= malloc(sizeof(MYX_USER_NAMES));
    int i;
    MYSQL_RES* res;
    MYSQL_ROW row;
    unsigned long *lengths;
    
    user_names->user_names_num= 0;

    //Get Total number of users and hosts
    //to allocate one block of memory
    
    if (!(myx_mysql_query(mysql, "SELECT DISTINCT User FROM mysql.user ORDER BY User")))
    {
      if ( (res= mysql_store_result(mysql)) )
      {
	      user_names->user_names_num= (int)mysql_num_rows(res);
	      user_names->user_names= malloc(sizeof(char *)*user_names->user_names_num);
      	
	      i=0;
	      while ( (row= mysql_fetch_row(res)) )
	      {
          //user_names->user_names[i]= strdup(row[0]);
	        lengths= mysql_fetch_lengths(res);
      	  
	        user_names->user_names[i]= malloc(sizeof(char)*(lengths[0]+1));
	        memcpy(user_names->user_names[i], row[0], lengths[0]);
	        user_names->user_names[i][lengths[0]]= 0;
      	  
	        i++;
	      }
      }
      else
      {
	      free(user_names);
	      user_names= NULL;
      }

        mysql_free_result(res);
    }
    else
    {
        free(user_names);
        user_names= NULL;
    }

    return user_names;
}

int myx_free_user_names(MYX_USER_NAMES *user_names)
{
    unsigned int i;

    for(i=0;i<user_names->user_names_num;i++)
    {
        free(user_names->user_names[i]);
    }
    free(user_names->user_names);
    free(user_names);

    return 0;
}

int myx_check_mysql_user_info_table(MYSQL *mysql)
{
    MYSQL_RES* res;
    int result;

    result= 0;

    mysql_select_db(mysql, "mysql");

    if (!(res= mysql_list_tables(mysql, USER_INFO_TABLENAME)))
    {
        return 0;
    }
    else
    {
        //If the USER_INFO_TABLE does not exist, create it
        if (mysql_num_rows(res)==0)
        {
            //Create table
            if (myx_mysql_query(mysql, "CREATE TABLE mysql." USER_INFO_TABLENAME "(User VARCHAR(16) NOT NULL, Full_name VARCHAR(60), "\
                "Description VARCHAR(255), Email VARCHAR(80), Contact_information TEXT, Icon BLOB, PRIMARY KEY(User), "\
                "INDEX user_info_Full_name(Full_name)) TYPE=MYISAM COMMENT='Stores additional user information'"))
            {
                //error
            }
            else
            {
                result= 1;
            }
        }
        else
        {
            result= 1;
        }

        mysql_free_result(res);
    }

    return result;
}

MYX_USER * myx_get_user_opt_privileges(MYSQL *mysql, const char *user_name, int get_privileges)
{
    MYX_USER *user= malloc(sizeof(MYX_USER));
    MYX_USER_OBJECT_PRIVILEGES *user_priv, *def_priv;
    MYSQL_RES* res;
    MYSQL_ROW row;
    MYSQL_FIELD *fields;
    unsigned int num_fields;

    unsigned long *field_lengths;
    unsigned char sqlcmd[255];
    unsigned char s[255], s1[60];
    unsigned int i, j;

    char *token;

    //Copy user_name
    user->user_name= malloc(sizeof(char)*(strlen(user_name)+1));
    strcpy(user->user_name, user_name);

    user->password= malloc(sizeof(char));
    *user->password= 0;
    user->full_name= malloc(sizeof(char));
    *user->full_name= 0;
    user->description= malloc(sizeof(char));
    *user->description= 0;
    user->email= malloc(sizeof(char));
    *user->email= 0;
    user->contact_information= malloc(sizeof(char));
    *user->contact_information= 0;
    user->icon_length= 0;
    user->icon= NULL;

    user->hosts_num= 0;
    user->hosts= NULL;

    user->user_object_privileges_num= 0;
    user->user_object_privileges= NULL;

    // ------------------------------------------------------------------------------------------------------------------
    //Get additional user infos
    sprintf(sqlcmd, "SELECT ui.Full_name, ui.Description, ui.Email, ui.Contact_information, ui.Icon "\
        "FROM mysql." USER_INFO_TABLENAME " ui  "\
        "WHERE ui.User='%s'", user_name);

    if (!(myx_mysql_query(mysql, sqlcmd)))
    {
        if ( (res= mysql_store_result(mysql)) )
        {
	    if ( (row= mysql_fetch_row(res)))
            {
                user->full_name= realloc(user->full_name, sizeof(char)*(strlen(row[0])+1));
                strcpy(user->full_name, row[0]);

                user->description= realloc(user->description, sizeof(char)*(strlen(row[1])+1));
                strcpy(user->description, row[1]);

                user->email= realloc(user->email, sizeof(char)*(strlen(row[2])+1));
                strcpy(user->email, row[2]);

                user->contact_information= realloc(user->contact_information, sizeof(char)*(strlen(row[3])+1));
                strcpy(user->contact_information, row[3]);

                //Get field lengths
                field_lengths= mysql_fetch_lengths(res);

                //Copy Icon data
                user->icon_length= field_lengths[4];
                if (user->icon_length>0)
                {
                    user->icon= malloc(sizeof(char)*user->icon_length);
                    memcpy(user->icon, row[4], user->icon_length);
                }
            }
        }
        else
        {
            //error
            myx_free_user(user);
            return NULL;
        }

        mysql_free_result(res);
    }
    else
    {
        // error
        // Check if user_info tables exists. if not, create it
        if (!(myx_check_mysql_user_info_table(mysql)))
        {
            myx_free_user(user);
            return NULL;
        }
    }

    // ------------------------------------------------------------------------------------------------------------------
    // get all hosts the user can connect from and
    // get global privileges and max_ values
    sprintf(sqlcmd, "SELECT * "\
        "FROM mysql.user "\
        "WHERE User='%s'", user_name);

    if (!(myx_mysql_query(mysql, sqlcmd)))
    {
        if ( (res= mysql_store_result(mysql)))
        {
            i= 0;
            if (mysql_num_rows(res)==0)
            {
              myx_free_user(user);
              return NULL;
            }

            user->hosts_num= (unsigned int)mysql_num_rows(res);
            user->hosts= malloc(sizeof(char *)*user->hosts_num);

            user->user_object_privileges_num= user->hosts_num;
            user->user_object_privileges= malloc(sizeof(MYX_USER_OBJECT_PRIVILEGES)*user->user_object_privileges_num);

            while ((row= mysql_fetch_row(res)))
            {
                user_priv= &user->user_object_privileges[i];

                //Depending on the version of the server there might be different columns
                num_fields = mysql_num_fields(res);
                fields = mysql_fetch_fields(res);

                user_priv->user_privileges_num= 0;
                user_priv->user_privileges= malloc(sizeof(char **)*num_fields-2);

                for(j=0;j<num_fields;j++)
                {
                    //if the column is Host, set host
                    if (strcmp(fields[j].name, "Host")==0)
                    {
                        user->hosts[i]= malloc(sizeof(char)*strlen(row[0])+1);
                        strcpy(user->hosts[i], row[0]);

                        user_priv->host= malloc(sizeof(char)*strlen(row[0])+1);
                        strcpy(user_priv->host, row[0]);

                        user_priv->object_name= NULL;
                    } 
                    else if ((strcmp(str_right(s, fields[j].name, 5), "_priv")==0)||\
                        (strcmp(str_left(s, fields[j].name, 4), "max_")==0))
                    {
                        user_priv->user_privileges_num++;
                        user_priv->user_privileges[user_priv->user_privileges_num-1]= malloc(sizeof(char)*strlen(fields[j].name)+strlen(row[j])+2);
                        sprintf(user_priv->user_privileges[user_priv->user_privileges_num-1], "%s=%s", fields[j].name, row[j]);
                    }
                }
                i++;
            }
        }
        else
        {
            myx_free_user(user);
            return NULL;
        }

        mysql_free_result(res);
    }
    else
    {
        myx_free_user(user);
        return NULL;
    }

    if(get_privileges)
    {
      // ------------------------------------------------------------------------------------------------------------------
      // get schema privileges
      sprintf(sqlcmd, "SELECT * "\
          "FROM mysql.db "\
          "WHERE User='%s'", user_name);

      if (!(myx_mysql_query(mysql, sqlcmd)))
      {
	  if ( (res= mysql_store_result(mysql)))
          {
              i= user->user_object_privileges_num;

              user->user_object_privileges_num= user->user_object_privileges_num+(unsigned int)mysql_num_rows(res);
              user->user_object_privileges= realloc(user->user_object_privileges, sizeof(MYX_USER_OBJECT_PRIVILEGES)*user->user_object_privileges_num);

              while ((row= mysql_fetch_row(res)))
              {
                  user_priv= &user->user_object_privileges[i];

                  //Depending on the version of the server there might be different columns
                  num_fields = mysql_num_fields(res);
                  fields = mysql_fetch_fields(res);

                  user_priv->user_privileges_num= 0;
                  user_priv->user_privileges= malloc(sizeof(char **)*num_fields-2);

                  for(j=0;j<num_fields;j++)
                  {
                      //if the column is Host, set host
                      if (strcmp(fields[j].name, "Host")==0)
                      {
                          user_priv->host= malloc(sizeof(char)*strlen(row[j])+1);
                          strcpy(user_priv->host, row[j]);
                      }
                      //if the column is Db, set object_name
                      else if (strcmp(fields[j].name, "Db")==0)
                      {
                          user_priv->object_name= malloc(sizeof(char)*strlen(row[j])+1);
                          strcpy(user_priv->object_name, row[j]);
                      }
                      else if (strcmp(str_right(s, fields[j].name, 5), "_priv")==0)
                      {
                          user_priv->user_privileges_num++;
                          user_priv->user_privileges[user_priv->user_privileges_num-1]= malloc(sizeof(char)*strlen(fields[j].name)+2+1);
                          sprintf(user_priv->user_privileges[user_priv->user_privileges_num-1], "%s=%s", fields[j].name, row[j]);
                      }
                  }
                  i++;
              }
          }
          else
          {
              myx_free_user(user);
              return NULL;
          }

          mysql_free_result(res);
      }
      else
      {
          myx_free_user(user);
          return NULL;
      }

      // ------------------------------------------------------------------------------------------------------------------
      // get table privileges
      sprintf(sqlcmd, "SELECT Host, Db, Table_name, mysql.tables_priv.* "\
          "FROM mysql.tables_priv "\
          "WHERE User='%s'", user_name);

      if (!(myx_mysql_query(mysql, sqlcmd)))
      {
	if ( (res= mysql_store_result(mysql)) )
          {
            //Depending on the version of the server there might be different columns
            num_fields= mysql_num_fields(res);
            fields= mysql_fetch_fields(res);

            i= user->user_object_privileges_num;

            user->user_object_privileges_num= user->user_object_privileges_num+(unsigned int)mysql_num_rows(res);
            user->user_object_privileges= realloc(user->user_object_privileges, sizeof(MYX_USER_OBJECT_PRIVILEGES)*user->user_object_privileges_num);

            def_priv= myx_get_privilege_struct(mysql, "some.table");

            while ((row= mysql_fetch_row(res)))
            {
              user_priv= &user->user_object_privileges[i];

              //Fill host and objectname
              user_priv->host= malloc(sizeof(char)*(strlen(row[0])+1));
              strcpy(user_priv->host, row[0]);

              user_priv->object_name= malloc(sizeof(char)*(strlen(row[1])+strlen(row[2])+2));
              sprintf(user_priv->object_name, "%s.%s", row[1], row[2]);

              //Copy all user privileges from default
              user_priv->user_privileges_num= def_priv->user_privileges_num;
              user_priv->user_privileges= malloc(sizeof(char **)*user_priv->user_privileges_num);

              for(j=0;j<def_priv->user_privileges_num;j++)
              {
                user_priv->user_privileges[j]= malloc(sizeof(char)*(strlen(def_priv->user_privileges[j])+1));
                strcpy(user_priv->user_privileges[j], def_priv->user_privileges[j]);
              }

              for(j=3;j<num_fields;j++)
              {
                if (strcmp(str_right(s, fields[j].name, 5), "_priv")==0)
                {
                  strcpy(s, row[j]);

                  token = strtok(s, ",");
                  while(token)
                  {
                    sprintf(s1, "%s_%s", fields[j].name, token);
                    set_value((char**)user_priv->user_privileges, user_priv->user_privileges_num, s1, "Y");

                    token = strtok(NULL, ",");
                  }
                }
              }

              i++;
            }

            free(def_priv);
          }
          else
          {
              myx_free_user(user);
              return NULL;
          }

          mysql_free_result(res);
      }
      else
      {
          myx_free_user(user);
          return NULL;
      }

      // ------------------------------------------------------------------------------------------------------------------
      // get column privileges (basically the same like table priv, so should be optimized to one codeblock)
      sprintf(sqlcmd, "SELECT Host, Db, Table_name, Column_name, mysql.columns_priv.* "\
          "FROM mysql.columns_priv "\
          "WHERE User='%s'", user_name);

      if (!(myx_mysql_query(mysql, sqlcmd)))
      {
	if ( (res= mysql_store_result(mysql)))
          {
            //Depending on the version of the server there might be different columns
            num_fields= mysql_num_fields(res);
            fields= mysql_fetch_fields(res);

            i= user->user_object_privileges_num;

            user->user_object_privileges_num= user->user_object_privileges_num+(unsigned int)mysql_num_rows(res);
            user->user_object_privileges= realloc(user->user_object_privileges, sizeof(MYX_USER_OBJECT_PRIVILEGES)*user->user_object_privileges_num);

            def_priv= myx_get_privilege_struct(mysql, "some.table.column");

            while ((row= mysql_fetch_row(res)))
            {
              user_priv= &user->user_object_privileges[i];

              //Fill host and objectname
              user_priv->host= malloc(sizeof(char)*(strlen(row[0])+1));
              strcpy(user_priv->host, row[0]);

              user_priv->object_name= malloc(sizeof(char)*(strlen(row[1])+strlen(row[2])+strlen(row[3])+3));
              sprintf(user_priv->object_name, "%s.%s.%s", row[1], row[2], row[3]);

              //Copy all user privileges from default
              user_priv->user_privileges_num= def_priv->user_privileges_num;
              user_priv->user_privileges= malloc(sizeof(char **)*user_priv->user_privileges_num);

              for(j=0;j<def_priv->user_privileges_num;j++)
              {
                user_priv->user_privileges[j]= malloc(sizeof(char)*(strlen(def_priv->user_privileges[j])+1));
                strcpy(user_priv->user_privileges[j], def_priv->user_privileges[j]);
              }

              for(j=4;j<num_fields;j++)
              {
                if (strcmp(str_right(s, fields[j].name, 5), "_priv")==0)
                {
                  strcpy(s, row[j]);

                  token = strtok(s, ",");
                  while(token)
                  {
                    sprintf(s1, "%s_%s", fields[j].name, token);
                    set_value((char**)user_priv->user_privileges, user_priv->user_privileges_num, s1, "Y");

                    token = strtok(NULL, ",");
                  }
                }
              }

              i++;
            }

            free(def_priv);
          }
          else
          {
              myx_free_user(user);
              return NULL;
          }

          mysql_free_result(res);
      }
      else
      {
          myx_free_user(user);
          return NULL;
      }
    }

    return user;
}

MYX_USER * myx_get_user(MYSQL *mysql, const char *user_name)
{
   return myx_get_user_opt_privileges(mysql, user_name, 0);
}

MYX_USER * myx_get_user_with_privileges(MYSQL *mysql, const char *user_name)
{
  return myx_get_user_opt_privileges(mysql, user_name, 1);
}

int myx_free_user(MYX_USER *user)
{
    unsigned int i;

    free(user->user_name);
    free(user->password);
    free(user->full_name);
    free(user->description);
    free(user->email);
    free(user->contact_information);
    free(user->icon);

    for(i=0;i<user->hosts_num;i++)
    {
        free(user->hosts[i]);
    }
    free(user->hosts);

    for(i=0;i<user->user_object_privileges_num;i++)
    {
        myx_free_user_priv((MYX_USER_OBJECT_PRIVILEGES *)(user->user_object_privileges+i)); 
    }
    free(user->user_object_privileges);

    free(user);

    return 0;
}

int myx_free_user_priv(MYX_USER_OBJECT_PRIVILEGES *user_priv)
{
    unsigned int i;

    free(user_priv->host);
    free(user_priv->object_name);

    for(i=0;i<user_priv->user_privileges_num;i++)
    {
        free(user_priv->user_privileges[i]);
    }

    free(user_priv->user_privileges);

    return 0;
}

int myx_set_user(MYSQL *mysql, MYX_USER *user, const char *previous_user_name)
{
  MYX_USER *current_user;
  unsigned char sqlcmd[500];
  unsigned char field[30];
  unsigned char value[30];
  unsigned char *schema_name, *table_name, *column_name;
  unsigned char *s;
  unsigned char *user_name;
  unsigned int i, j, k;
  int dot_count, do_it, makeInsert;
  int icon_changed = 0;
  unsigned char *long_sqlcmd, *sqlcmd_end;
  char *encoded_password= NULL;
  MYSQL_RES* res;
  MYSQL_ROW row;

  //Update existing User
  if (previous_user_name[0])
  {
    if (!(current_user= myx_get_user(mysql, previous_user_name)))
    {
      return -1;
    }

    // check if name has changed
    if (strcmp(user->user_name, current_user->user_name))
    {
      //Update user name in mysql.user table
      sqlcmd_end= strmov(sqlcmd, "UPDATE mysql.user SET User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "' WHERE User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, current_user->user_name, (unsigned long)strlen(current_user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "'");

      if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
        return -1; }

      //Update user name in mysql.db table
      sqlcmd_end= strmov(sqlcmd, "UPDATE mysql.db SET User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "' WHERE User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, current_user->user_name, (unsigned long)strlen(current_user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "'");

      if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
        return -1; }

      //Update user name in mysql.tables_priv table
      sqlcmd_end= strmov(sqlcmd, "UPDATE mysql.tables_priv SET User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "' WHERE User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, current_user->user_name, (unsigned long)strlen(current_user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "'");

      if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
        return -1; }

      //Update user name in mysql.columns_priv table
      sqlcmd_end= strmov(sqlcmd, "UPDATE mysql.columns_priv SET User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "' WHERE User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, current_user->user_name, (unsigned long)strlen(current_user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "'");

      if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
        return -1; }
    }

    //Check if all user/host combination has already been created
    for(i=0;i<user->hosts_num;i++)
    {
      do_it=1;
      for(j=0;j<current_user->hosts_num;j++)
      {
        if (!strcmp(*(user->hosts+i), *(user->hosts+j)))
        {
          do_it=0;
          break;
        }
      }

      //If not, insert a new user/host combination, take password from existing Host
      if (do_it)
      {
        if(!encoded_password)
        {
          sqlcmd_end= strmov(sqlcmd, "SELECT Password FROM mysql.user WHERE User='");
          sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
          sqlcmd_end = strmov(sqlcmd_end, "' ORDER BY Host LIMIT 1");

          if (!(myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))))
          {
            if ( (res= mysql_store_result(mysql)) )
            {
              if ( (row= mysql_fetch_row(res)) )
              {
                encoded_password= strdup(row[0]);
              }
            }
          }
        }

        sqlcmd_end= strmov(sqlcmd, "INSERT INTO mysql.user(User, Host, Password) VALUES('");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
        sqlcmd_end = strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, *(user->hosts+i), (unsigned long)strlen(*(user->hosts+i)));
        sqlcmd_end = strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, encoded_password, (unsigned long)strlen(encoded_password));
        sqlcmd_end = strmov(sqlcmd_end, "')");

        if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
          return -1; }
      }
    }

    if(encoded_password)
    {
      free(encoded_password);
      encoded_password= NULL;
    }

    //Check for obsolete user/host combination
    for(i=0;i<current_user->hosts_num;i++)
    {
      do_it=1;
      for(j=0;j<user->hosts_num;j++)
      {
        if (!strcmp(*(current_user->hosts+i), *(user->hosts+j)))
        {
          do_it=0;
          break;
        }
      }

      //If there is a obsolete row, delete it
      if (do_it)
      {
        sqlcmd_end= strmov(sqlcmd, "DELETE FROM mysql.user WHERE User='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, current_user->user_name, (unsigned long)strlen(current_user->user_name));
        sqlcmd_end = strmov(sqlcmd_end, "' AND Host='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, *(current_user->hosts+i), (unsigned long)strlen(*(current_user->hosts+i)));
        sqlcmd_end = strmov(sqlcmd_end, "'");

        if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
          return -1; }
      }
    }

    //Check if the icon has changed
    icon_changed= (user->icon_length!=current_user->icon_length);
    if ((!icon_changed)&&(user->icon_length>0))
    {
      icon_changed= (memcmp(user->icon, current_user->icon, user->icon_length)!=0);
    }

    //check if user information has changed
    if (
      (strcmp(user->user_name, current_user->user_name))||
      (strcmp(user->full_name, current_user->full_name))||
      (strcmp(user->description, current_user->description))||
      (strcmp(user->email, current_user->email))||
      (strcmp(user->contact_information, current_user->contact_information))||
      (icon_changed)
      )
    {
      //Check if there already is an entry in the mysql.user_info table
      makeInsert= 0;

      sqlcmd_end= strmov(sqlcmd, "SELECT User FROM mysql." USER_INFO_TABLENAME " WHERE User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
      sqlcmd_end= strmov(sqlcmd_end, "'");

      if (!(myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))))
      {
        if ( (res= mysql_store_result(mysql)) )
        {
          if(!(mysql_num_rows(res)))
          {
            makeInsert= 1;
          }
        }

        mysql_free_result(res);
      }
      else
      {
        return -1;
      }

      
      long_sqlcmd= malloc(sizeof(char)*(unsigned long)(strlen(user->user_name)+strlen(user->full_name)+strlen(user->description)+
          strlen(user->email)+strlen(user->contact_information)+200));

      //Update user info in mysql.user_info table
      if(!(makeInsert))
      {
        sqlcmd_end= strmov(long_sqlcmd, "UPDATE mysql." USER_INFO_TABLENAME " SET User='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
        sqlcmd_end = strmov(sqlcmd_end, "', Full_name='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->full_name, (unsigned long)strlen(user->full_name));
        sqlcmd_end = strmov(sqlcmd_end, "', Description='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->description, (unsigned long)strlen(user->description));
        sqlcmd_end = strmov(sqlcmd_end, "', Email='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->email, (unsigned long)strlen(user->email));
        sqlcmd_end = strmov(sqlcmd_end, "', Contact_information='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->contact_information, (unsigned long)strlen(user->contact_information));
        sqlcmd_end = strmov(sqlcmd_end, "' WHERE User='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, current_user->user_name, (unsigned long)strlen(current_user->user_name));
        sqlcmd_end = strmov(sqlcmd_end, "'");
      }
      //Insert user info into mysql.user_info table
      else
      {
        sqlcmd_end= strmov(long_sqlcmd, "INSERT INTO mysql." USER_INFO_TABLENAME "(User, Full_name, Description, "\
          "Email, Contact_information) VALUES('");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
        sqlcmd_end = strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->full_name, (unsigned long)strlen(user->full_name));
        sqlcmd_end = strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->description, (unsigned long)strlen(user->description));
        sqlcmd_end = strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->email, (unsigned long)strlen(user->email));
        sqlcmd_end = strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->contact_information, (unsigned long)strlen(user->contact_information));
        sqlcmd_end = strmov(sqlcmd_end, "')");
      }

      if (myx_mysql_real_query(mysql, long_sqlcmd, (unsigned int)(sqlcmd_end-long_sqlcmd))) 
      {
        free(long_sqlcmd);
        return -1; 
      }
      else
      {
        free(long_sqlcmd);
      }


      //Update Icon
      user_name= myx_convert_utf8_dbstr(mysql, current_user->user_name);

      long_sqlcmd= malloc(sizeof(char)*(unsigned long)(strlen(user->user_name)+user->icon_length*2+200));


      sqlcmd_end= strmov(long_sqlcmd, "UPDATE mysql." USER_INFO_TABLENAME " SET Icon='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->icon, user->icon_length);
      sqlcmd_end = strmov(sqlcmd_end, "' WHERE User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user_name, (unsigned long)strlen(user_name));
      sqlcmd_end = strmov(sqlcmd_end, "'");

      g_free(user_name);

      //Use mysql_real_query and NOT myx_mysql_real_query therefore
      if (mysql_real_query(mysql, long_sqlcmd, (unsigned int)(sqlcmd_end-long_sqlcmd))) 
      {
        free(long_sqlcmd);
        return -1; 
      }
      else
      {
        free(long_sqlcmd);
      }
    }

    if (strcmp(user->password, "________"))
    {
      //Update password in mysql.user table
      sqlcmd_end= strmov(sqlcmd, "UPDATE mysql.user SET Password=Password('");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->password, (unsigned long)strlen(user->password));
      sqlcmd_end = strmov(sqlcmd_end, "') WHERE User='");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "'");

      if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
        return -1; }
    }
  }
  //Create new user
  else
  {
    //Insert into mysql.user table,
    //one insert for each host
    for(i=0;i<user->hosts_num;i++)
    {
      sqlcmd_end= strmov(sqlcmd, "INSERT INTO mysql.user(User, Host, Password) VALUES('");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "', '");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->hosts[i], (unsigned long)strlen(user->hosts[i]));
      sqlcmd_end = strmov(sqlcmd_end, "', Password('");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->password, (unsigned long)strlen(user->password));
      sqlcmd_end = strmov(sqlcmd_end, "'))");

      if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
        return -1; }
    }

    //Insert into mysql.user_info table
    if ((user->full_name[0])||
      (user->description[0])||
      (user->email[0])||
      (user->contact_information[0])||
      (user->icon_length>0))
    {
      long_sqlcmd= malloc(sizeof(char)*(unsigned long)(strlen(user->user_name)+strlen(user->full_name)+strlen(user->description)+
          strlen(user->email)+strlen(user->contact_information)+user->icon_length*2+200));
      sqlcmd_end= strmov(long_sqlcmd, "INSERT INTO mysql." USER_INFO_TABLENAME "(User, Full_name, Description, "\
        "Email, Contact_information, Icon) VALUES('");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
      sqlcmd_end = strmov(sqlcmd_end, "', '");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->full_name, (unsigned long)strlen(user->full_name));
      sqlcmd_end = strmov(sqlcmd_end, "', '");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->description, (unsigned long)strlen(user->description));
      sqlcmd_end = strmov(sqlcmd_end, "', '");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->email, (unsigned long)strlen(user->email));
      sqlcmd_end = strmov(sqlcmd_end, "', '");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->contact_information, (unsigned long)strlen(user->contact_information));
      sqlcmd_end = strmov(sqlcmd_end, "', '");
      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->icon, user->icon_length);
      sqlcmd_end = strmov(sqlcmd_end, "')");
      
      if (myx_mysql_real_query(mysql, long_sqlcmd, (unsigned int)(sqlcmd_end-long_sqlcmd))) 
      {
        free(long_sqlcmd);
        return -1; 
      }
      else
      {
        free(long_sqlcmd);
      }
    }
  }

  //Username/host and user_info have been updated or create
  //now update/insert privileges
  if (user->user_object_privileges_num>0)
  {
    //Delete previous settings from mysql.db table
    sqlcmd_end= strmov(sqlcmd, "DELETE FROM mysql.db WHERE User='");
    sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
    sqlcmd_end= strmov(sqlcmd_end, "'");

    if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) 
    {
      return -1; 
    }

    //Delete previous settings from mysql.tables_priv table
    sqlcmd_end= strmov(sqlcmd, "DELETE FROM mysql.tables_priv WHERE User='");
    sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
    sqlcmd_end= strmov(sqlcmd_end, "'");

    if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) 
    {
      return -1; 
    }

    //Delete previous settings from mysql.columns_priv table
    sqlcmd_end= strmov(sqlcmd, "DELETE FROM mysql.columns_priv WHERE User='");
    sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
    sqlcmd_end= strmov(sqlcmd_end, "'");

    if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) 
    {
      return -1; 
    }

    long_sqlcmd= malloc(sizeof(char)*700);

    for(i=0;i<user->user_object_privileges_num;i++)
    {
      MYX_USER_OBJECT_PRIVILEGES *privs= user->user_object_privileges+i;
      
      if (privs->object_name)
        dot_count= sub_str_count(".", privs->object_name);
      else
        dot_count= -1;

      //Global privileges
      if (dot_count < 0 || !privs->object_name[0])
      {
        sqlcmd_end= strmov(long_sqlcmd, "UPDATE mysql.user SET ");
        for(j=0;j<privs->user_privileges_num;j++)
        {
          sqlcmd_end= strmov(sqlcmd_end, name_of_str(field, privs->user_privileges[j]));
          sqlcmd_end= strmov(sqlcmd_end, "='");
          sqlcmd_end= strmov(sqlcmd_end, value_of_str(value, privs->user_privileges[j]));
          if (j<privs->user_privileges_num-1)
          {
            sqlcmd_end= strmov(sqlcmd_end, "', ");
          }
          else
          {
            sqlcmd_end= strmov(sqlcmd_end, "' ");
          }
        }
        sqlcmd_end= strmov(sqlcmd_end, "WHERE User='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
        sqlcmd_end= strmov(sqlcmd_end, "' AND Host='");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, privs->host, 
          (unsigned long)strlen(privs->host));
        sqlcmd_end= strmov(sqlcmd_end, "'");

        if (myx_mysql_real_query(mysql, long_sqlcmd, (unsigned int)(sqlcmd_end-long_sqlcmd))) 
        {
          free(long_sqlcmd);
          return -1; 
        }
      }
      //Schema Privileges
      else if (dot_count==0)
      {
        //Insert new settings
        sqlcmd_end= strmov(long_sqlcmd, "INSERT INTO mysql.db(User, Host, Db, ");
        for(j=0;j<privs->user_privileges_num;j++)
        {
          sqlcmd_end= strmov(sqlcmd_end, name_of_str(field, privs->user_privileges[j]));
          if (j<privs->user_privileges_num-1)
          {
            sqlcmd_end= strmov(sqlcmd_end, ", ");
          }
        }
        sqlcmd_end= strmov(sqlcmd_end, ") VALUES('");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
        sqlcmd_end= strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, privs->host, 
          (unsigned long)strlen(privs->host));
        sqlcmd_end= strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, privs->object_name, 
          (unsigned long)strlen(privs->object_name));
        sqlcmd_end= strmov(sqlcmd_end, "', '");

        for(j=0;j<privs->user_privileges_num;j++)
        {
          sqlcmd_end= strmov(sqlcmd_end, value_of_str(value, privs->user_privileges[j]));
          if (j<privs->user_privileges_num-1)
          {
            sqlcmd_end= strmov(sqlcmd_end, "', '");
          }
          else
          {
            sqlcmd_end= strmov(sqlcmd_end, "')");
          }
        }

        if (myx_mysql_real_query(mysql, long_sqlcmd, (unsigned int)(sqlcmd_end-long_sqlcmd))) 
        {
          free(long_sqlcmd);
          return -1; 
        }
      }
      //Schema Table privileges
      else
      {
        s= malloc(sizeof(char)*(64+64+64+2));
        s= strcpy(s, privs->object_name);
        schema_name= strtok(s, ".");
        table_name= strtok(NULL, ".");
        if (dot_count==1)
        {
          sqlcmd_end= strmov(long_sqlcmd, "INSERT INTO mysql.tables_priv(Host, Db, User, Table_name, Timestamp, Grantor, "\
            "Table_priv, Column_priv) VALUES('");
        }
        else
        {
          column_name= strtok(NULL, ".");

          sqlcmd_end= strmov(long_sqlcmd, "INSERT INTO mysql.columns_priv(Host, Db, User, Table_name, Timestamp, Column_name, "\
            "Column_priv) VALUES('");
        }

        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, privs->host, 
          (unsigned long)strlen(privs->host));
        sqlcmd_end= strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, schema_name, (unsigned long)strlen(schema_name));
        sqlcmd_end= strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->user_name, (unsigned long)strlen(user->user_name));
        sqlcmd_end= strmov(sqlcmd_end, "', '");
        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, table_name, (unsigned long)strlen(table_name));
        sqlcmd_end= strmov(sqlcmd_end, "', now(), '");

        if (dot_count==1)
        {
          //No grantor at this time
          sqlcmd_end= strmov(sqlcmd_end, "', '");

          //Get selected Table_privs
          k= 0;
          for(j=0;j<privs->user_privileges_num;j++)
          {
            if((strstr(privs->user_privileges[j], "Table_priv"))&&
              (!(strcmp(
                value_of_str(value, privs->user_privileges[j]), 
                "Y"))))
            {
              sqlcmd_end= strmov(sqlcmd_end, strstr( 
                name_of_str(field, privs->user_privileges[j]),
                "_priv_")+6);
              sqlcmd_end= strmov(sqlcmd_end, ",");
              k++;
            }
          }
          //Remove last ,
          if(k>0)
          {
            sqlcmd_end--;
          }
        }
        else
        {
          sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, column_name, (unsigned long)strlen(column_name));
        }

        sqlcmd_end= strmov(sqlcmd_end, "', '");

        //Get selected Column_privs
        k= 0;
        for(j=0;j<privs->user_privileges_num;j++)
        {
          if((strstr(privs->user_privileges[j], "Column_priv"))&&
            (!(strcmp(
              value_of_str(value, privs->user_privileges[j]), 
              "Y"))))
          {
            sqlcmd_end= strmov(sqlcmd_end, strstr( 
              name_of_str(field, privs->user_privileges[j]),
              "_priv_")+6);
            sqlcmd_end= strmov(sqlcmd_end, ",");
            k++;
          }
        }
        //Remove last ,
        if(k>0)
        {
          sqlcmd_end--;
        }

        sqlcmd_end= strmov(sqlcmd_end, "')");

        free(s);

        if (myx_mysql_real_query(mysql, long_sqlcmd, (unsigned int)(sqlcmd_end-long_sqlcmd))) 
        {
          free(long_sqlcmd);
          return -1; 
        }
      }
    }

    free(long_sqlcmd);
  }

  //Flush Privileges

  if (myx_mysql_query(mysql, "FLUSH PRIVILEGES")) {
    return -1; }

  return 0;
}

MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql, const char *object_name)
{
  MYSQL_RES* res;
  MYSQL_FIELD *fields;
  MYSQL_ROW row;
  unsigned int num_fields;

  MYX_USER_OBJECT_PRIVILEGES *user_priv = malloc(sizeof(MYX_USER_OBJECT_PRIVILEGES));
  unsigned int point_count;
  char sqlcmd[255]= "";
  unsigned char s[255];
  unsigned int j;
  char *token;
  char seps[]   = "(',)";

  user_priv->host= NULL;
  user_priv->object_name= malloc(sizeof(char)*strlen(object_name)+1);
  strcpy(user_priv->object_name, object_name);

  user_priv->user_privileges_num= 0;
  user_priv->user_privileges= NULL;

  point_count= sub_str_count(".", object_name);

  //Global Privileges
  if (!object_name[0])
  {
    strcpy(sqlcmd, "SELECT * "\
        "FROM mysql.user "\
        "WHERE 1=2");
  }
  //Schema
  else if (point_count==0)
  {
    strcpy(sqlcmd, "SELECT * "\
        "FROM mysql.db "\
        "WHERE 1=2");
  }
  //Schema.Table
  else if (point_count==1)
  {
    strcpy(sqlcmd, "SHOW COLUMNS FROM mysql.tables_priv");
  }
  //Schema.Table.Column
  else
  {
    strcpy(sqlcmd, "SHOW COLUMNS FROM mysql.columns_priv");
  }

  if (!point_count)
  {
    if (!(myx_mysql_query(mysql, sqlcmd)))
    {
      if ( (res= mysql_store_result(mysql)) )
      {
        //Depending on the version of the server there might be different columns
        num_fields = mysql_num_fields(res);
        fields = mysql_fetch_fields(res);

        //Allocate maximum fields
        user_priv->user_privileges= malloc(sizeof(char **)*num_fields-2);

        for(j=0;j<num_fields;j++)
        {
          if ((strcmp(str_right(s, fields[j].name, 5), "_priv")==0)||\
              (strcmp(str_left(s, fields[j].name, 4), "max_")==0))
          {
            user_priv->user_privileges_num++;
            user_priv->user_privileges[user_priv->user_privileges_num-1]= malloc(sizeof(char)*(strlen(fields[j].name)+2+1));
            if (strcmp(str_right(s, fields[j].name, 5), "_priv")==0)
            {
              sprintf(user_priv->user_privileges[user_priv->user_privileges_num-1], "%s=%s", fields[j].name, "N");
            }
            else
            {
              sprintf(user_priv->user_privileges[user_priv->user_privileges_num-1], "%s=%s", fields[j].name, "0");
            }
          }
        }
      }
      else
      {
        free(user_priv);
        return NULL;
      }

      mysql_free_result(res);
    }
    else
    {
      free(user_priv);
      return NULL;
    }
  }  
  else if ((point_count==1)||(point_count==2))
  {
    if (!(myx_mysql_query(mysql, sqlcmd)))
    {
      if ( (res= mysql_store_result(mysql)) )
      {
        //Attention!! This has to be increased if there are more different table/column _privs
        user_priv->user_privileges= malloc(sizeof(char **)*30);

        while ((row= mysql_fetch_row(res)))
        {
          if (strcmp(str_right(s, row[0], 5), "_priv")==0)
          {
            strcpy(s, row[1]+3);

            token = strtok(s, seps);
            while(token)
            {
              user_priv->user_privileges_num++;
              user_priv->user_privileges[user_priv->user_privileges_num-1]= malloc(sizeof(char)*(strlen(row[0])+strlen(token)+3+1));
              sprintf(user_priv->user_privileges[user_priv->user_privileges_num-1], "%s_%s=%s", row[0], token, "N");

              token = strtok(NULL, seps);              
            }
          }
        }
      }
      else
      {
        free(user_priv);
        return NULL;
      }

      mysql_free_result(res);
    }
    else
    {
      free(user_priv);
      return NULL;
    }
  }


  return user_priv;
}

int myx_del_user(MYSQL *mysql, const char *user_name)
{
  unsigned char sqlcmd[500], *sqlcmd_end;
  unsigned char esc_user_name[17];

  mysql_real_escape_string(mysql, esc_user_name, user_name, (unsigned long)strlen(user_name));

  //Delete user in mysql.columns_priv
  sqlcmd_end= strmov(sqlcmd, "DELETE FROM mysql.columns_priv WHERE User='");
  sqlcmd_end= strmov(sqlcmd_end, esc_user_name);
  sqlcmd_end= strmov(sqlcmd_end, "'");

  if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
    return -1; }

  //Delete user in mysql.tables_priv
  sqlcmd_end= strmov(sqlcmd, "DELETE FROM mysql.tables_priv WHERE User='");
  sqlcmd_end= strmov(sqlcmd_end, esc_user_name);
  sqlcmd_end= strmov(sqlcmd_end, "'");

  if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
    return -1; }

  //Delete user in mysql.db
  sqlcmd_end= strmov(sqlcmd, "DELETE FROM mysql.db WHERE User='");
  sqlcmd_end= strmov(sqlcmd_end, esc_user_name);
  sqlcmd_end= strmov(sqlcmd_end, "'");

  if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
    return -1; }

  //Delete user in mysql.user_info
  sqlcmd_end= strmov(sqlcmd, "DELETE FROM mysql." USER_INFO_TABLENAME " WHERE User='");
  sqlcmd_end= strmov(sqlcmd_end, esc_user_name);
  sqlcmd_end= strmov(sqlcmd_end, "'");

  if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
    return -1; }

  //Delete user in mysql.user
  sqlcmd_end= strmov(sqlcmd, "DELETE FROM mysql.user WHERE User='");
  sqlcmd_end= strmov(sqlcmd_end, esc_user_name);
  sqlcmd_end= strmov(sqlcmd_end, "'");

  if (myx_mysql_real_query(mysql, sqlcmd, (unsigned int)(sqlcmd_end-sqlcmd))) {
    return -1; }

  return 0;
}
