/* Copyright (C) 2003, 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#include "myadmin.h"

#include "MAdministrator.h"

#include <string>
#include <MGConnectDialog.h>

#include <errno.h>

#include <mysql.h>



// global
MGPreferences prefs;


// cmd line options
static bool args_do_not_connect= false;
static MYX_USER_CONNECTION *args_user_conn= NULL;


static char *get_arg(char **argv, int index)
{
  char *arg= argv[index];
  char *ptr= strchr(arg,'=');
  if (ptr)
  {
    if (*(ptr+1)) /* --foo=bla */
      return ptr+1;
    else
      return NULL; /* --foo= */
  }
  else
  {
    if (strncmp(arg,"--",2)==0) /* --foo bla */
      return argv[index+1];
    else
    {
      if (strlen(arg)==2) /* -f bla */
        return argv[index+1];
      else
        return argv[index]+2; /* -fbla */
    }
  }
}


bool parse_cmd_line_args(int argc, char **argv)
{
  char *value;
  int i;

#define CHECK_OPTION(s,v) (Glib::str_has_prefix(s,v"+") || strcmp(s,v)==0)
  
  for (i= 1; i < argc; i++)
  {
    if (strncmp(argv[i], "-u", 2)==0 || CHECK_OPTION(argv[i],"--user"))
    {
      value= get_arg(argv, i);
      if (!value)
        goto missing_arg;
      if (!args_user_conn)
        args_user_conn= (MYX_USER_CONNECTION*)g_malloc0(sizeof(MYX_USER_CONNECTION));
      args_user_conn->username= g_strdup(value);
    }
    else if (strncmp(argv[i], "-p", 2)==0 || CHECK_OPTION(argv[i],"--password"))
    {
      value= get_arg(argv, i);
      if (!value)
        goto missing_arg;
      if (!args_user_conn)
        args_user_conn= (MYX_USER_CONNECTION*)g_malloc0(sizeof(MYX_USER_CONNECTION));
      args_user_conn->password= g_strdup(value);
    }
    else if (strncmp(argv[i], "-h", 2)==0 || CHECK_OPTION(argv[i],"--host"))
    {
      value= get_arg(argv, i);
      if (!value)
        goto missing_arg;
      if (!args_user_conn)
        args_user_conn= (MYX_USER_CONNECTION*)g_malloc0(sizeof(MYX_USER_CONNECTION));
      args_user_conn->hostname= g_strdup(value);
    }
    else if (strncmp(argv[i], "-P", 2)==0 || CHECK_OPTION(argv[i],"--port"))
    {
      int port;
      value= get_arg(argv, i);
      if (!value)
        goto missing_arg;
      if (sscanf(value, "%i", &port)!=1 || port < 1 || port > (1<<16))
      {
        g_print(_("Bad port number value '%s' for option\n"), value);
        return false;
      }
      if (!args_user_conn)
        args_user_conn= (MYX_USER_CONNECTION*)g_malloc0(sizeof(MYX_USER_CONNECTION));
      args_user_conn->port= port;
    }
    else if (strncmp(argv[i], "-S", 2)==0 || CHECK_OPTION(argv[i],"--socket"))
    {
      value= get_arg(argv, i);
      if (!value)
        goto missing_arg;
      if (!args_user_conn)
        args_user_conn= (MYX_USER_CONNECTION*)g_malloc0(sizeof(MYX_USER_CONNECTION));
      args_user_conn->advanced_options_num= 1;
      args_user_conn->advanced_options= (char**)g_malloc0(sizeof(char*));
      args_user_conn->advanced_options[0]= g_strdup_printf("SOCKET_PATH=%s", value);
    }
    else if (Glib::str_has_prefix(argv[i], "--mycnf="))
    {
      value= get_arg(argv, i);
      if (!value)
        goto missing_arg;
      myx_set_my_cnf_path(value);
    }
    else if (strcmp(argv[i], "--dont-connect")==0)
    {
      args_do_not_connect= true;
    }
    else if (strcmp(argv[i], "--help")==0 || strcmp(argv[i], "-h")==0)
    {
      g_print("MySQL Administrator Ver %s\n", VERSION);
      g_print("Copyright (c) 2004 MySQL AB\n");
      g_print(_("Usage: %s [OPTIONS]\n"), g_get_prgname());
      g_print(_("  -h,--help             Display this help and exit.\n"));
      g_print(_("  -u,--user=name        User for login if not current user.\n"));
      g_print(_("  -p,--password=s       Password to use when connecting to server.\n"));
      g_print(_("  -h,--host=name        Connect to host.\n"));
      g_print(_("  -P,--port=#           Port number to use for connection.\n"));
      g_print(_("  -S,--socket=name      Socket file to use for connection.\n"));
      g_print(_("  --dont-connect        Start without connecting to a MySQL server.\n"));
      g_print(_("  --mycnf=path          Path for my.cnf file.\n"));
      exit(0);
    }
    else
    {
      g_print(_("%s: unknown option %s\n"), g_get_prgname(), argv[i]);
      return false;
    }
  }
  
  
  if (args_user_conn)
  {
    // set some defaults
    if (!args_user_conn->hostname)
      args_user_conn->hostname= g_strdup("localhost");
    if (!args_user_conn->port)
      args_user_conn->port= 3306;
  }
  
  
  return true;

missing_arg:
  g_print(_("%s: missing value for option %s\n"), g_get_prgname(), argv[i]);
  return false;
}


#include "myx_admin_public_interface.h"
int main(int argc, char **argv)
{
  Gtk::Main app(argc, argv, true);
  Glib::ustring open_panel= getenv("DEFAULT_PANEL")?:"";

#ifdef ENABLE_NLS
  bindtextdomain(GETTEXT_PACKAGE, APP_LOCALE_DIR);
  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
  textdomain(GETTEXT_PACKAGE);
#endif

  // hack for binary release
  // since the statically linked client lib checks for my.cnf in /etc, 
  // it wouldnt work in distros with it in /etc/mysql/
#ifdef MY_CNF_PATH_TRY_GUESS
  if (Glib::file_test(DEFAULT_MY_CNF_PATH, Glib::FILE_TEST_EXISTS))
    myx_set_my_cnf_path(DEFAULT_MY_CNF_PATH);
  else if (Glib::file_test("/etc/my.cnf", Glib::FILE_TEST_EXISTS))
    myx_set_my_cnf_path("/etc/my.cnf");
  else if (Glib::file_test("/etc/mysql/my.cnf", Glib::FILE_TEST_EXISTS))
    myx_set_my_cnf_path("/etc/mysql/my.cnf");
#else
  if (Glib::file_test(DEFAULT_MY_CNF_PATH, Glib::FILE_TEST_EXISTS))
    myx_set_my_cnf_path(DEFAULT_MY_CNF_PATH);
#endif
  else
  {
    g_print(_("WARNING: Could not locate my.cnf file. Use the --mycnf=<path> option to specify its location.\n"));
  }

  if (!parse_cmd_line_args(argc, argv))
    return 1;

  // load preferences
  prefs.load("mysqlx_options.xml");

  // check app specific rc dir
  prefs.check_directory(prefs.build_path_to("administrator"), true);

  {
    MAdministrator madmin;
    MYSQL *mysql= NULL;
    MYX::UserConnection user_connection;
    
    if (!args_do_not_connect)
    {
      MGConnectDialog connect_dlg(&prefs);

      if (args_user_conn)
      {
        if (connect_dlg.run_with_defaults("MySQL Administrator",
                                          args_user_conn, 0))
        {
          mysql= connect_dlg.get_connection();
          user_connection= connect_dlg.get_user_connection();
        }
        else
        {
          // if we tried to connect using cmd line defaults, exit
          return 1;
        }
      }
      else
      {
        if (connect_dlg.run("MySQL Administrator", 0))
        {
          mysql= connect_dlg.get_connection();
          user_connection= connect_dlg.get_user_connection();
        }
      }
    }
    
    MInstanceInfo instance(&madmin, mysql, user_connection);

    if (!madmin.init(&instance))
    {
      g_error(_("Error initializing GUI"));
      return 1;
    }

    madmin.show();

    if (!open_panel.empty())
        madmin.switch_panel(open_panel);

    app.run();
  }

  return 0;
}
