unit AdminCatalogTableCheck;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Progress, InstanceDataModule,
  myx_public_interface, myx_admin_public_interface, ExtCtrls, ComCtrls,
  ApplicationDataModule, AuxFuncs;

type
  TableAction = (taOptimize=0, taCheck=1, taRepair=2);

  TAdminCatalogTableCheckForm = class(TForm)
    PageControl: TPageControl;
    OptionsTabSheet: TTabSheet;
    GroupBox1: TGroupBox;
    QuickRBtn: TRadioButton;
    FastRBtn: TRadioButton;
    MediumRBtn: TRadioButton;
    ExtendedRBtn: TRadioButton;
    ChangedRBtn: TRadioButton;
    ResultTabSheet: TTabSheet;
    PageControlHidePnl: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ResultMemo: TMemo;
    Label3: TLabel;
    HeaderLbl: TLabel;
    Label4: TLabel;
    OptimizeTabSheet: TTabSheet;
    Label5: TLabel;
    Label6: TLabel;
    GroupBox2: TGroupBox;
    OptLocalCBox: TCheckBox;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    RepairTabSheet: TTabSheet;
    Label12: TLabel;
    Label13: TLabel;
    GroupBox3: TGroupBox;
    GroupBox4: TGroupBox;
    Label14: TLabel;
    Label17: TLabel;
    RepExtendedCBox: TCheckBox;
    RepQuickCBox: TCheckBox;
    RepUseFRMCBox: TCheckBox;
    Label15: TLabel;
    Label16: TLabel;
    RepLocalCBox: TCheckBox;
    Label18: TLabel;
    RepairWarning: TLabel;
    Panel1: TPanel;
    ExecuteTableActionBtn: TButton;
    CloseBtn: TButton;
    Label19: TLabel;
    Label20: TLabel;

    constructor Create(AOwner: TComponent; Tables: string;
      InstanceDM: TInstanceDM;
      Action: TableAction); reintroduce;

    procedure ExecuteTableActionBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure ExecuteTableAction(Sender: TObject);
    procedure FinishedTableAction(Sender: TObject);
    procedure UpdateProgressForm(Sender: TObject);
  private
    { Private declarations }
    TableList: TStringList;
    InstanceDM: TInstanceDM;

    ProgressForm: TProgressForm;
  public
    { Public declarations }
    CheckType: integer;
    Messages: string;
    CurrentPos: integer;
    CurrentTable: string;
    Action: TableAction;
  end;

var
  AdminCatalogTableCheckForm: TAdminCatalogTableCheckForm;

implementation

uses AdminCatalog;

{$R *.dfm}

constructor TAdminCatalogTableCheckForm.Create(AOwner: TComponent;
  Tables: string;
  InstanceDM: TInstanceDM;
  Action: TableAction);
begin
  inherited Create(AOwner);

  TableList:=TStringList.Create;
  TableList.Text:=Tables;

  self.InstanceDM:=InstanceDM;
  self.Action:=Action;
end;

procedure TAdminCatalogTableCheckForm.FormCreate(Sender: TObject);
begin
  PageControl.ActivePageIndex:=Ord(Action);
  PageControlHidePnl.BringToFront;

  case Action of
    taOptimize:
    begin
      Caption:='Optimize Tables';
      ExecuteTableActionBtn.Caption:='Optimize Tables';

      Height:=_s(204);
    end;
    taCheck:
    begin
      Caption:='Check Tables';
      ExecuteTableActionBtn.Caption:='Check Tables';

      Height:=_s(360);
    end;
    taRepair:
    begin
      Caption:='Repair Tables';
      ExecuteTableActionBtn.Caption:='Repair Tables';

      Height:=_s(515);
    end;
  end;
end;

procedure TAdminCatalogTableCheckForm.FormDestroy(Sender: TObject);
begin
  TableList.Free;

  TAdminCatalogForm(Owner).AdminCatalogTableCheckForm:=nil;
end;

procedure TAdminCatalogTableCheckForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action:=caFree;
end;

procedure TAdminCatalogTableCheckForm.ExecuteTableAction(Sender: TObject);
var i, j: integer;
  PMySQL: Pointer;
  PStatuses: PMYX_TABLE_COMMAND_STATUSES;
  Statuses: TMYX_TABLE_COMMAND_STATUSES;
  ErrorString, SuccessString: string;
begin
  with TAdminCatalogTableCheckForm(TFetchDataThread(Sender).Target) do
  begin
    case Action of
      taOptimize:
      begin
        ErrorString:='Error - Table %s could not be optimized.';
        SuccessString:='Table %s optimized.';
      end;
      taCheck:
      begin
        ErrorString:='Error - Table %s could not be checked.';
        SuccessString:='Status checked for table %s.';
      end;
    else
      ErrorString:='Error - Table %s could not be repaired.';
      SuccessString:='Table %s repaired.';
    end;

    PMySQL:=myx_mysql_init;
    try
      myx_connect_to_instance(InstanceDM.user_connection.get_record_pointer, PMySQL);

      for i:=0 to TableList.Count-1 do
      begin
        CurrentPos:=i;
        CurrentTable:=TableList[i];

        TFetchDataThread(Sender).ExecuteSynchronized(
          UpdateProgressForm);

        case Action of
          taOptimize:
            PStatuses:=myx_optimize_table(PMySQL,
              PChar(AnsiToUtf8(TableList[i])), CheckType);
          taCheck:
            PStatuses:=myx_check_table(PMySQL,
              PChar(AnsiToUtf8(TableList[i])), CheckType);
        else
          PStatuses:=myx_repair_table(PMySQL,
            PChar(AnsiToUtf8(TableList[i])), CheckType);
        end;


        if(PStatuses=nil)then
        begin
          Messages:=Messages+StringOfChar('-', 120)+#13#10;

          Messages:=Messages+
            Format(ErrorString, [TableList[i]])+#13#10+
            'MySQL Error Nr.'+IntToStr(myx_mysql_errno(PMySQL))+#13#10+
            myx_mysql_error(PMySQL)+#13#10;
        end
        else
        begin
          try
            Statuses:=TMYX_TABLE_COMMAND_STATUSES.Create(PStatuses);
            try
              for j:=0 to Statuses.status.Count-1 do
              begin
                Messages:=Messages+StringOfChar('-', 120)+#13#10;
                Messages:=Messages+Format(SuccessString, [Statuses.status[j].table])+#13#10;
                Messages:=Messages+Statuses.status[j].message+#13#10;
              end;
            finally
              Statuses.Free;
            end;
          finally
            myx_free_command_status(PStatuses);
          end;
        end;

        if(ProgressForm.Stopping)then
          break;
      end;

      ProgressForm.ProgressBar.Position:=TableList.Count;
      ProgressForm.Refresh;
    finally
      myx_mysql_close(PMySQL);
    end;
  end;
end;

procedure TAdminCatalogTableCheckForm.FinishedTableAction(Sender: TObject);
begin
  with TAdminCatalogTableCheckForm(TFetchDataThread(Sender).Target) do
  begin
    ProgressForm.Free;

    ResultMemo.Text:=Messages;

    PageControl.ActivePage:=ResultTabSheet;
    ExecuteTableActionBtn.Hide;
    
    Height:=_s(463);
    Show;
  end;
end;

procedure TAdminCatalogTableCheckForm.UpdateProgressForm(Sender: TObject);
begin
  ProgressForm.ActionLbl.Caption:=CurrentTable;
  ProgressForm.ProgressBar.Position:=CurrentPos;
  ProgressForm.Refresh;
end;


procedure TAdminCatalogTableCheckForm.ExecuteTableActionBtnClick(Sender: TObject);
begin
  Hide;

  CheckType:=0;
  if(Action=taOptimize)then
  begin
    if(OptLocalCBox.Checked)then
      CheckType:=Ord(MYX_REPAIR_NO_WRITE_TO_BINLOG);
  end
  else if(Action=taCheck)then
  begin
    if(QuickRBtn.Checked)then
      CheckType:=Ord(MYX_CHECK_QUICK)
    else if(FastRBtn.Checked)then
      CheckType:=Ord(MYX_CHECK_FAST)
    else if(MediumRBtn.Checked)then
      CheckType:=Ord(MYX_CHECK_MEDIUM)
    else if(ExtendedRBtn.Checked)then
      CheckType:=Ord(MYX_CHECK_EXTENDED)
    else if(ChangedRBtn.Checked)then
      CheckType:=Ord(MYX_CHECK_CHANGED)
    else
      CheckType:=Ord(MYX_CHECK_MEDIUM);
  end
  else if(Action=taCheck)then
  begin
    if(RepLocalCBox.Checked)then
      CheckType:=CheckType+Ord(MYX_REPAIR_NO_WRITE_TO_BINLOG);

    if(RepQuickCBox.Checked)then
      CheckType:=CheckType+Ord(MYX_CHECK_QUICK);

    if(RepExtendedCBox.Checked)then
      CheckType:=CheckType+Ord(MYX_CHECK_EXTENDED);

    if(RepUseFRMCBox.Checked)then
      CheckType:=CheckType+Ord(MYX_REPAIR_USE_FRM);
  end;

  Messages:='';
  CurrentTable:='';
  CurrentPos:=0;

  ProgressForm:=TProgressForm.Create(nil);
  try
    case Action of
      taOptimize:
      begin
        ProgressForm.Caption:='Optimize Table Progress ...';
        ProgressForm.ActionCaptionLbl.Caption:='Optimize Table:';
      end;
      taCheck:
      begin
        ProgressForm.Caption:='Check Table Progress ...';
        ProgressForm.ActionCaptionLbl.Caption:='Check Table:';
      end;
    else
      ProgressForm.Caption:='Repair Table Progress ...';
      ProgressForm.ActionCaptionLbl.Caption:='Repair Table:';
    end;

    ProgressForm.ProgressBar.Max:=TableList.Count;
    ProgressForm.ProgressBar.Position:=0;

    ProgressForm.Show;

    InstanceDM.FetchData(KindOfData_TableAction,
      ExecuteTableAction, FinishedTableAction,
      self, 'Checking tables ...', True);
  except
    ProgressForm.Free;
  end;
end;

procedure TAdminCatalogTableCheckForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

end.
