unit AdminRestore;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls,
  InstanceSections, AdditionalClasses,
  ImgList, Buttons, Contnrs, Menus, ApplicationDataModule, AuxFuncs,
  myx_public_interface, myx_admin_public_interface, SchemataTreeView,
  PNGImage, AuxAdminBackupRestore, InstanceDataModule, MyxError;

type
  TAdminRestoreForm = class(TInstanceSectionForm)
    ServerBackupRestorePnl: TPanel;
    BackupRestorePageControl: TPageControl;
    ContentSheet: TTabSheet;
    ItemSelectImageList: TImageList;
    Panel1: TPanel;
    RestoreBevel: TBevel;
    Label3: TLabel;
    RestoreImg: TImage;
    RestoreTreeView: TTreeView;
    RestoreTreeViewHeaderControl: THeaderControl;
    BackupTreeViewPopupMenu: TPopupMenu;
    RemoveBackupNodeMI: TMenuItem;
    SubTreeSearchPopupMenu: TPopupMenu;
    AllcatalogsMI: TMenuItem;
    system1: TMenuItem;
    dev1: TMenuItem;
    world1: TMenuItem;
    GeneralSheet: TTabSheet;
    Panel6: TPanel;
    BackupBevel: TBevel;
    Label6: TLabel;
    Restore2Img: TImage;
    Label7: TLabel;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    FileToRestoreEd: TEdit;
    BackupFileTypesLU: TComboBox;
    TargetSchemaCBox: TComboBox;
    Panel7: TPanel;
    Panel8: TPanel;
    StartRestoreBtn: TButton;
    OpenBackupFileBtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure RefreshSchemaTree(searchStr: String = '');
    procedure ServerBackupRestorePnlResize(Sender: TObject);
    procedure BackupTreeViewAdvancedCustomDrawItem(Sender: TCustomTreeView;
      Node: TTreeNode; State: TCustomDrawState; Stage: TCustomDrawStage;
      var PaintImages, DefaultDraw: Boolean);
    procedure BackupTreeViewMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure SetChildrenSelectState(Node: TTreeNode);
    procedure CheckParentSelectState(Node: TTreeNode);
    function AllChildNodeSelected(Node: TTreeNode): TBackupCheckType;
    procedure RestoreTreeViewHeaderControlSectionResize(
      HeaderControl: THeaderControl; Section: THeaderSection);

    procedure SetNodeSelectState(Node: TTreeNode; Selected: TBackupCheckType);
    procedure SchemaTreeViewExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);

    procedure RemoveTreeNode(Node: TTreeNode);
    procedure SchemaTreeViewCollapsing(Sender: TObject; Node: TTreeNode;
      var AllowCollapse: Boolean);
    procedure SubTreeSeachEdMagnifyGlassImgClick(Sender: TObject);

    function BackupProgress(current_table_name: PChar;
      num_tables: Integer; num_tables_processed: Integer;
      num_rows: Integer; num_rows_processed: Integer): integer;
    procedure OpenBackupFileBtnClick(Sender: TObject);
    procedure BackupRestorePageControlChange(Sender: TObject);
    procedure GetBackupFileContent;
    procedure StartRestoreBtnClick(Sender: TObject);
    procedure ClearControls;

    procedure Disconnected(var Message: TMessage); message WM_Disconnected;
    procedure Reconnected(var Message: TMessage); message WM_Reconnected;
    procedure RestoreTreeViewChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
  private
    { Private declarations }
    //MousePos: TPoint;

    RestorePNGImg: TPNGObject;
  public
    { Public declarations }
    CurrentCatalog: TMYX_CATALOG;
    CurrentSchema: TMYX_SCHEMA;

    //User must not expand SchemaTree manually
    LockSchemaTreeExpansion: Boolean;
  end;

  function GetBackupFileContentProgress(bytes_read: Int64; bytes_total: Int64; user_data: Pointer):Integer cdecl;
  function RestoreBackup(bytes_read: Int64; bytes_total: Int64; user_data: Pointer):Integer cdecl;
  procedure RestoreWarning(msg: PChar; user_data: Pointer) cdecl;

var
  AdminRestoreForm: TAdminRestoreForm;

implementation

{$R *.dfm}

uses Main, AdminBackup;

procedure TAdminRestoreForm.FormCreate(Sender: TObject);
begin
  DockedPanel:=ServerBackupRestorePnl;

  BackupRestorePageControl.ActivePageIndex:=0;

  CurrentCatalog:=nil;
  CurrentSchema:=nil;

  LockSchemaTreeExpansion:=True;

  RestorePNGImg:=LoadPNGImageFromResource('restore', RestoreImg);
  Restore2Img.Picture.Assign(RestorePNGImg);

  DisableEnableControls(GeneralSheet, False);
  DisableEnableControls(ContentSheet, False);

  if(Not(InstanceDM.Connected))then
    OpenBackupFileBtn.Enabled:=False;
end;

procedure TAdminRestoreForm.FormDestroy(Sender: TObject);
begin
  RestorePNGImg.Free;
end;

procedure TAdminRestoreForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  //
end;

procedure TAdminRestoreForm.RefreshSchemaTree(searchStr: String = '');
begin
  //
end;

procedure TAdminRestoreForm.ServerBackupRestorePnlResize(
  Sender: TObject);
begin
  RestoreTreeViewHeaderControl.Width:=RestoreTreeView.Width-2-18;
end;

procedure TAdminRestoreForm.BackupTreeViewAdvancedCustomDrawItem(
  Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState;
  Stage: TCustomDrawStage; var PaintImages, DefaultDraw: Boolean);
var NodeRect: TRect;
  XPos: integer;
  theHeaderControl: THeaderControl;
  i: integer;
begin
  if(Stage=cdPrePaint)then
  begin
    Sender.Canvas.Pen.Style:=psSolid;
    if(cdsFocused in State)then
      Sender.Canvas.Pen.Color:=clGray
    else
      Sender.Canvas.Pen.Color:=clSilver;

    NodeRect:=Node.DisplayRect(False);

    if(cdsSelected in State)then
    begin
      Sender.Canvas.Brush.Color:=clSilver;
      Sender.Canvas.Rectangle(NodeRect.Left+19+Node.Level*19+5-2, NodeRect.Top,
        Sender.Width-4, NodeRect.Bottom);
    end
    else
    begin
      Sender.Canvas.Brush.Color:=clWhite;
      Sender.Canvas.FillRect(Rect(NodeRect.Left+19+Node.Level*19+5-2, NodeRect.Top,
        Sender.Width-4, NodeRect.Bottom+1));
    end;

    DefaultDraw:=True;
  end
  else if(Stage=cdPostPaint)then
  begin
    if(Node.Data<>nil)then
    begin
      NodeRect:=Node.DisplayRect(False);

      if(cdsFocused in State)then
      begin
        Sender.Canvas.Pen.Color:=clGray;
        Sender.Canvas.Pen.Style:=psSolid;
        Sender.Canvas.Brush.Style:=bsClear;
        Sender.Canvas.Rectangle(NodeRect.Left+19+Node.Level*19+5-2, NodeRect.Top,
          Sender.Width-4, NodeRect.Bottom);

        Sender.Canvas.Brush.Color:=clSilver;
        Sender.Canvas.Brush.Style:=bsSolid;
        Sender.Canvas.FillRect(Rect(NodeRect.Left+19+Node.Level*19+5+
          Sender.Canvas.TextWidth(Node.Text)+1, NodeRect.Top+1,
          NodeRect.Left+19+Node.Level*19+5+
          Sender.Canvas.TextWidth(Node.Text)+1+5, NodeRect.Bottom-1));
      end;

      ItemSelectImageList.Draw(Sender.Canvas,
        NodeRect.Left+19+Node.Level*19+5, NodeRect.Top+1,
        Ord(TBackupNode(Node.Data).GetSelectState));

      if(TBackupNode(Node.Data).GetObjType=BOTTable)and
        (TBackupNode(Node.Data).Substrings<>nil)then
      begin
        {if(Sender.Name='BackupTreeView')then
          theHeaderControl:=BackupTreeViewHeaderControl
        else}
          theHeaderControl:=RestoreTreeViewHeaderControl;

        XPos:=0;
        for i:=0 to 3 do
        begin
          XPos:=XPos+theHeaderControl.Sections[i].Width;
          Sender.Canvas.TextOut(XPos+2, NodeRect.Top+2,
            TBackupNode(Node.Data).Substrings[i]);
        end;
      end;
    end;
  end;
end;

procedure TAdminRestoreForm.SetNodeSelectState(Node: TTreeNode; Selected: TBackupCheckType);
begin
  TBackupNode(Node.Data).SetSelectState(Selected);

  //Check/Uncheck all childs as well
  SetChildrenSelectState(Node);

  //See, if the current select state of the parent is valid
  CheckParentSelectState(Node);

  Node.TreeView.Invalidate;
end;

procedure TAdminRestoreForm.BackupTreeViewMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Node: TTreeNode;
  NodeRect: TRect;
  Selected: TBackupCheckType;
begin
  Node:=TTreeView(Sender).GetNodeAt(x, y);
  if(Assigned(Node))then
  begin
    NodeRect:=Node.DisplayRect(False);

    if(X>NodeRect.Left+19+Node.Level*19+5)and(X<NodeRect.Left+19+Node.Level*19+5+14)and
      (Node.Data<>nil)then
    begin
      if(TBackupNode(Node.Data).GetSelectState<>BCTAll)then
        Selected:=BCTAll
      else
        Selected:=BCTNone;

      SetNodeSelectState(Node, Selected);
    end;
  end;
end;

procedure TAdminRestoreForm.SetChildrenSelectState(Node: TTreeNode);
var ChildNode: TTreeNode;
begin
  //Set Select state for all childs
  ChildNode:=Node.getFirstChild;
  while(ChildNode<>nil)do
  begin
    if(ChildNode.Data<>nil)then
      TBackupNode(ChildNode.Data).SetSelectState(TBackupNode(Node.Data).GetSelectState);

    //Call recursive if the child has another child
    if(ChildNode.getFirstChild<>nil)then
      SetChildrenSelectState(ChildNode);

    ChildNode:=Node.GetNextChild(ChildNode);
  end;
end;

procedure TAdminRestoreForm.CheckParentSelectState(Node: TTreeNode);
begin
  if(Node.Parent<>nil)then
    if(Node.Parent.Data<>nil)then
    begin
      //Check, if all nodes under the parent are selected. If so, select the parent as well
      TBackupNode(Node.Parent.Data).SetSelectState(AllChildNodeSelected(Node.Parent));

      if(Node.Parent.Parent<>nil)then
        CheckParentSelectState(Node.Parent);
    end;
end;

function TAdminRestoreForm.AllChildNodeSelected(Node: TTreeNode): TBackupCheckType;
var ChildNode: TTreeNode;
  AllSelected, AtLeastOneSelected: Boolean;
  SubChildsSelectState: TBackupCheckType;
begin
  AllSelected:=True;
  AtLeastOneSelected:=False;

  //check if all child nodes are selected
  ChildNode:=Node.getFirstChild;
  while(ChildNode<>nil)do
  begin
    if(ChildNode.Data<>nil)then
    begin
      AllSelected:=AllSelected and
        (TBackupNode(ChildNode.Data).GetSelectState=BCTAll);

      AtLeastOneSelected:=AtLeastOneSelected or
        (TBackupNode(ChildNode.Data).GetSelectState=BCTAll) or
        (TBackupNode(ChildNode.Data).GetSelectState=BCTSome);
    end;

    //Call recursive if the child has another child
    if(ChildNode.getFirstChild<>nil)then
    begin
      SubChildsSelectState:=AllChildNodeSelected(ChildNode);
      AllSelected:=AllSelected and (SubChildsSelectState=BCTAll);
      AtLeastOneSelected:=AtLeastOneSelected or
        (SubChildsSelectState=BCTAll) or
        (SubChildsSelectState=BCTSome);
    end;

    ChildNode:=Node.GetNextChild(ChildNode);
  end;

  if(AllSelected)then
    AllChildNodeSelected:=BCTAll
  else if(AtLeastOneSelected)then
    AllChildNodeSelected:=BCTSome
  else
    AllChildNodeSelected:=BCTNone;
end;

procedure TAdminRestoreForm.RestoreTreeViewHeaderControlSectionResize(
  HeaderControl: THeaderControl; Section: THeaderSection);
begin
  RestoreTreeView.Invalidate;
end;

procedure TAdminRestoreForm.SchemaTreeViewExpanding(Sender: TObject;
  Node: TTreeNode; var AllowExpansion: Boolean);
begin
  //Expansion not allowed in Backup/Restore
  AllowExpansion:=Not(LockSchemaTreeExpansion);
end;

procedure TAdminRestoreForm.RemoveTreeNode(Node: TTreeNode);
begin
  //Remove Child nodes first
  while(Node.Count>0)do
    RemoveTreeNode(Node.Item[0]);

  TBackupNode(Node.Data).Free;

  TTreeView(Node.TreeView).Items.Delete(Node);
end;

procedure TAdminRestoreForm.SchemaTreeViewCollapsing(Sender: TObject;
  Node: TTreeNode; var AllowCollapse: Boolean);
begin
  if(Node.Data<>nil)then
    if(TObject(Node.Data) is TMYX_CATALOG)then
      AllowCollapse:=False;
end;

procedure TAdminRestoreForm.SubTreeSeachEdMagnifyGlassImgClick(
  Sender: TObject);
begin
  SubTreeSearchPopupMenu.Popup(Mouse.CursorPos.X, Mouse.CursorPos.Y);
end;

function Test: integer;
begin
  ShowMessage('Test');

  Result:=0;
end;

function TAdminRestoreForm.BackupProgress(current_table_name: PChar;
  num_tables: Integer; num_tables_processed: Integer;
  num_rows: Integer; num_rows_processed: Integer): integer;
begin
  //Show Progress

  Result:=0;
end;

procedure TAdminRestoreForm.OpenBackupFileBtnClick(Sender: TObject);
var OpenDialog: TOpenDialog;
begin
  BackupRestorePageControl.ActivePage:=GeneralSheet;

  OpenDialog:=TOpenDialog.Create(self);
  try
    OpenDialog.Filter:='SQL Files|*.sql|All Files|*.*';
    
    if(OpenDialog.Execute)then
    begin
      ClearTreeView(RestoreTreeView, myx_ndt_tobject);

      DisableEnablePages(BackupRestorePageControl, True);

      FileToRestoreEd.Text:=OpenDialog.FileName;

      StartRestoreBtn.Enabled:=True;
    end;
  finally
    OpenDialog.Free;
  end;
end;

procedure TAdminRestoreForm.BackupRestorePageControlChange(
  Sender: TObject);
begin
  if(BackupRestorePageControl.ActivePage=ContentSheet)and
    (RestoreTreeView.Items.Count=0)and
    (FileToRestoreEd.Text<>'')then
  begin
    GetBackupFileContent;
  end;
end;

function GetBackupFileContentProgress(bytes_read: Int64; bytes_total: Int64; user_data: Pointer):Integer;
var PSender: ^TAdminRestoreForm;
begin
  PSender:=user_data;

  PSender.SetStatusText(IntToStr(bytes_read)+'/'+
    IntToStr(bytes_total)+' Bytes read.');

  Result:=0;
end;

procedure TAdminRestoreForm.GetBackupFileContent;
var PBackupContent: PMYX_BACKUP_CONTENT;
  BackupContent: TMYX_BACKUP_CONTENT;
  BackupType: MYX_BACKUP_TYPE;
  BackupError: MYX_BACKUP_ERROR;
  ErrorTxt: string;
begin
  //Get Backup FileType
  case BackupFileTypesLU.ItemIndex of
    0:
      BackupType:=MYX_BT_SQL_SCRIPT;
  else
    BackupType:=MYX_BT_SQL_SCRIPT;
  end;

  //Get tables from backup file
  PBackupContent:=myx_get_backup_content(
    PChar(AnsiToUtf8(FileToRestoreEd.Text)), BackupType, 128,
    GetBackupFileContentProgress, Addr(self), @BackupError);

  SetStatusText('');

  if(PBackupContent=nil)or(BackupError<>MYX_BACKUP_NO_ERROR)then
  begin
    ErrorTxt:=GetBackupErrorMsg(BackupError);
    case BackupError of
      MYX_BACKUP_MYSQL_ERROR:
        ErrorTxt:=Format(ErrorTxt, [myx_mysql_errno(InstanceDM.PMySQL),
          myx_mysql_error(InstanceDM.PMySQL)]);
      MYX_BACKUP_CANT_OPEN_FILE:
        ErrorTxt:=Format(ErrorTxt, [FileToRestoreEd.Text]);
    end;

    raise EMyxError.Create('Error while retrieving Backup content. '+#13#10#13#10+
      ErrorTxt);
  end;

  BackupContent:=TMYX_BACKUP_CONTENT.create(PBackupContent);
  try
    BuildContentTreeFromRestoreContent(RestoreTreeView,
      BackupContent);
  finally
    BackupContent.Free;
    myx_free_backup_content(PBackupContent);
  end;
end;

function RestoreBackup(bytes_read: Int64; bytes_total: Int64; user_data: Pointer):Integer;
//var PSender: ^TAdminRestoreForm;
begin
  //PSender:=user_data;

  {PSender.SetStatusText(IntToStr(bytes_read)+'/'+
    IntToStr(bytes_total)+' Bytes read.');}

  Result:=0;
end;

procedure RestoreWarning(msg: PChar; user_data: Pointer);
begin
  //
end;

procedure TAdminRestoreForm.StartRestoreBtnClick(Sender: TObject);
var BackupError: MYX_BACKUP_ERROR;
  BackupContent: TMYX_BACKUP_CONTENT;
  BackupType: MYX_BACKUP_TYPE;
  target_catalog, target_schema: string;
  ErrorTxt: string;
begin
  if(RestoreTreeView.Items.Count=0)then
    GetBackupFileContent;

  target_catalog:='';

  if(TargetSchemaCBox.Text<>'Original Schema')then
    target_schema:=TargetSchemaCBox.Text
  else
    target_schema:='';

  //Get Backup FileType
  case BackupFileTypesLU.ItemIndex of
    0:
      BackupType:=MYX_BT_SQL_SCRIPT;
  else
    BackupType:=MYX_BT_SQL_SCRIPT;
  end;

  BackupContent:=GetBackupContent(RestoreTreeView);
  try
    BackupError:=myx_restore_backup(InstanceDM.PMySQL,
      PChar(AnsiToUtf8(FileToRestoreEd.Text)), BackupContent.get_record_pointer,
      PChar(AnsiToUtf8(target_catalog)), PChar(AnsiToUtf8(target_schema)), BackupType,
      0, 512, RestoreBackup, self, RestoreWarning, self);
    if(BackupError<>MYX_BACKUP_NO_ERROR)then
    begin
      ErrorTxt:=GetBackupErrorMsg(BackupError);
      case BackupError of
        MYX_BACKUP_MYSQL_ERROR:
          ErrorTxt:=Format(ErrorTxt, [myx_mysql_errno(InstanceDM.PMySQL),
            myx_mysql_error(InstanceDM.PMySQL)]);
        MYX_BACKUP_CANT_OPEN_FILE:
          ErrorTxt:=Format(ErrorTxt, [FileToRestoreEd.Text]);
      end;

      ShowModalDialog('Backup error',
        'An Error occured while executing the backup.'#13#10#13#10+
        ErrorTxt,
        myx_mtConfirmation, 'OK');
    end
    else
      ShowModalDialog('Backup restored successfully',
        'The backup was restored successfully',
        myx_mtInformation, 'OK');


  finally
    BackupContent.Free;
  end;
end;

procedure TAdminRestoreForm.ClearControls;
begin
  FileToRestoreEd.Text:='';
  TargetSchemaCBox.ItemIndex:=0;
  BackupFileTypesLU.ItemIndex:=0;
  ClearTreeView(RestoreTreeView, myx_ndt_tobject);
end;

procedure TAdminRestoreForm.Disconnected(var Message: TMessage);
begin
  DisableEnablePages(BackupRestorePageControl, False);
  ClearControls;

  OpenBackupFileBtn.Enabled:=False;
end;

procedure TAdminRestoreForm.Reconnected(var Message: TMessage);
begin
  OpenBackupFileBtn.Enabled:=True;
end;

procedure TAdminRestoreForm.RestoreTreeViewChanging(Sender: TObject;
  Node: TTreeNode; var AllowChange: Boolean);
begin
  if(RestoreTreeView.Items.Count>0)then
    if(Node=RestoreTreeView.Items[0])then
      AllowChange:=False;
end;

end.
