unit myx_public_interface;

interface

uses
  Classes, SysUtils, StrUtils, Contnrs;

{$Z4}

//------------------------------------------------------------------------------
// PUBLIC_INTERFACE "libmysqlx"

const
  libmysqlx_PUBLIC_INTERFACE_VERSION: Integer = 2;

type


  //----------------------------------------------------------------------------
  // Enum definitions

  PMYX_LIB_ERROR = ^MYX_LIB_ERROR;
  MYX_LIB_ERROR =
  (
    MYX_NO_ERROR = 0, MYX_ERROR_CANT_OPEN_FILE, MYX_ERROR_CANT_CONNECT_TO_INSTANCE, MYX_XML_PARSE_ERROR, 
    MYX_XML_NO_VALID_DOCUMENT, MYX_XML_EMPTY_DOCUMENT
  );

  PMYX_USER_CONNECTION_TYPE = ^MYX_USER_CONNECTION_TYPE;
  MYX_USER_CONNECTION_TYPE =
  (
    MYX_MYSQL_CONN = 0, MYX_ODBC_CONN, MYX_ORACLE_CONN, MYX_MSSQL_CONN, MYX_SQLITE_CONN
  );

  PMYX_USER_CONNECTION_STORAGE_TYPE = ^MYX_USER_CONNECTION_STORAGE_TYPE;
  MYX_USER_CONNECTION_STORAGE_TYPE =
  (
    MYX_FAVORITE_USER_CONNECTION = 1, MYX_HISTORY_USER_CONNECTION
  );

  PMYX_PASSWORD_STORAGE_TYPE = ^MYX_PASSWORD_STORAGE_TYPE;
  MYX_PASSWORD_STORAGE_TYPE =
  (
	MYX_PASSWORD_NOT_STORED = 1, MYX_PASSWORD_PLAINTEXT, MYX_PASSWORD_OBSCURED, MYX_PASSWORD_OS_SPECIFIC
  );

  PMYX_SYN_TYPE = ^MYX_SYN_TYPE;
  MYX_SYN_TYPE =
  (
  MYX_SYN_NORMAL,MYX_SYN_TABLE, MYX_SYN_COLUMN, MYX_SYN_COMMENT, MYX_SYN_STRING, MYX_SYN_SYMBOL, MYX_SYN_FUNCTION
  );


  //----------------------------------------------------------------------------
  // Struct definitions

  PMYX_USER_CONNECTION = ^MYX_USER_CONNECTION;
  PMYX_USER_CONNECTIONS = ^MYX_USER_CONNECTIONS;
  PMYX_NAME_VALUE_PAIR = ^MYX_NAME_VALUE_PAIR;
  PMYX_OPTION_GROUP = ^MYX_OPTION_GROUP;
  PMYX_APPLICATION_OPTIONS = ^MYX_APPLICATION_OPTIONS;
  PMYX_MACHINE_INFO = ^MYX_MACHINE_INFO;
  PMYX_PING_RESULT = ^MYX_PING_RESULT;
  PMYX_SCHEMA_TABLE_COLUMN = ^MYX_SCHEMA_TABLE_COLUMN;
  PMYX_TABLE_INDEX_COLUMN = ^MYX_TABLE_INDEX_COLUMN;
  PMYX_TABLE_INDEX = ^MYX_TABLE_INDEX;
  PMYX_TABLE_STATUS = ^MYX_TABLE_STATUS;
  PMYX_SCHEMA_TABLE_STATUS = ^MYX_SCHEMA_TABLE_STATUS;
  PMYX_SCHEMA_TABLE = ^MYX_SCHEMA_TABLE;
  PMYX_SCHEMA_TABLES = ^MYX_SCHEMA_TABLES;
  PMYX_SCHEMA = ^MYX_SCHEMA;
  PMYX_CATALOG = ^MYX_CATALOG;
  PMYX_CATALOGS = ^MYX_CATALOGS;
  PMYX_TRANS_TEXT = ^MYX_TRANS_TEXT;
  PMYX_TRANS_TEXTGROUP = ^MYX_TRANS_TEXTGROUP;
  PMYX_TRANS = ^MYX_TRANS;
  PMYX_SYN_WORD = ^MYX_SYN_WORD;
  PMYX_SQL_HIGHLIGHTING = ^MYX_SQL_HIGHLIGHTING;
  PMYX_SYN_SUGGESTION = ^MYX_SYN_SUGGESTION;
  PMYX_SYN_SUGGESTIONS = ^MYX_SYN_SUGGESTIONS;

  MYX_USER_CONNECTION = record
    connection_name: PChar;
    username: PChar;
    password: PChar;
    hostname: PChar;
    port: Cardinal;
    schema: PChar;
    advanced_options_num: Cardinal;
    advanced_options: PPChar;
    storage_path: PChar;
    notes: PChar;
    connection_type: myx_user_connection_type;
    storage_type: myx_user_connection_storage_type;
  end;

  MYX_USER_CONNECTIONS = record
    user_connections_num: Cardinal;
    last_connection: Integer;
    user_connections: PMYX_USER_CONNECTION;
  end;

  MYX_NAME_VALUE_PAIR = record
    name: PChar;
    value: PChar;
  end;

  MYX_OPTION_GROUP = record
    name: PChar;
    name_value_pairs_num: Cardinal;
    name_value_pairs: PMYX_NAME_VALUE_PAIR;
  end;

  MYX_APPLICATION_OPTIONS = record
    option_groups_num: Cardinal;
    option_groups: PMYX_OPTION_GROUP;
  end;

  MYX_MACHINE_INFO = record
    version: PChar;
    network_name: PChar;
    IP: PChar;
    OS: PChar;
    hardware: PChar;
  end;

  MYX_PING_RESULT = record
    round_trip_time: Integer;
    ttl: Integer;
  end;

  MYX_SCHEMA_TABLE_COLUMN = record
    column_name: PChar;
    column_type: PChar;
    default_value: PChar;
    extra: PChar;
    is_primary_key: Cardinal;
    is_null_allowed: Cardinal;
  end;

  MYX_TABLE_INDEX_COLUMN = record
    column_name: PChar;
    seq_in_index: PChar;
    collation: PChar;
  end;

  MYX_TABLE_INDEX = record
    key_name: PChar;
    index_type: PChar;
    is_null_allowed: Cardinal;
    is_non_unique: Cardinal;
    index_columns_num: Cardinal;
    index_columns: PMYX_TABLE_INDEX_COLUMN;
  end;

  MYX_TABLE_STATUS = record
    table_name: PChar;
    table_type: PChar;
    row_format: PChar;
    rows: PChar;
    avg_row_length: PChar;
    data_length: PChar;
    max_data_length: PChar;
    index_length: PChar;
    data_free: PChar;
    auto_increment: PChar;
    create_time: PChar;
    update_time: PChar;
    check_time: PChar;
    create_options: PChar;
    comment: PChar;
    indexes_num: Cardinal;
    indexes: PMYX_TABLE_INDEX;
    columns_num: Cardinal;
    columns: PMYX_SCHEMA_TABLE_COLUMN;
  end;

  MYX_SCHEMA_TABLE_STATUS = record
    schema_tables_num: Cardinal;
    schema_tables: PMYX_TABLE_STATUS;
  end;

  MYX_SCHEMA_TABLE = record
    table_name: PChar;
    columns_num: Cardinal;
    columns: PMYX_SCHEMA_TABLE_COLUMN;
  end;

  MYX_SCHEMA_TABLES = record
    schema_tables_num: Cardinal;
    schema_tables: PMYX_SCHEMA_TABLE;
  end;

  MYX_SCHEMA = record
    schema_name: PChar;
    catalog_name: PChar;
    schema_tables: PMYX_SCHEMA_TABLES;
  end;

  MYX_CATALOG = record
    catalog_name: PChar;
    schemata_num: Cardinal;
    schemata: PMYX_SCHEMA;
  end;

  MYX_CATALOGS = record
    catalogs_num: Cardinal;
    catalogs: PMYX_CATALOG;
  end;

  MYX_TRANS_TEXT = record
    id: PChar;
    trans: PChar;
  end;

  MYX_TRANS_TEXTGROUP = record
    id: PChar;
    texts_num: Cardinal;
    texts: PMYX_TRANS_TEXT;
  end;

  MYX_TRANS = record
    textgroups_general_num: Cardinal;
    textgroups_general: PMYX_TRANS_TEXTGROUP;
    current_textgroup_general: PMYX_TRANS_TEXTGROUP;
    textgroups_num: Cardinal;
    textgroups: PMYX_TRANS_TEXTGROUP;
    current_textgroup: PMYX_TRANS_TEXTGROUP;
  end;

  MYX_SYN_WORD = record
    word_begin: Integer;
    word_end: Integer;
    word_type: MYX_SYN_TYPE;
  end;

  MYX_SQL_HIGHLIGHTING = record
    line_states_size: Cardinal;
    line_states: Pointer;
    words_size: Cardinal;
    words_num: Cardinal;
    words: PMYX_SYN_WORD;
  end;

  MYX_SYN_SUGGESTION = record
    name: PChar;
    s_type: MYX_SYN_TYPE;
  end;

  MYX_SYN_SUGGESTIONS = record
    suggestions_num: Cardinal;
    suggestions: PMYX_SYN_SUGGESTION;
  end;

function myx_get_public_interface_version(): Integer; cdecl; external 'libmysqlx.dll';
procedure myx_free_lib_str(str: PChar);  cdecl; external 'libmysqlx.dll';
function myx_mysql_init(): Pointer; cdecl; external 'libmysqlx.dll';
function myx_set_my_cnf_path(path: PChar): Integer; cdecl; external 'libmysqlx.dll';
function myx_connect_to_instance(user_connection: PMYX_USER_CONNECTION; mysql: Pointer): Integer; cdecl; external 'libmysqlx.dll';
function myx_mysql_close(mysql: Pointer): Integer; cdecl; external 'libmysqlx.dll';
function myx_mysql_query(mysql: Pointer; cmd: PChar): Integer; cdecl; external 'libmysqlx.dll';
function myx_mysql_real_query(mysql: Pointer; cmd: PChar; len: Cardinal): Integer; cdecl; external 'libmysqlx.dll';
function myx_get_mysql_major_version(mysql: Pointer): Integer; cdecl; external 'libmysqlx.dll';
function myx_get_mysql_minor_version(mysql: Pointer): Integer; cdecl; external 'libmysqlx.dll';
function myx_get_server_variable(mysql: Pointer; name: PChar): PChar; cdecl; external 'libmysqlx.dll';
function myx_get_server_charset_name(mysql: Pointer): PChar; cdecl; external 'libmysqlx.dll';
function myx_convert_dbstr_utf8(mysql: Pointer; str: PChar): PChar; cdecl; external 'libmysqlx.dll';
function myx_convert_utf8_dbstr(mysql: Pointer; str: PChar): PChar; cdecl; external 'libmysqlx.dll';
function myx_mysql_errno(mysql: Pointer): Integer; cdecl; external 'libmysqlx.dll';
function myx_mysql_error(mysql: Pointer): PChar; cdecl; external 'libmysqlx.dll';
function myx_resolve_network_name(hostname: PChar): Longint; cdecl; external 'libmysqlx.dll';
function myx_ping_host(ip: Longint; ping_timeout: Integer; ping_result: PMYX_PING_RESULT): Integer; cdecl; external 'libmysqlx.dll';
function myx_is_localhost(hostname: PChar): Integer; cdecl; external 'libmysqlx.dll';
function myx_load_user_connections(filename: PChar; error_code: PMYX_LIB_ERROR): PMYX_USER_CONNECTIONS; cdecl; external 'libmysqlx.dll';
function myx_store_user_connections(user_connections: PMYX_USER_CONNECTIONS; pwd_storage_type: MYX_PASSWORD_STORAGE_TYPE; filename: PChar): Integer; cdecl; external 'libmysqlx.dll';
function myx_free_user_connections(user_connections: PMYX_USER_CONNECTIONS): Integer; cdecl; external 'libmysqlx.dll';
function myx_add_user_connection(user_connections: PMYX_USER_CONNECTIONS; new_connection: PMYX_USER_CONNECTION): Integer; cdecl; external 'libmysqlx.dll';
function myx_get_server_info(user_conn: PMYX_USER_CONNECTION; mysql: Pointer): PMYX_MACHINE_INFO; cdecl; external 'libmysqlx.dll';
function myx_get_client_info(mysql: Pointer): PMYX_MACHINE_INFO; cdecl; external 'libmysqlx.dll';
function myx_free_pc_info(machine_info: PMYX_MACHINE_INFO): Integer; cdecl; external 'libmysqlx.dll';
function myx_get_catalogs(mysql: Pointer): PMYX_CATALOGS; cdecl; external 'libmysqlx.dll';
function myx_get_schema_tables(mysql: Pointer; catalog_name: PChar; schema_name: PChar): PMYX_SCHEMA_TABLES; cdecl; external 'libmysqlx.dll';
function myx_get_schema_table_status(mysql: Pointer; catalog_name: PChar; schema_name: PChar): PMYX_SCHEMA_TABLE_STATUS; cdecl; external 'libmysqlx.dll';
function myx_free_catalogs(catalogs: PMYX_CATALOGS): Integer; cdecl; external 'libmysqlx.dll';
function myx_free_schema_tables(schema_tables: PMYX_SCHEMA_TABLES): Integer; cdecl; external 'libmysqlx.dll';
function myx_free_schema_table_status(schema_tables: PMYX_SCHEMA_TABLE_STATUS): Integer; cdecl; external 'libmysqlx.dll';
function myx_store_application_options(options: PMYX_APPLICATION_OPTIONS; filename: PChar): Integer; cdecl; external 'libmysqlx.dll';
function myx_get_application_options(filename: PChar; error_code: PMYX_LIB_ERROR): PMYX_APPLICATION_OPTIONS; cdecl; external 'libmysqlx.dll';
function myx_free_application_options(options: PMYX_APPLICATION_OPTIONS): Integer; cdecl; external 'libmysqlx.dll';
function myx_init_trans(translation_file_name: PChar; application_translation_file_name: PChar; language: PChar; error_code: PMYX_LIB_ERROR): PMYX_TRANS; cdecl; external 'libmysqlx.dll';
function myx_free_trans(trans: PMYX_TRANS): Integer; cdecl; external 'libmysqlx.dll';
function myx_t(trans: PMYX_TRANS; id_textgroup: PChar; id_text: PChar; original_text: PChar): PChar; cdecl; external 'libmysqlx.dll';
function myx_tg(trans: PMYX_TRANS; id_textgroup: PChar; id_text: PChar; original_text: PChar): PChar; cdecl; external 'libmysqlx.dll';
function myx_match_string(text: PChar; search: PChar; case_sensitive: Integer): Integer; cdecl; external 'libmysqlx.dll';
function myx_refresh_dbinfo(mysql: Pointer; _syn: Pointer): Pointer; cdecl; external 'libmysqlx.dll';
function myx_free_syn(syn: Pointer): Integer; cdecl; external 'libmysqlx.dll';
function myx_init_sql_parsing(syn: Pointer): PMYX_SQL_HIGHLIGHTING; cdecl; external 'libmysqlx.dll';
function myx_free_sql_highlighting(h: PMYX_SQL_HIGHLIGHTING): Integer; cdecl; external 'libmysqlx.dll';
function myx_highlight(sql_highlighting: PMYX_SQL_HIGHLIGHTING; text: PChar; line_no: Cardinal): Integer; cdecl; external 'libmysqlx.dll';
function myx_lookup_line(syn: Pointer; line: PChar): PMYX_SYN_SUGGESTIONS; cdecl; external 'libmysqlx.dll';
function myx_lookup_word(syn: Pointer; word_beg: PChar): PMYX_SYN_SUGGESTIONS; cdecl; external 'libmysqlx.dll';
function myx_free_syn_suggestions(s: PMYX_SYN_SUGGESTIONS): Integer; cdecl; external 'libmysqlx.dll';



type

  //----------------------------------------------------------------------------
  // class definitions

  TMYX_USER_CONNECTION = Class(TPersistent)
  private
    record_pointer: PMYX_USER_CONNECTION;

    Fconnection_name: string;
    Fusername: string;
    Fpassword: string;
    Fhostname: string;
    Fport: Cardinal;
    Fschema: string;
    Fstorage_path: string;
    Fnotes: string;
    Fconnection_type: myx_user_connection_type;
    Fstorage_type: myx_user_connection_storage_type;

  public

    advanced_options: TStringList;

    constructor create(connection_name: string;username: string;password: string;hostname: string;port: Cardinal;schema: string;storage_path: string;notes: string;connection_type: myx_user_connection_type;storage_type: myx_user_connection_storage_type); overload;
    constructor create(record_pointer: PMYX_USER_CONNECTION); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_USER_CONNECTION;

    procedure Assign(Source: TPersistent); override;

    property connection_name: string read Fconnection_name write Fconnection_name;
    property username: string read Fusername write Fusername;
    property password: string read Fpassword write Fpassword;
    property hostname: string read Fhostname write Fhostname;
    property port: Cardinal read Fport write Fport;
    property schema: string read Fschema write Fschema;
    property storage_path: string read Fstorage_path write Fstorage_path;
    property notes: string read Fnotes write Fnotes;
    property connection_type: myx_user_connection_type read Fconnection_type write Fconnection_type;
    property storage_type: myx_user_connection_storage_type read Fstorage_type write Fstorage_type;
  end;

  TMYX_USER_CONNECTION_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_USER_CONNECTION; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_USER_CONNECTION); overload;
  public
    function Add(AObject: TMYX_USER_CONNECTION): Integer; overload;
    function Extract(Item: TMYX_USER_CONNECTION): TMYX_USER_CONNECTION; overload;
    function Remove(AObject: TMYX_USER_CONNECTION): Integer; overload;
    function IndexOf(AObject: TMYX_USER_CONNECTION): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_USER_CONNECTION); overload;
    function First: TMYX_USER_CONNECTION; overload;
    function Last: TMYX_USER_CONNECTION; overload;
    property Items[Index: Integer]: TMYX_USER_CONNECTION read GetItem write SetItem; default;
  end;

  TMYX_USER_CONNECTIONS = Class(TPersistent)
  private
    record_pointer: PMYX_USER_CONNECTIONS;

    Flast_connection: Integer;

  public
    user_connections: TMYX_USER_CONNECTION_List;



    constructor create(last_connection: Integer); overload;
    constructor create(record_pointer: PMYX_USER_CONNECTIONS); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_USER_CONNECTIONS;

    procedure Assign(Source: TPersistent); override;

    property last_connection: Integer read Flast_connection write Flast_connection;
  end;

  TMYX_USER_CONNECTIONS_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_USER_CONNECTIONS; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_USER_CONNECTIONS); overload;
  public
    function Add(AObject: TMYX_USER_CONNECTIONS): Integer; overload;
    function Extract(Item: TMYX_USER_CONNECTIONS): TMYX_USER_CONNECTIONS; overload;
    function Remove(AObject: TMYX_USER_CONNECTIONS): Integer; overload;
    function IndexOf(AObject: TMYX_USER_CONNECTIONS): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_USER_CONNECTIONS); overload;
    function First: TMYX_USER_CONNECTIONS; overload;
    function Last: TMYX_USER_CONNECTIONS; overload;
    property Items[Index: Integer]: TMYX_USER_CONNECTIONS read GetItem write SetItem; default;
  end;

  TMYX_NAME_VALUE_PAIR = Class(TPersistent)
  private
    record_pointer: PMYX_NAME_VALUE_PAIR;

    Fname: string;
    Fvalue: string;

  public


    constructor create(name: string;value: string); overload;
    constructor create(record_pointer: PMYX_NAME_VALUE_PAIR); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_NAME_VALUE_PAIR;

    procedure Assign(Source: TPersistent); override;

    property name: string read Fname write Fname;
    property value: string read Fvalue write Fvalue;
  end;

  TMYX_NAME_VALUE_PAIR_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_NAME_VALUE_PAIR; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_NAME_VALUE_PAIR); overload;
  public
    function Add(AObject: TMYX_NAME_VALUE_PAIR): Integer; overload;
    function Extract(Item: TMYX_NAME_VALUE_PAIR): TMYX_NAME_VALUE_PAIR; overload;
    function Remove(AObject: TMYX_NAME_VALUE_PAIR): Integer; overload;
    function IndexOf(AObject: TMYX_NAME_VALUE_PAIR): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_NAME_VALUE_PAIR); overload;
    function First: TMYX_NAME_VALUE_PAIR; overload;
    function Last: TMYX_NAME_VALUE_PAIR; overload;
    property Items[Index: Integer]: TMYX_NAME_VALUE_PAIR read GetItem write SetItem; default;
  end;

  TMYX_OPTION_GROUP = Class(TPersistent)
  private
    record_pointer: PMYX_OPTION_GROUP;

    Fname: string;

  public
    name_value_pairs: TMYX_NAME_VALUE_PAIR_List;



    constructor create(name: string); overload;
    constructor create(record_pointer: PMYX_OPTION_GROUP); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_OPTION_GROUP;

    procedure Assign(Source: TPersistent); override;

    property name: string read Fname write Fname;
  end;

  TMYX_OPTION_GROUP_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_OPTION_GROUP; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_OPTION_GROUP); overload;
  public
    function Add(AObject: TMYX_OPTION_GROUP): Integer; overload;
    function Extract(Item: TMYX_OPTION_GROUP): TMYX_OPTION_GROUP; overload;
    function Remove(AObject: TMYX_OPTION_GROUP): Integer; overload;
    function IndexOf(AObject: TMYX_OPTION_GROUP): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_OPTION_GROUP); overload;
    function First: TMYX_OPTION_GROUP; overload;
    function Last: TMYX_OPTION_GROUP; overload;
    property Items[Index: Integer]: TMYX_OPTION_GROUP read GetItem write SetItem; default;
  end;

  TMYX_APPLICATION_OPTIONS = Class(TPersistent)
  private
    record_pointer: PMYX_APPLICATION_OPTIONS;

  public
    option_groups: TMYX_OPTION_GROUP_List;



    constructor create(); overload;
    constructor create(record_pointer: PMYX_APPLICATION_OPTIONS); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_APPLICATION_OPTIONS;

    procedure Assign(Source: TPersistent); override;

  end;

  TMYX_APPLICATION_OPTIONS_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_APPLICATION_OPTIONS; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_APPLICATION_OPTIONS); overload;
  public
    function Add(AObject: TMYX_APPLICATION_OPTIONS): Integer; overload;
    function Extract(Item: TMYX_APPLICATION_OPTIONS): TMYX_APPLICATION_OPTIONS; overload;
    function Remove(AObject: TMYX_APPLICATION_OPTIONS): Integer; overload;
    function IndexOf(AObject: TMYX_APPLICATION_OPTIONS): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_APPLICATION_OPTIONS); overload;
    function First: TMYX_APPLICATION_OPTIONS; overload;
    function Last: TMYX_APPLICATION_OPTIONS; overload;
    property Items[Index: Integer]: TMYX_APPLICATION_OPTIONS read GetItem write SetItem; default;
  end;

  TMYX_SCHEMA_TABLE_COLUMN = Class(TPersistent)
  private
    record_pointer: PMYX_SCHEMA_TABLE_COLUMN;

    Fcolumn_name: string;
    Fcolumn_type: string;
    Fdefault_value: string;
    Fextra: string;
    Fis_primary_key: Cardinal;
    Fis_null_allowed: Cardinal;

  public


    constructor create(column_name: string;column_type: string;default_value: string;extra: string;is_primary_key: Cardinal;is_null_allowed: Cardinal); overload;
    constructor create(record_pointer: PMYX_SCHEMA_TABLE_COLUMN); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_SCHEMA_TABLE_COLUMN;

    procedure Assign(Source: TPersistent); override;

    property column_name: string read Fcolumn_name write Fcolumn_name;
    property column_type: string read Fcolumn_type write Fcolumn_type;
    property default_value: string read Fdefault_value write Fdefault_value;
    property extra: string read Fextra write Fextra;
    property is_primary_key: Cardinal read Fis_primary_key write Fis_primary_key;
    property is_null_allowed: Cardinal read Fis_null_allowed write Fis_null_allowed;
  end;

  TMYX_SCHEMA_TABLE_COLUMN_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_SCHEMA_TABLE_COLUMN; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_SCHEMA_TABLE_COLUMN); overload;
  public
    function Add(AObject: TMYX_SCHEMA_TABLE_COLUMN): Integer; overload;
    function Extract(Item: TMYX_SCHEMA_TABLE_COLUMN): TMYX_SCHEMA_TABLE_COLUMN; overload;
    function Remove(AObject: TMYX_SCHEMA_TABLE_COLUMN): Integer; overload;
    function IndexOf(AObject: TMYX_SCHEMA_TABLE_COLUMN): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_SCHEMA_TABLE_COLUMN); overload;
    function First: TMYX_SCHEMA_TABLE_COLUMN; overload;
    function Last: TMYX_SCHEMA_TABLE_COLUMN; overload;
    property Items[Index: Integer]: TMYX_SCHEMA_TABLE_COLUMN read GetItem write SetItem; default;
  end;

  TMYX_TABLE_INDEX_COLUMN = Class(TPersistent)
  private
    record_pointer: PMYX_TABLE_INDEX_COLUMN;

    Fcolumn_name: string;
    Fseq_in_index: string;
    Fcollation: string;

  public


    constructor create(column_name: string;seq_in_index: string;collation: string); overload;
    constructor create(record_pointer: PMYX_TABLE_INDEX_COLUMN); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_TABLE_INDEX_COLUMN;

    procedure Assign(Source: TPersistent); override;

    property column_name: string read Fcolumn_name write Fcolumn_name;
    property seq_in_index: string read Fseq_in_index write Fseq_in_index;
    property collation: string read Fcollation write Fcollation;
  end;

  TMYX_TABLE_INDEX_COLUMN_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_TABLE_INDEX_COLUMN; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_TABLE_INDEX_COLUMN); overload;
  public
    function Add(AObject: TMYX_TABLE_INDEX_COLUMN): Integer; overload;
    function Extract(Item: TMYX_TABLE_INDEX_COLUMN): TMYX_TABLE_INDEX_COLUMN; overload;
    function Remove(AObject: TMYX_TABLE_INDEX_COLUMN): Integer; overload;
    function IndexOf(AObject: TMYX_TABLE_INDEX_COLUMN): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_TABLE_INDEX_COLUMN); overload;
    function First: TMYX_TABLE_INDEX_COLUMN; overload;
    function Last: TMYX_TABLE_INDEX_COLUMN; overload;
    property Items[Index: Integer]: TMYX_TABLE_INDEX_COLUMN read GetItem write SetItem; default;
  end;

  TMYX_TABLE_INDEX = Class(TPersistent)
  private
    record_pointer: PMYX_TABLE_INDEX;

    Fkey_name: string;
    Findex_type: string;
    Fis_null_allowed: Cardinal;
    Fis_non_unique: Cardinal;

  public
    index_columns: TMYX_TABLE_INDEX_COLUMN_List;



    constructor create(key_name: string;index_type: string;is_null_allowed: Cardinal;is_non_unique: Cardinal); overload;
    constructor create(record_pointer: PMYX_TABLE_INDEX); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_TABLE_INDEX;

    procedure Assign(Source: TPersistent); override;

    property key_name: string read Fkey_name write Fkey_name;
    property index_type: string read Findex_type write Findex_type;
    property is_null_allowed: Cardinal read Fis_null_allowed write Fis_null_allowed;
    property is_non_unique: Cardinal read Fis_non_unique write Fis_non_unique;
  end;

  TMYX_TABLE_INDEX_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_TABLE_INDEX; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_TABLE_INDEX); overload;
  public
    function Add(AObject: TMYX_TABLE_INDEX): Integer; overload;
    function Extract(Item: TMYX_TABLE_INDEX): TMYX_TABLE_INDEX; overload;
    function Remove(AObject: TMYX_TABLE_INDEX): Integer; overload;
    function IndexOf(AObject: TMYX_TABLE_INDEX): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_TABLE_INDEX); overload;
    function First: TMYX_TABLE_INDEX; overload;
    function Last: TMYX_TABLE_INDEX; overload;
    property Items[Index: Integer]: TMYX_TABLE_INDEX read GetItem write SetItem; default;
  end;

  TMYX_TABLE_STATUS = Class(TPersistent)
  private
    record_pointer: PMYX_TABLE_STATUS;

    Ftable_name: string;
    Ftable_type: string;
    Frow_format: string;
    Frows: string;
    Favg_row_length: string;
    Fdata_length: string;
    Fmax_data_length: string;
    Findex_length: string;
    Fdata_free: string;
    Fauto_increment: string;
    Fcreate_time: string;
    Fupdate_time: string;
    Fcheck_time: string;
    Fcreate_options: string;
    Fcomment: string;

  public
    indexes: TMYX_TABLE_INDEX_List;
    columns: TMYX_SCHEMA_TABLE_COLUMN_List;



    constructor create(table_name: string;table_type: string;row_format: string;rows: string;avg_row_length: string;data_length: string;max_data_length: string;index_length: string;data_free: string;auto_increment: string;create_time: string;update_time: string;check_time: string;create_options: string;comment: string); overload;
    constructor create(record_pointer: PMYX_TABLE_STATUS); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_TABLE_STATUS;

    procedure Assign(Source: TPersistent); override;

    property table_name: string read Ftable_name write Ftable_name;
    property table_type: string read Ftable_type write Ftable_type;
    property row_format: string read Frow_format write Frow_format;
    property rows: string read Frows write Frows;
    property avg_row_length: string read Favg_row_length write Favg_row_length;
    property data_length: string read Fdata_length write Fdata_length;
    property max_data_length: string read Fmax_data_length write Fmax_data_length;
    property index_length: string read Findex_length write Findex_length;
    property data_free: string read Fdata_free write Fdata_free;
    property auto_increment: string read Fauto_increment write Fauto_increment;
    property create_time: string read Fcreate_time write Fcreate_time;
    property update_time: string read Fupdate_time write Fupdate_time;
    property check_time: string read Fcheck_time write Fcheck_time;
    property create_options: string read Fcreate_options write Fcreate_options;
    property comment: string read Fcomment write Fcomment;
  end;

  TMYX_TABLE_STATUS_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_TABLE_STATUS; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_TABLE_STATUS); overload;
  public
    function Add(AObject: TMYX_TABLE_STATUS): Integer; overload;
    function Extract(Item: TMYX_TABLE_STATUS): TMYX_TABLE_STATUS; overload;
    function Remove(AObject: TMYX_TABLE_STATUS): Integer; overload;
    function IndexOf(AObject: TMYX_TABLE_STATUS): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_TABLE_STATUS); overload;
    function First: TMYX_TABLE_STATUS; overload;
    function Last: TMYX_TABLE_STATUS; overload;
    property Items[Index: Integer]: TMYX_TABLE_STATUS read GetItem write SetItem; default;
  end;

  TMYX_SCHEMA_TABLE_STATUS = Class(TPersistent)
  private
    record_pointer: PMYX_SCHEMA_TABLE_STATUS;

  public
    schema_tables: TMYX_TABLE_STATUS_List;



    constructor create(); overload;
    constructor create(record_pointer: PMYX_SCHEMA_TABLE_STATUS); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_SCHEMA_TABLE_STATUS;

    procedure Assign(Source: TPersistent); override;

  end;

  TMYX_SCHEMA_TABLE_STATUS_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_SCHEMA_TABLE_STATUS; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_SCHEMA_TABLE_STATUS); overload;
  public
    function Add(AObject: TMYX_SCHEMA_TABLE_STATUS): Integer; overload;
    function Extract(Item: TMYX_SCHEMA_TABLE_STATUS): TMYX_SCHEMA_TABLE_STATUS; overload;
    function Remove(AObject: TMYX_SCHEMA_TABLE_STATUS): Integer; overload;
    function IndexOf(AObject: TMYX_SCHEMA_TABLE_STATUS): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_SCHEMA_TABLE_STATUS); overload;
    function First: TMYX_SCHEMA_TABLE_STATUS; overload;
    function Last: TMYX_SCHEMA_TABLE_STATUS; overload;
    property Items[Index: Integer]: TMYX_SCHEMA_TABLE_STATUS read GetItem write SetItem; default;
  end;

  TMYX_SCHEMA_TABLE = Class(TPersistent)
  private
    record_pointer: PMYX_SCHEMA_TABLE;

    Ftable_name: string;

  public
    columns: TMYX_SCHEMA_TABLE_COLUMN_List;



    constructor create(table_name: string); overload;
    constructor create(record_pointer: PMYX_SCHEMA_TABLE); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_SCHEMA_TABLE;

    procedure Assign(Source: TPersistent); override;

    property table_name: string read Ftable_name write Ftable_name;
  end;

  TMYX_SCHEMA_TABLE_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_SCHEMA_TABLE; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_SCHEMA_TABLE); overload;
  public
    function Add(AObject: TMYX_SCHEMA_TABLE): Integer; overload;
    function Extract(Item: TMYX_SCHEMA_TABLE): TMYX_SCHEMA_TABLE; overload;
    function Remove(AObject: TMYX_SCHEMA_TABLE): Integer; overload;
    function IndexOf(AObject: TMYX_SCHEMA_TABLE): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_SCHEMA_TABLE); overload;
    function First: TMYX_SCHEMA_TABLE; overload;
    function Last: TMYX_SCHEMA_TABLE; overload;
    property Items[Index: Integer]: TMYX_SCHEMA_TABLE read GetItem write SetItem; default;
  end;

  TMYX_SCHEMA_TABLES = Class(TPersistent)
  private
    record_pointer: PMYX_SCHEMA_TABLES;

  public
    schema_tables: TMYX_SCHEMA_TABLE_List;



    constructor create(); overload;
    constructor create(record_pointer: PMYX_SCHEMA_TABLES); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_SCHEMA_TABLES;

    procedure Assign(Source: TPersistent); override;

  end;

  TMYX_SCHEMA_TABLES_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_SCHEMA_TABLES; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_SCHEMA_TABLES); overload;
  public
    function Add(AObject: TMYX_SCHEMA_TABLES): Integer; overload;
    function Extract(Item: TMYX_SCHEMA_TABLES): TMYX_SCHEMA_TABLES; overload;
    function Remove(AObject: TMYX_SCHEMA_TABLES): Integer; overload;
    function IndexOf(AObject: TMYX_SCHEMA_TABLES): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_SCHEMA_TABLES); overload;
    function First: TMYX_SCHEMA_TABLES; overload;
    function Last: TMYX_SCHEMA_TABLES; overload;
    property Items[Index: Integer]: TMYX_SCHEMA_TABLES read GetItem write SetItem; default;
  end;

  TMYX_SCHEMA = Class(TPersistent)
  private
    record_pointer: PMYX_SCHEMA;

    Fschema_name: string;
    Fcatalog_name: string;

  public
    schema_tables: TMYX_SCHEMA_TABLES;


    constructor create(schema_name: string;catalog_name: string;schema_tables: PMYX_SCHEMA_TABLES); overload;
    constructor create(record_pointer: PMYX_SCHEMA); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_SCHEMA;

    procedure Assign(Source: TPersistent); override;

    property schema_name: string read Fschema_name write Fschema_name;
    property catalog_name: string read Fcatalog_name write Fcatalog_name;
  end;

  TMYX_SCHEMA_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_SCHEMA; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_SCHEMA); overload;
  public
    function Add(AObject: TMYX_SCHEMA): Integer; overload;
    function Extract(Item: TMYX_SCHEMA): TMYX_SCHEMA; overload;
    function Remove(AObject: TMYX_SCHEMA): Integer; overload;
    function IndexOf(AObject: TMYX_SCHEMA): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_SCHEMA); overload;
    function First: TMYX_SCHEMA; overload;
    function Last: TMYX_SCHEMA; overload;
    property Items[Index: Integer]: TMYX_SCHEMA read GetItem write SetItem; default;
  end;

  TMYX_CATALOG = Class(TPersistent)
  private
    record_pointer: PMYX_CATALOG;

    Fcatalog_name: string;

  public
    schemata: TMYX_SCHEMA_List;



    constructor create(catalog_name: string); overload;
    constructor create(record_pointer: PMYX_CATALOG); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_CATALOG;

    procedure Assign(Source: TPersistent); override;

    property catalog_name: string read Fcatalog_name write Fcatalog_name;
  end;

  TMYX_CATALOG_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_CATALOG; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_CATALOG); overload;
  public
    function Add(AObject: TMYX_CATALOG): Integer; overload;
    function Extract(Item: TMYX_CATALOG): TMYX_CATALOG; overload;
    function Remove(AObject: TMYX_CATALOG): Integer; overload;
    function IndexOf(AObject: TMYX_CATALOG): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_CATALOG); overload;
    function First: TMYX_CATALOG; overload;
    function Last: TMYX_CATALOG; overload;
    property Items[Index: Integer]: TMYX_CATALOG read GetItem write SetItem; default;
  end;

  TMYX_CATALOGS = Class(TPersistent)
  private
    record_pointer: PMYX_CATALOGS;

  public
    catalogs: TMYX_CATALOG_List;



    constructor create(); overload;
    constructor create(record_pointer: PMYX_CATALOGS); overload;
    destructor destroy; override;

    function get_record_pointer: PMYX_CATALOGS;

    procedure Assign(Source: TPersistent); override;

  end;

  TMYX_CATALOGS_List = Class(TObjectList)
  protected
    function GetItem(Index: Integer): TMYX_CATALOGS; overload;
    procedure SetItem(Index: Integer; AObject: TMYX_CATALOGS); overload;
  public
    function Add(AObject: TMYX_CATALOGS): Integer; overload;
    function Extract(Item: TMYX_CATALOGS): TMYX_CATALOGS; overload;
    function Remove(AObject: TMYX_CATALOGS): Integer; overload;
    function IndexOf(AObject: TMYX_CATALOGS): Integer; overload;
    procedure Insert(Index: Integer; AObject: TMYX_CATALOGS); overload;
    function First: TMYX_CATALOGS; overload;
    function Last: TMYX_CATALOGS; overload;
    property Items[Index: Integer]: TMYX_CATALOGS read GetItem write SetItem; default;
  end;



implementation

  //----------------------------------------------------------------------------
  // class implementations

constructor TMYX_USER_CONNECTION.create(connection_name: string;username: string;password: string;hostname: string;port: Cardinal;schema: string;storage_path: string;notes: string;connection_type: myx_user_connection_type;storage_type: myx_user_connection_storage_type);
begin
  inherited create;

  self.Fconnection_name:=connection_name;
  self.Fusername:=username;
  self.Fpassword:=password;
  self.Fhostname:=hostname;
  self.Fport:=port;
  self.Fschema:=schema;
  self.Fstorage_path:=storage_path;
  self.Fnotes:=notes;
  self.Fconnection_type:=connection_type;
  self.Fstorage_type:=storage_type;
  advanced_options:=TStringList.Create;
end;

constructor TMYX_USER_CONNECTION.create(record_pointer: PMYX_USER_CONNECTION);
var i: integer;
begin

  create(Utf8ToAnsi(record_pointer.connection_name),Utf8ToAnsi(record_pointer.username),Utf8ToAnsi(record_pointer.password),Utf8ToAnsi(record_pointer.hostname),record_pointer.port,Utf8ToAnsi(record_pointer.schema),Utf8ToAnsi(record_pointer.storage_path),Utf8ToAnsi(record_pointer.notes),record_pointer.connection_type,record_pointer.storage_type);

  self.record_pointer:=nil;

  for i:=0 to record_pointer.advanced_options_num-1 do
  begin
    advanced_options.Add(
        PPChar(
          integer(record_pointer.advanced_options)+sizeof(PChar)*i
        )^
      );
  end;
end;

destructor TMYX_USER_CONNECTION.destroy;
begin
  //Free Mem allocated for List
  if(record_pointer<>nil)then
    if(record_pointer.advanced_options<>nil)then
      FreeMem(record_pointer.advanced_options);

  //Free List
  advanced_options.Free;

  if(record_pointer<>nil)then
  begin
    //Free mem for string variable connection_name
    if(record_pointer.connection_name=nil)then
      FreeMem(record_pointer.connection_name);
    //Free mem for string variable username
    if(record_pointer.username=nil)then
      FreeMem(record_pointer.username);
    //Free mem for string variable password
    if(record_pointer.password=nil)then
      FreeMem(record_pointer.password);
    //Free mem for string variable hostname
    if(record_pointer.hostname=nil)then
      FreeMem(record_pointer.hostname);
    //Free mem for string variable schema
    if(record_pointer.schema=nil)then
      FreeMem(record_pointer.schema);
    //Free mem for string variable storage_path
    if(record_pointer.storage_path=nil)then
      FreeMem(record_pointer.storage_path);
    //Free mem for string variable notes
    if(record_pointer.notes=nil)then
      FreeMem(record_pointer.notes);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_USER_CONNECTION.get_record_pointer: PMYX_USER_CONNECTION;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_USER_CONNECTION));
    record_pointer.connection_name:=nil;
    record_pointer.username:=nil;
    record_pointer.password:=nil;
    record_pointer.hostname:=nil;
    record_pointer.schema:=nil;
    record_pointer.storage_path:=nil;
    record_pointer.notes:=nil;
    record_pointer.advanced_options:=nil;
  end;

  //Allocate memory for string variable connection_name
  if(record_pointer.connection_name=nil)then
    GetMem(record_pointer.connection_name, Length(AnsiToUtf8(Fconnection_name))+1)
  else
    ReallocMem(record_pointer.connection_name, Length(AnsiToUtf8(Fconnection_name))+1);
  StrMove(record_pointer.connection_name, PChar(AnsiToUtf8(Fconnection_name)), Length(AnsiToUtf8(Fconnection_name))+1);

  //Allocate memory for string variable username
  if(record_pointer.username=nil)then
    GetMem(record_pointer.username, Length(AnsiToUtf8(Fusername))+1)
  else
    ReallocMem(record_pointer.username, Length(AnsiToUtf8(Fusername))+1);
  StrMove(record_pointer.username, PChar(AnsiToUtf8(Fusername)), Length(AnsiToUtf8(Fusername))+1);

  //Allocate memory for string variable password
  if(record_pointer.password=nil)then
    GetMem(record_pointer.password, Length(AnsiToUtf8(Fpassword))+1)
  else
    ReallocMem(record_pointer.password, Length(AnsiToUtf8(Fpassword))+1);
  StrMove(record_pointer.password, PChar(AnsiToUtf8(Fpassword)), Length(AnsiToUtf8(Fpassword))+1);

  //Allocate memory for string variable hostname
  if(record_pointer.hostname=nil)then
    GetMem(record_pointer.hostname, Length(AnsiToUtf8(Fhostname))+1)
  else
    ReallocMem(record_pointer.hostname, Length(AnsiToUtf8(Fhostname))+1);
  StrMove(record_pointer.hostname, PChar(AnsiToUtf8(Fhostname)), Length(AnsiToUtf8(Fhostname))+1);

  record_pointer.port:=Cardinal(Fport);
  //Allocate memory for string variable schema
  if(record_pointer.schema=nil)then
    GetMem(record_pointer.schema, Length(AnsiToUtf8(Fschema))+1)
  else
    ReallocMem(record_pointer.schema, Length(AnsiToUtf8(Fschema))+1);
  StrMove(record_pointer.schema, PChar(AnsiToUtf8(Fschema)), Length(AnsiToUtf8(Fschema))+1);

  //Allocate memory for string variable storage_path
  if(record_pointer.storage_path=nil)then
    GetMem(record_pointer.storage_path, Length(AnsiToUtf8(Fstorage_path))+1)
  else
    ReallocMem(record_pointer.storage_path, Length(AnsiToUtf8(Fstorage_path))+1);
  StrMove(record_pointer.storage_path, PChar(AnsiToUtf8(Fstorage_path)), Length(AnsiToUtf8(Fstorage_path))+1);

  //Allocate memory for string variable notes
  if(record_pointer.notes=nil)then
    GetMem(record_pointer.notes, Length(AnsiToUtf8(Fnotes))+1)
  else
    ReallocMem(record_pointer.notes, Length(AnsiToUtf8(Fnotes))+1);
  StrMove(record_pointer.notes, PChar(AnsiToUtf8(Fnotes)), Length(AnsiToUtf8(Fnotes))+1);

  record_pointer.connection_type:=myx_user_connection_type(Fconnection_type);
  record_pointer.storage_type:=myx_user_connection_storage_type(Fstorage_type);
  record_pointer.advanced_options_num:=advanced_options.Count;
  if(advanced_options.Count=0)then
  begin
    if(record_pointer.advanced_options<>nil)then
    begin
      FreeMem(record_pointer.advanced_options);
      record_pointer.advanced_options:=nil;
    end;
  end
  else
  begin
    ReallocMem(record_pointer.advanced_options,
      advanced_options.Count*sizeof(PChar));
  end;

  for i:=0 to advanced_options.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PPChar(
      Integer(record_pointer.advanced_options)+sizeof(PChar)*i
      )^:=PChar(advanced_options[i]);
  end;  Result:=record_pointer;
end;

procedure TMYX_USER_CONNECTION.Assign(Source: TPersistent);
begin
  if(Source is TMYX_USER_CONNECTION)then
  begin
    with TMYX_USER_CONNECTION(Source) do
    begin
      self.connection_name:=connection_name;
      self.username:=username;
      self.password:=password;
      self.hostname:=hostname;
      self.port:=port;
      self.schema:=schema;
      self.storage_path:=storage_path;
      self.notes:=notes;
      self.connection_type:=connection_type;
      self.storage_type:=storage_type;
      self.advanced_options.Assign(advanced_options);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_USER_CONNECTION_List.GetItem(Index: Integer): TMYX_USER_CONNECTION;
begin
  Result:=TMYX_USER_CONNECTION(inherited GetItem(Index));
end;

procedure TMYX_USER_CONNECTION_List.SetItem(Index: Integer; AObject: TMYX_USER_CONNECTION);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_USER_CONNECTION_List.Add(AObject: TMYX_USER_CONNECTION): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_USER_CONNECTION_List.Extract(Item: TMYX_USER_CONNECTION): TMYX_USER_CONNECTION;
begin
  Result:=TMYX_USER_CONNECTION(inherited Extract(Item));
end;

function TMYX_USER_CONNECTION_List.Remove(AObject: TMYX_USER_CONNECTION): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_USER_CONNECTION_List.IndexOf(AObject: TMYX_USER_CONNECTION): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_USER_CONNECTION_List.Insert(Index: Integer; AObject: TMYX_USER_CONNECTION);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_USER_CONNECTION_List.First: TMYX_USER_CONNECTION;
begin
  Result:=TMYX_USER_CONNECTION(inherited First);
end;

function TMYX_USER_CONNECTION_List.Last: TMYX_USER_CONNECTION;
begin
  Result:=TMYX_USER_CONNECTION(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_USER_CONNECTIONS.create(last_connection: Integer);
begin
  inherited create;

  self.Flast_connection:=last_connection;
  self.user_connections:=TMYX_USER_CONNECTION_List.Create;
end;

constructor TMYX_USER_CONNECTIONS.create(record_pointer: PMYX_USER_CONNECTIONS);
var i: integer;
begin

  create(record_pointer.last_connection);

  self.record_pointer:=nil;

  for i:=0 to record_pointer.user_connections_num-1 do
  begin
    user_connections.Add(TMYX_USER_CONNECTION.Create(
      PMYX_USER_CONNECTION(
        (Integer(record_pointer.user_connections)+sizeof(MYX_USER_CONNECTION)*i)
        )
      ));
  end;
end;

destructor TMYX_USER_CONNECTIONS.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.user_connections<>nil)then
      FreeMem(record_pointer.user_connections);

  user_connections.Free;

  if(record_pointer<>nil)then
  begin
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_USER_CONNECTIONS.get_record_pointer: PMYX_USER_CONNECTIONS;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_USER_CONNECTIONS));
    record_pointer.user_connections:=nil;
  end;

  record_pointer.last_connection:=Integer(Flast_connection);
  record_pointer.user_connections_num:=user_connections.Count;
  //if the number of user_connections is now 0, free memory
  if(user_connections.Count=0)then
  begin
    if(record_pointer.user_connections<>nil)then
    begin
      FreeMem(record_pointer.user_connections);
      record_pointer.user_connections:=nil;
    end;
  end
  //make sure enough memory is allocated for all user_connections in the "array"
  else
  begin
    ReallocMem(record_pointer.user_connections,
      user_connections.Count*sizeof(MYX_USER_CONNECTION));
  end;

  //Get all records in the "array"
  for i:=0 to user_connections.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_USER_CONNECTION(
      Integer(record_pointer.user_connections)+sizeof(MYX_USER_CONNECTION)*i
      )^:=(TMYX_USER_CONNECTION(user_connections[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_USER_CONNECTIONS.Assign(Source: TPersistent);
begin
  if(Source is TMYX_USER_CONNECTIONS)then
  begin
    with TMYX_USER_CONNECTIONS(Source) do
    begin
      self.last_connection:=last_connection;
      self.user_connections.Assign(user_connections);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_USER_CONNECTIONS_List.GetItem(Index: Integer): TMYX_USER_CONNECTIONS;
begin
  Result:=TMYX_USER_CONNECTIONS(inherited GetItem(Index));
end;

procedure TMYX_USER_CONNECTIONS_List.SetItem(Index: Integer; AObject: TMYX_USER_CONNECTIONS);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_USER_CONNECTIONS_List.Add(AObject: TMYX_USER_CONNECTIONS): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_USER_CONNECTIONS_List.Extract(Item: TMYX_USER_CONNECTIONS): TMYX_USER_CONNECTIONS;
begin
  Result:=TMYX_USER_CONNECTIONS(inherited Extract(Item));
end;

function TMYX_USER_CONNECTIONS_List.Remove(AObject: TMYX_USER_CONNECTIONS): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_USER_CONNECTIONS_List.IndexOf(AObject: TMYX_USER_CONNECTIONS): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_USER_CONNECTIONS_List.Insert(Index: Integer; AObject: TMYX_USER_CONNECTIONS);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_USER_CONNECTIONS_List.First: TMYX_USER_CONNECTIONS;
begin
  Result:=TMYX_USER_CONNECTIONS(inherited First);
end;

function TMYX_USER_CONNECTIONS_List.Last: TMYX_USER_CONNECTIONS;
begin
  Result:=TMYX_USER_CONNECTIONS(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_NAME_VALUE_PAIR.create(name: string;value: string);
begin
  inherited create;

  self.Fname:=name;
  self.Fvalue:=value;
end;

constructor TMYX_NAME_VALUE_PAIR.create(record_pointer: PMYX_NAME_VALUE_PAIR);
begin

  create(Utf8ToAnsi(record_pointer.name),Utf8ToAnsi(record_pointer.value));

  self.record_pointer:=nil;

end;

destructor TMYX_NAME_VALUE_PAIR.destroy;
begin
  if(record_pointer<>nil)then
  begin
    //Free mem for string variable name
    if(record_pointer.name=nil)then
      FreeMem(record_pointer.name);
    //Free mem for string variable value
    if(record_pointer.value=nil)then
      FreeMem(record_pointer.value);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_NAME_VALUE_PAIR.get_record_pointer: PMYX_NAME_VALUE_PAIR;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_NAME_VALUE_PAIR));
    record_pointer.name:=nil;
    record_pointer.value:=nil;
  end;

  //Allocate memory for string variable name
  if(record_pointer.name=nil)then
    GetMem(record_pointer.name, Length(AnsiToUtf8(Fname))+1)
  else
    ReallocMem(record_pointer.name, Length(AnsiToUtf8(Fname))+1);
  StrMove(record_pointer.name, PChar(AnsiToUtf8(Fname)), Length(AnsiToUtf8(Fname))+1);

  //Allocate memory for string variable value
  if(record_pointer.value=nil)then
    GetMem(record_pointer.value, Length(AnsiToUtf8(Fvalue))+1)
  else
    ReallocMem(record_pointer.value, Length(AnsiToUtf8(Fvalue))+1);
  StrMove(record_pointer.value, PChar(AnsiToUtf8(Fvalue)), Length(AnsiToUtf8(Fvalue))+1);

  Result:=record_pointer;
end;

procedure TMYX_NAME_VALUE_PAIR.Assign(Source: TPersistent);
begin
  if(Source is TMYX_NAME_VALUE_PAIR)then
  begin
    with TMYX_NAME_VALUE_PAIR(Source) do
    begin
      self.name:=name;
      self.value:=value;
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_NAME_VALUE_PAIR_List.GetItem(Index: Integer): TMYX_NAME_VALUE_PAIR;
begin
  Result:=TMYX_NAME_VALUE_PAIR(inherited GetItem(Index));
end;

procedure TMYX_NAME_VALUE_PAIR_List.SetItem(Index: Integer; AObject: TMYX_NAME_VALUE_PAIR);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_NAME_VALUE_PAIR_List.Add(AObject: TMYX_NAME_VALUE_PAIR): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_NAME_VALUE_PAIR_List.Extract(Item: TMYX_NAME_VALUE_PAIR): TMYX_NAME_VALUE_PAIR;
begin
  Result:=TMYX_NAME_VALUE_PAIR(inherited Extract(Item));
end;

function TMYX_NAME_VALUE_PAIR_List.Remove(AObject: TMYX_NAME_VALUE_PAIR): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_NAME_VALUE_PAIR_List.IndexOf(AObject: TMYX_NAME_VALUE_PAIR): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_NAME_VALUE_PAIR_List.Insert(Index: Integer; AObject: TMYX_NAME_VALUE_PAIR);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_NAME_VALUE_PAIR_List.First: TMYX_NAME_VALUE_PAIR;
begin
  Result:=TMYX_NAME_VALUE_PAIR(inherited First);
end;

function TMYX_NAME_VALUE_PAIR_List.Last: TMYX_NAME_VALUE_PAIR;
begin
  Result:=TMYX_NAME_VALUE_PAIR(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_OPTION_GROUP.create(name: string);
begin
  inherited create;

  self.Fname:=name;
  self.name_value_pairs:=TMYX_NAME_VALUE_PAIR_List.Create;
end;

constructor TMYX_OPTION_GROUP.create(record_pointer: PMYX_OPTION_GROUP);
var i: integer;
begin

  create(Utf8ToAnsi(record_pointer.name));

  self.record_pointer:=nil;

  for i:=0 to record_pointer.name_value_pairs_num-1 do
  begin
    name_value_pairs.Add(TMYX_NAME_VALUE_PAIR.Create(
      PMYX_NAME_VALUE_PAIR(
        (Integer(record_pointer.name_value_pairs)+sizeof(MYX_NAME_VALUE_PAIR)*i)
        )
      ));
  end;
end;

destructor TMYX_OPTION_GROUP.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.name_value_pairs<>nil)then
      FreeMem(record_pointer.name_value_pairs);

  name_value_pairs.Free;

  if(record_pointer<>nil)then
  begin
    //Free mem for string variable name
    if(record_pointer.name=nil)then
      FreeMem(record_pointer.name);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_OPTION_GROUP.get_record_pointer: PMYX_OPTION_GROUP;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_OPTION_GROUP));
    record_pointer.name:=nil;
    record_pointer.name_value_pairs:=nil;
  end;

  //Allocate memory for string variable name
  if(record_pointer.name=nil)then
    GetMem(record_pointer.name, Length(AnsiToUtf8(Fname))+1)
  else
    ReallocMem(record_pointer.name, Length(AnsiToUtf8(Fname))+1);
  StrMove(record_pointer.name, PChar(AnsiToUtf8(Fname)), Length(AnsiToUtf8(Fname))+1);

  record_pointer.name_value_pairs_num:=name_value_pairs.Count;
  //if the number of name_value_pairs is now 0, free memory
  if(name_value_pairs.Count=0)then
  begin
    if(record_pointer.name_value_pairs<>nil)then
    begin
      FreeMem(record_pointer.name_value_pairs);
      record_pointer.name_value_pairs:=nil;
    end;
  end
  //make sure enough memory is allocated for all name_value_pairs in the "array"
  else
  begin
    ReallocMem(record_pointer.name_value_pairs,
      name_value_pairs.Count*sizeof(MYX_NAME_VALUE_PAIR));
  end;

  //Get all records in the "array"
  for i:=0 to name_value_pairs.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_NAME_VALUE_PAIR(
      Integer(record_pointer.name_value_pairs)+sizeof(MYX_NAME_VALUE_PAIR)*i
      )^:=(TMYX_NAME_VALUE_PAIR(name_value_pairs[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_OPTION_GROUP.Assign(Source: TPersistent);
begin
  if(Source is TMYX_OPTION_GROUP)then
  begin
    with TMYX_OPTION_GROUP(Source) do
    begin
      self.name:=name;
      self.name_value_pairs.Assign(name_value_pairs);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_OPTION_GROUP_List.GetItem(Index: Integer): TMYX_OPTION_GROUP;
begin
  Result:=TMYX_OPTION_GROUP(inherited GetItem(Index));
end;

procedure TMYX_OPTION_GROUP_List.SetItem(Index: Integer; AObject: TMYX_OPTION_GROUP);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_OPTION_GROUP_List.Add(AObject: TMYX_OPTION_GROUP): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_OPTION_GROUP_List.Extract(Item: TMYX_OPTION_GROUP): TMYX_OPTION_GROUP;
begin
  Result:=TMYX_OPTION_GROUP(inherited Extract(Item));
end;

function TMYX_OPTION_GROUP_List.Remove(AObject: TMYX_OPTION_GROUP): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_OPTION_GROUP_List.IndexOf(AObject: TMYX_OPTION_GROUP): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_OPTION_GROUP_List.Insert(Index: Integer; AObject: TMYX_OPTION_GROUP);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_OPTION_GROUP_List.First: TMYX_OPTION_GROUP;
begin
  Result:=TMYX_OPTION_GROUP(inherited First);
end;

function TMYX_OPTION_GROUP_List.Last: TMYX_OPTION_GROUP;
begin
  Result:=TMYX_OPTION_GROUP(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_APPLICATION_OPTIONS.create();
begin
  inherited create;

  self.option_groups:=TMYX_OPTION_GROUP_List.Create;
end;

constructor TMYX_APPLICATION_OPTIONS.create(record_pointer: PMYX_APPLICATION_OPTIONS);
var i: integer;
begin

  create();

  self.record_pointer:=nil;

  for i:=0 to record_pointer.option_groups_num-1 do
  begin
    option_groups.Add(TMYX_OPTION_GROUP.Create(
      PMYX_OPTION_GROUP(
        (Integer(record_pointer.option_groups)+sizeof(MYX_OPTION_GROUP)*i)
        )
      ));
  end;
end;

destructor TMYX_APPLICATION_OPTIONS.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.option_groups<>nil)then
      FreeMem(record_pointer.option_groups);

  option_groups.Free;

  if(record_pointer<>nil)then
  begin
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_APPLICATION_OPTIONS.get_record_pointer: PMYX_APPLICATION_OPTIONS;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_APPLICATION_OPTIONS));
    record_pointer.option_groups:=nil;
  end;

  record_pointer.option_groups_num:=option_groups.Count;
  //if the number of option_groups is now 0, free memory
  if(option_groups.Count=0)then
  begin
    if(record_pointer.option_groups<>nil)then
    begin
      FreeMem(record_pointer.option_groups);
      record_pointer.option_groups:=nil;
    end;
  end
  //make sure enough memory is allocated for all option_groups in the "array"
  else
  begin
    ReallocMem(record_pointer.option_groups,
      option_groups.Count*sizeof(MYX_OPTION_GROUP));
  end;

  //Get all records in the "array"
  for i:=0 to option_groups.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_OPTION_GROUP(
      Integer(record_pointer.option_groups)+sizeof(MYX_OPTION_GROUP)*i
      )^:=(TMYX_OPTION_GROUP(option_groups[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_APPLICATION_OPTIONS.Assign(Source: TPersistent);
begin
  if(Source is TMYX_APPLICATION_OPTIONS)then
  begin
    with TMYX_APPLICATION_OPTIONS(Source) do
    begin
      self.option_groups.Assign(option_groups);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_APPLICATION_OPTIONS_List.GetItem(Index: Integer): TMYX_APPLICATION_OPTIONS;
begin
  Result:=TMYX_APPLICATION_OPTIONS(inherited GetItem(Index));
end;

procedure TMYX_APPLICATION_OPTIONS_List.SetItem(Index: Integer; AObject: TMYX_APPLICATION_OPTIONS);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_APPLICATION_OPTIONS_List.Add(AObject: TMYX_APPLICATION_OPTIONS): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_APPLICATION_OPTIONS_List.Extract(Item: TMYX_APPLICATION_OPTIONS): TMYX_APPLICATION_OPTIONS;
begin
  Result:=TMYX_APPLICATION_OPTIONS(inherited Extract(Item));
end;

function TMYX_APPLICATION_OPTIONS_List.Remove(AObject: TMYX_APPLICATION_OPTIONS): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_APPLICATION_OPTIONS_List.IndexOf(AObject: TMYX_APPLICATION_OPTIONS): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_APPLICATION_OPTIONS_List.Insert(Index: Integer; AObject: TMYX_APPLICATION_OPTIONS);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_APPLICATION_OPTIONS_List.First: TMYX_APPLICATION_OPTIONS;
begin
  Result:=TMYX_APPLICATION_OPTIONS(inherited First);
end;

function TMYX_APPLICATION_OPTIONS_List.Last: TMYX_APPLICATION_OPTIONS;
begin
  Result:=TMYX_APPLICATION_OPTIONS(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_SCHEMA_TABLE_COLUMN.create(column_name: string;column_type: string;default_value: string;extra: string;is_primary_key: Cardinal;is_null_allowed: Cardinal);
begin
  inherited create;

  self.Fcolumn_name:=column_name;
  self.Fcolumn_type:=column_type;
  self.Fdefault_value:=default_value;
  self.Fextra:=extra;
  self.Fis_primary_key:=is_primary_key;
  self.Fis_null_allowed:=is_null_allowed;
end;

constructor TMYX_SCHEMA_TABLE_COLUMN.create(record_pointer: PMYX_SCHEMA_TABLE_COLUMN);
begin

  create(Utf8ToAnsi(record_pointer.column_name),Utf8ToAnsi(record_pointer.column_type),Utf8ToAnsi(record_pointer.default_value),Utf8ToAnsi(record_pointer.extra),record_pointer.is_primary_key,record_pointer.is_null_allowed);

  self.record_pointer:=nil;

end;

destructor TMYX_SCHEMA_TABLE_COLUMN.destroy;
begin
  if(record_pointer<>nil)then
  begin
    //Free mem for string variable column_name
    if(record_pointer.column_name=nil)then
      FreeMem(record_pointer.column_name);
    //Free mem for string variable column_type
    if(record_pointer.column_type=nil)then
      FreeMem(record_pointer.column_type);
    //Free mem for string variable default_value
    if(record_pointer.default_value=nil)then
      FreeMem(record_pointer.default_value);
    //Free mem for string variable extra
    if(record_pointer.extra=nil)then
      FreeMem(record_pointer.extra);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_SCHEMA_TABLE_COLUMN.get_record_pointer: PMYX_SCHEMA_TABLE_COLUMN;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_SCHEMA_TABLE_COLUMN));
    record_pointer.column_name:=nil;
    record_pointer.column_type:=nil;
    record_pointer.default_value:=nil;
    record_pointer.extra:=nil;
  end;

  //Allocate memory for string variable column_name
  if(record_pointer.column_name=nil)then
    GetMem(record_pointer.column_name, Length(AnsiToUtf8(Fcolumn_name))+1)
  else
    ReallocMem(record_pointer.column_name, Length(AnsiToUtf8(Fcolumn_name))+1);
  StrMove(record_pointer.column_name, PChar(AnsiToUtf8(Fcolumn_name)), Length(AnsiToUtf8(Fcolumn_name))+1);

  //Allocate memory for string variable column_type
  if(record_pointer.column_type=nil)then
    GetMem(record_pointer.column_type, Length(AnsiToUtf8(Fcolumn_type))+1)
  else
    ReallocMem(record_pointer.column_type, Length(AnsiToUtf8(Fcolumn_type))+1);
  StrMove(record_pointer.column_type, PChar(AnsiToUtf8(Fcolumn_type)), Length(AnsiToUtf8(Fcolumn_type))+1);

  //Allocate memory for string variable default_value
  if(record_pointer.default_value=nil)then
    GetMem(record_pointer.default_value, Length(AnsiToUtf8(Fdefault_value))+1)
  else
    ReallocMem(record_pointer.default_value, Length(AnsiToUtf8(Fdefault_value))+1);
  StrMove(record_pointer.default_value, PChar(AnsiToUtf8(Fdefault_value)), Length(AnsiToUtf8(Fdefault_value))+1);

  //Allocate memory for string variable extra
  if(record_pointer.extra=nil)then
    GetMem(record_pointer.extra, Length(AnsiToUtf8(Fextra))+1)
  else
    ReallocMem(record_pointer.extra, Length(AnsiToUtf8(Fextra))+1);
  StrMove(record_pointer.extra, PChar(AnsiToUtf8(Fextra)), Length(AnsiToUtf8(Fextra))+1);

  record_pointer.is_primary_key:=Cardinal(Fis_primary_key);
  record_pointer.is_null_allowed:=Cardinal(Fis_null_allowed);
  Result:=record_pointer;
end;

procedure TMYX_SCHEMA_TABLE_COLUMN.Assign(Source: TPersistent);
begin
  if(Source is TMYX_SCHEMA_TABLE_COLUMN)then
  begin
    with TMYX_SCHEMA_TABLE_COLUMN(Source) do
    begin
      self.column_name:=column_name;
      self.column_type:=column_type;
      self.default_value:=default_value;
      self.extra:=extra;
      self.is_primary_key:=is_primary_key;
      self.is_null_allowed:=is_null_allowed;
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_SCHEMA_TABLE_COLUMN_List.GetItem(Index: Integer): TMYX_SCHEMA_TABLE_COLUMN;
begin
  Result:=TMYX_SCHEMA_TABLE_COLUMN(inherited GetItem(Index));
end;

procedure TMYX_SCHEMA_TABLE_COLUMN_List.SetItem(Index: Integer; AObject: TMYX_SCHEMA_TABLE_COLUMN);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_SCHEMA_TABLE_COLUMN_List.Add(AObject: TMYX_SCHEMA_TABLE_COLUMN): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_SCHEMA_TABLE_COLUMN_List.Extract(Item: TMYX_SCHEMA_TABLE_COLUMN): TMYX_SCHEMA_TABLE_COLUMN;
begin
  Result:=TMYX_SCHEMA_TABLE_COLUMN(inherited Extract(Item));
end;

function TMYX_SCHEMA_TABLE_COLUMN_List.Remove(AObject: TMYX_SCHEMA_TABLE_COLUMN): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_SCHEMA_TABLE_COLUMN_List.IndexOf(AObject: TMYX_SCHEMA_TABLE_COLUMN): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_SCHEMA_TABLE_COLUMN_List.Insert(Index: Integer; AObject: TMYX_SCHEMA_TABLE_COLUMN);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_SCHEMA_TABLE_COLUMN_List.First: TMYX_SCHEMA_TABLE_COLUMN;
begin
  Result:=TMYX_SCHEMA_TABLE_COLUMN(inherited First);
end;

function TMYX_SCHEMA_TABLE_COLUMN_List.Last: TMYX_SCHEMA_TABLE_COLUMN;
begin
  Result:=TMYX_SCHEMA_TABLE_COLUMN(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_TABLE_INDEX_COLUMN.create(column_name: string;seq_in_index: string;collation: string);
begin
  inherited create;

  self.Fcolumn_name:=column_name;
  self.Fseq_in_index:=seq_in_index;
  self.Fcollation:=collation;
end;

constructor TMYX_TABLE_INDEX_COLUMN.create(record_pointer: PMYX_TABLE_INDEX_COLUMN);
begin

  create(Utf8ToAnsi(record_pointer.column_name),Utf8ToAnsi(record_pointer.seq_in_index),Utf8ToAnsi(record_pointer.collation));

  self.record_pointer:=nil;

end;

destructor TMYX_TABLE_INDEX_COLUMN.destroy;
begin
  if(record_pointer<>nil)then
  begin
    //Free mem for string variable column_name
    if(record_pointer.column_name=nil)then
      FreeMem(record_pointer.column_name);
    //Free mem for string variable seq_in_index
    if(record_pointer.seq_in_index=nil)then
      FreeMem(record_pointer.seq_in_index);
    //Free mem for string variable collation
    if(record_pointer.collation=nil)then
      FreeMem(record_pointer.collation);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_TABLE_INDEX_COLUMN.get_record_pointer: PMYX_TABLE_INDEX_COLUMN;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_TABLE_INDEX_COLUMN));
    record_pointer.column_name:=nil;
    record_pointer.seq_in_index:=nil;
    record_pointer.collation:=nil;
  end;

  //Allocate memory for string variable column_name
  if(record_pointer.column_name=nil)then
    GetMem(record_pointer.column_name, Length(AnsiToUtf8(Fcolumn_name))+1)
  else
    ReallocMem(record_pointer.column_name, Length(AnsiToUtf8(Fcolumn_name))+1);
  StrMove(record_pointer.column_name, PChar(AnsiToUtf8(Fcolumn_name)), Length(AnsiToUtf8(Fcolumn_name))+1);

  //Allocate memory for string variable seq_in_index
  if(record_pointer.seq_in_index=nil)then
    GetMem(record_pointer.seq_in_index, Length(AnsiToUtf8(Fseq_in_index))+1)
  else
    ReallocMem(record_pointer.seq_in_index, Length(AnsiToUtf8(Fseq_in_index))+1);
  StrMove(record_pointer.seq_in_index, PChar(AnsiToUtf8(Fseq_in_index)), Length(AnsiToUtf8(Fseq_in_index))+1);

  //Allocate memory for string variable collation
  if(record_pointer.collation=nil)then
    GetMem(record_pointer.collation, Length(AnsiToUtf8(Fcollation))+1)
  else
    ReallocMem(record_pointer.collation, Length(AnsiToUtf8(Fcollation))+1);
  StrMove(record_pointer.collation, PChar(AnsiToUtf8(Fcollation)), Length(AnsiToUtf8(Fcollation))+1);

  Result:=record_pointer;
end;

procedure TMYX_TABLE_INDEX_COLUMN.Assign(Source: TPersistent);
begin
  if(Source is TMYX_TABLE_INDEX_COLUMN)then
  begin
    with TMYX_TABLE_INDEX_COLUMN(Source) do
    begin
      self.column_name:=column_name;
      self.seq_in_index:=seq_in_index;
      self.collation:=collation;
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_TABLE_INDEX_COLUMN_List.GetItem(Index: Integer): TMYX_TABLE_INDEX_COLUMN;
begin
  Result:=TMYX_TABLE_INDEX_COLUMN(inherited GetItem(Index));
end;

procedure TMYX_TABLE_INDEX_COLUMN_List.SetItem(Index: Integer; AObject: TMYX_TABLE_INDEX_COLUMN);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_TABLE_INDEX_COLUMN_List.Add(AObject: TMYX_TABLE_INDEX_COLUMN): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_TABLE_INDEX_COLUMN_List.Extract(Item: TMYX_TABLE_INDEX_COLUMN): TMYX_TABLE_INDEX_COLUMN;
begin
  Result:=TMYX_TABLE_INDEX_COLUMN(inherited Extract(Item));
end;

function TMYX_TABLE_INDEX_COLUMN_List.Remove(AObject: TMYX_TABLE_INDEX_COLUMN): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_TABLE_INDEX_COLUMN_List.IndexOf(AObject: TMYX_TABLE_INDEX_COLUMN): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_TABLE_INDEX_COLUMN_List.Insert(Index: Integer; AObject: TMYX_TABLE_INDEX_COLUMN);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_TABLE_INDEX_COLUMN_List.First: TMYX_TABLE_INDEX_COLUMN;
begin
  Result:=TMYX_TABLE_INDEX_COLUMN(inherited First);
end;

function TMYX_TABLE_INDEX_COLUMN_List.Last: TMYX_TABLE_INDEX_COLUMN;
begin
  Result:=TMYX_TABLE_INDEX_COLUMN(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_TABLE_INDEX.create(key_name: string;index_type: string;is_null_allowed: Cardinal;is_non_unique: Cardinal);
begin
  inherited create;

  self.Fkey_name:=key_name;
  self.Findex_type:=index_type;
  self.Fis_null_allowed:=is_null_allowed;
  self.Fis_non_unique:=is_non_unique;
  self.index_columns:=TMYX_TABLE_INDEX_COLUMN_List.Create;
end;

constructor TMYX_TABLE_INDEX.create(record_pointer: PMYX_TABLE_INDEX);
var i: integer;
begin

  create(Utf8ToAnsi(record_pointer.key_name),Utf8ToAnsi(record_pointer.index_type),record_pointer.is_null_allowed,record_pointer.is_non_unique);

  self.record_pointer:=nil;

  for i:=0 to record_pointer.index_columns_num-1 do
  begin
    index_columns.Add(TMYX_TABLE_INDEX_COLUMN.Create(
      PMYX_TABLE_INDEX_COLUMN(
        (Integer(record_pointer.index_columns)+sizeof(MYX_TABLE_INDEX_COLUMN)*i)
        )
      ));
  end;
end;

destructor TMYX_TABLE_INDEX.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.index_columns<>nil)then
      FreeMem(record_pointer.index_columns);

  index_columns.Free;

  if(record_pointer<>nil)then
  begin
    //Free mem for string variable key_name
    if(record_pointer.key_name=nil)then
      FreeMem(record_pointer.key_name);
    //Free mem for string variable index_type
    if(record_pointer.index_type=nil)then
      FreeMem(record_pointer.index_type);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_TABLE_INDEX.get_record_pointer: PMYX_TABLE_INDEX;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_TABLE_INDEX));
    record_pointer.key_name:=nil;
    record_pointer.index_type:=nil;
    record_pointer.index_columns:=nil;
  end;

  //Allocate memory for string variable key_name
  if(record_pointer.key_name=nil)then
    GetMem(record_pointer.key_name, Length(AnsiToUtf8(Fkey_name))+1)
  else
    ReallocMem(record_pointer.key_name, Length(AnsiToUtf8(Fkey_name))+1);
  StrMove(record_pointer.key_name, PChar(AnsiToUtf8(Fkey_name)), Length(AnsiToUtf8(Fkey_name))+1);

  //Allocate memory for string variable index_type
  if(record_pointer.index_type=nil)then
    GetMem(record_pointer.index_type, Length(AnsiToUtf8(Findex_type))+1)
  else
    ReallocMem(record_pointer.index_type, Length(AnsiToUtf8(Findex_type))+1);
  StrMove(record_pointer.index_type, PChar(AnsiToUtf8(Findex_type)), Length(AnsiToUtf8(Findex_type))+1);

  record_pointer.is_null_allowed:=Cardinal(Fis_null_allowed);
  record_pointer.is_non_unique:=Cardinal(Fis_non_unique);
  record_pointer.index_columns_num:=index_columns.Count;
  //if the number of index_columns is now 0, free memory
  if(index_columns.Count=0)then
  begin
    if(record_pointer.index_columns<>nil)then
    begin
      FreeMem(record_pointer.index_columns);
      record_pointer.index_columns:=nil;
    end;
  end
  //make sure enough memory is allocated for all index_columns in the "array"
  else
  begin
    ReallocMem(record_pointer.index_columns,
      index_columns.Count*sizeof(MYX_TABLE_INDEX_COLUMN));
  end;

  //Get all records in the "array"
  for i:=0 to index_columns.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_TABLE_INDEX_COLUMN(
      Integer(record_pointer.index_columns)+sizeof(MYX_TABLE_INDEX_COLUMN)*i
      )^:=(TMYX_TABLE_INDEX_COLUMN(index_columns[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_TABLE_INDEX.Assign(Source: TPersistent);
begin
  if(Source is TMYX_TABLE_INDEX)then
  begin
    with TMYX_TABLE_INDEX(Source) do
    begin
      self.key_name:=key_name;
      self.index_type:=index_type;
      self.is_null_allowed:=is_null_allowed;
      self.is_non_unique:=is_non_unique;
      self.index_columns.Assign(index_columns);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_TABLE_INDEX_List.GetItem(Index: Integer): TMYX_TABLE_INDEX;
begin
  Result:=TMYX_TABLE_INDEX(inherited GetItem(Index));
end;

procedure TMYX_TABLE_INDEX_List.SetItem(Index: Integer; AObject: TMYX_TABLE_INDEX);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_TABLE_INDEX_List.Add(AObject: TMYX_TABLE_INDEX): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_TABLE_INDEX_List.Extract(Item: TMYX_TABLE_INDEX): TMYX_TABLE_INDEX;
begin
  Result:=TMYX_TABLE_INDEX(inherited Extract(Item));
end;

function TMYX_TABLE_INDEX_List.Remove(AObject: TMYX_TABLE_INDEX): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_TABLE_INDEX_List.IndexOf(AObject: TMYX_TABLE_INDEX): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_TABLE_INDEX_List.Insert(Index: Integer; AObject: TMYX_TABLE_INDEX);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_TABLE_INDEX_List.First: TMYX_TABLE_INDEX;
begin
  Result:=TMYX_TABLE_INDEX(inherited First);
end;

function TMYX_TABLE_INDEX_List.Last: TMYX_TABLE_INDEX;
begin
  Result:=TMYX_TABLE_INDEX(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_TABLE_STATUS.create(table_name: string;table_type: string;row_format: string;rows: string;avg_row_length: string;data_length: string;max_data_length: string;index_length: string;data_free: string;auto_increment: string;create_time: string;update_time: string;check_time: string;create_options: string;comment: string);
begin
  inherited create;

  self.Ftable_name:=table_name;
  self.Ftable_type:=table_type;
  self.Frow_format:=row_format;
  self.Frows:=rows;
  self.Favg_row_length:=avg_row_length;
  self.Fdata_length:=data_length;
  self.Fmax_data_length:=max_data_length;
  self.Findex_length:=index_length;
  self.Fdata_free:=data_free;
  self.Fauto_increment:=auto_increment;
  self.Fcreate_time:=create_time;
  self.Fupdate_time:=update_time;
  self.Fcheck_time:=check_time;
  self.Fcreate_options:=create_options;
  self.Fcomment:=comment;
  self.indexes:=TMYX_TABLE_INDEX_List.Create;
  self.columns:=TMYX_SCHEMA_TABLE_COLUMN_List.Create;
end;

constructor TMYX_TABLE_STATUS.create(record_pointer: PMYX_TABLE_STATUS);
var i: integer;
begin

  create(Utf8ToAnsi(record_pointer.table_name),Utf8ToAnsi(record_pointer.table_type),Utf8ToAnsi(record_pointer.row_format),Utf8ToAnsi(record_pointer.rows),Utf8ToAnsi(record_pointer.avg_row_length),Utf8ToAnsi(record_pointer.data_length),Utf8ToAnsi(record_pointer.max_data_length),Utf8ToAnsi(record_pointer.index_length),Utf8ToAnsi(record_pointer.data_free),Utf8ToAnsi(record_pointer.auto_increment),Utf8ToAnsi(record_pointer.create_time),Utf8ToAnsi(record_pointer.update_time),Utf8ToAnsi(record_pointer.check_time),Utf8ToAnsi(record_pointer.create_options),Utf8ToAnsi(record_pointer.comment));

  self.record_pointer:=nil;

  for i:=0 to record_pointer.indexes_num-1 do
  begin
    indexes.Add(TMYX_TABLE_INDEX.Create(
      PMYX_TABLE_INDEX(
        (Integer(record_pointer.indexes)+sizeof(MYX_TABLE_INDEX)*i)
        )
      ));
  end;
  for i:=0 to record_pointer.columns_num-1 do
  begin
    columns.Add(TMYX_SCHEMA_TABLE_COLUMN.Create(
      PMYX_SCHEMA_TABLE_COLUMN(
        (Integer(record_pointer.columns)+sizeof(MYX_SCHEMA_TABLE_COLUMN)*i)
        )
      ));
  end;
end;

destructor TMYX_TABLE_STATUS.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.indexes<>nil)then
      FreeMem(record_pointer.indexes);

  indexes.Free;

  if(record_pointer<>nil)then
    if(record_pointer.columns<>nil)then
      FreeMem(record_pointer.columns);

  columns.Free;

  if(record_pointer<>nil)then
  begin
    //Free mem for string variable table_name
    if(record_pointer.table_name=nil)then
      FreeMem(record_pointer.table_name);
    //Free mem for string variable table_type
    if(record_pointer.table_type=nil)then
      FreeMem(record_pointer.table_type);
    //Free mem for string variable row_format
    if(record_pointer.row_format=nil)then
      FreeMem(record_pointer.row_format);
    //Free mem for string variable rows
    if(record_pointer.rows=nil)then
      FreeMem(record_pointer.rows);
    //Free mem for string variable avg_row_length
    if(record_pointer.avg_row_length=nil)then
      FreeMem(record_pointer.avg_row_length);
    //Free mem for string variable data_length
    if(record_pointer.data_length=nil)then
      FreeMem(record_pointer.data_length);
    //Free mem for string variable max_data_length
    if(record_pointer.max_data_length=nil)then
      FreeMem(record_pointer.max_data_length);
    //Free mem for string variable index_length
    if(record_pointer.index_length=nil)then
      FreeMem(record_pointer.index_length);
    //Free mem for string variable data_free
    if(record_pointer.data_free=nil)then
      FreeMem(record_pointer.data_free);
    //Free mem for string variable auto_increment
    if(record_pointer.auto_increment=nil)then
      FreeMem(record_pointer.auto_increment);
    //Free mem for string variable create_time
    if(record_pointer.create_time=nil)then
      FreeMem(record_pointer.create_time);
    //Free mem for string variable update_time
    if(record_pointer.update_time=nil)then
      FreeMem(record_pointer.update_time);
    //Free mem for string variable check_time
    if(record_pointer.check_time=nil)then
      FreeMem(record_pointer.check_time);
    //Free mem for string variable create_options
    if(record_pointer.create_options=nil)then
      FreeMem(record_pointer.create_options);
    //Free mem for string variable comment
    if(record_pointer.comment=nil)then
      FreeMem(record_pointer.comment);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_TABLE_STATUS.get_record_pointer: PMYX_TABLE_STATUS;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_TABLE_STATUS));
    record_pointer.table_name:=nil;
    record_pointer.table_type:=nil;
    record_pointer.row_format:=nil;
    record_pointer.rows:=nil;
    record_pointer.avg_row_length:=nil;
    record_pointer.data_length:=nil;
    record_pointer.max_data_length:=nil;
    record_pointer.index_length:=nil;
    record_pointer.data_free:=nil;
    record_pointer.auto_increment:=nil;
    record_pointer.create_time:=nil;
    record_pointer.update_time:=nil;
    record_pointer.check_time:=nil;
    record_pointer.create_options:=nil;
    record_pointer.comment:=nil;
    record_pointer.indexes:=nil;
    record_pointer.columns:=nil;
  end;

  //Allocate memory for string variable table_name
  if(record_pointer.table_name=nil)then
    GetMem(record_pointer.table_name, Length(AnsiToUtf8(Ftable_name))+1)
  else
    ReallocMem(record_pointer.table_name, Length(AnsiToUtf8(Ftable_name))+1);
  StrMove(record_pointer.table_name, PChar(AnsiToUtf8(Ftable_name)), Length(AnsiToUtf8(Ftable_name))+1);

  //Allocate memory for string variable table_type
  if(record_pointer.table_type=nil)then
    GetMem(record_pointer.table_type, Length(AnsiToUtf8(Ftable_type))+1)
  else
    ReallocMem(record_pointer.table_type, Length(AnsiToUtf8(Ftable_type))+1);
  StrMove(record_pointer.table_type, PChar(AnsiToUtf8(Ftable_type)), Length(AnsiToUtf8(Ftable_type))+1);

  //Allocate memory for string variable row_format
  if(record_pointer.row_format=nil)then
    GetMem(record_pointer.row_format, Length(AnsiToUtf8(Frow_format))+1)
  else
    ReallocMem(record_pointer.row_format, Length(AnsiToUtf8(Frow_format))+1);
  StrMove(record_pointer.row_format, PChar(AnsiToUtf8(Frow_format)), Length(AnsiToUtf8(Frow_format))+1);

  //Allocate memory for string variable rows
  if(record_pointer.rows=nil)then
    GetMem(record_pointer.rows, Length(AnsiToUtf8(Frows))+1)
  else
    ReallocMem(record_pointer.rows, Length(AnsiToUtf8(Frows))+1);
  StrMove(record_pointer.rows, PChar(AnsiToUtf8(Frows)), Length(AnsiToUtf8(Frows))+1);

  //Allocate memory for string variable avg_row_length
  if(record_pointer.avg_row_length=nil)then
    GetMem(record_pointer.avg_row_length, Length(AnsiToUtf8(Favg_row_length))+1)
  else
    ReallocMem(record_pointer.avg_row_length, Length(AnsiToUtf8(Favg_row_length))+1);
  StrMove(record_pointer.avg_row_length, PChar(AnsiToUtf8(Favg_row_length)), Length(AnsiToUtf8(Favg_row_length))+1);

  //Allocate memory for string variable data_length
  if(record_pointer.data_length=nil)then
    GetMem(record_pointer.data_length, Length(AnsiToUtf8(Fdata_length))+1)
  else
    ReallocMem(record_pointer.data_length, Length(AnsiToUtf8(Fdata_length))+1);
  StrMove(record_pointer.data_length, PChar(AnsiToUtf8(Fdata_length)), Length(AnsiToUtf8(Fdata_length))+1);

  //Allocate memory for string variable max_data_length
  if(record_pointer.max_data_length=nil)then
    GetMem(record_pointer.max_data_length, Length(AnsiToUtf8(Fmax_data_length))+1)
  else
    ReallocMem(record_pointer.max_data_length, Length(AnsiToUtf8(Fmax_data_length))+1);
  StrMove(record_pointer.max_data_length, PChar(AnsiToUtf8(Fmax_data_length)), Length(AnsiToUtf8(Fmax_data_length))+1);

  //Allocate memory for string variable index_length
  if(record_pointer.index_length=nil)then
    GetMem(record_pointer.index_length, Length(AnsiToUtf8(Findex_length))+1)
  else
    ReallocMem(record_pointer.index_length, Length(AnsiToUtf8(Findex_length))+1);
  StrMove(record_pointer.index_length, PChar(AnsiToUtf8(Findex_length)), Length(AnsiToUtf8(Findex_length))+1);

  //Allocate memory for string variable data_free
  if(record_pointer.data_free=nil)then
    GetMem(record_pointer.data_free, Length(AnsiToUtf8(Fdata_free))+1)
  else
    ReallocMem(record_pointer.data_free, Length(AnsiToUtf8(Fdata_free))+1);
  StrMove(record_pointer.data_free, PChar(AnsiToUtf8(Fdata_free)), Length(AnsiToUtf8(Fdata_free))+1);

  //Allocate memory for string variable auto_increment
  if(record_pointer.auto_increment=nil)then
    GetMem(record_pointer.auto_increment, Length(AnsiToUtf8(Fauto_increment))+1)
  else
    ReallocMem(record_pointer.auto_increment, Length(AnsiToUtf8(Fauto_increment))+1);
  StrMove(record_pointer.auto_increment, PChar(AnsiToUtf8(Fauto_increment)), Length(AnsiToUtf8(Fauto_increment))+1);

  //Allocate memory for string variable create_time
  if(record_pointer.create_time=nil)then
    GetMem(record_pointer.create_time, Length(AnsiToUtf8(Fcreate_time))+1)
  else
    ReallocMem(record_pointer.create_time, Length(AnsiToUtf8(Fcreate_time))+1);
  StrMove(record_pointer.create_time, PChar(AnsiToUtf8(Fcreate_time)), Length(AnsiToUtf8(Fcreate_time))+1);

  //Allocate memory for string variable update_time
  if(record_pointer.update_time=nil)then
    GetMem(record_pointer.update_time, Length(AnsiToUtf8(Fupdate_time))+1)
  else
    ReallocMem(record_pointer.update_time, Length(AnsiToUtf8(Fupdate_time))+1);
  StrMove(record_pointer.update_time, PChar(AnsiToUtf8(Fupdate_time)), Length(AnsiToUtf8(Fupdate_time))+1);

  //Allocate memory for string variable check_time
  if(record_pointer.check_time=nil)then
    GetMem(record_pointer.check_time, Length(AnsiToUtf8(Fcheck_time))+1)
  else
    ReallocMem(record_pointer.check_time, Length(AnsiToUtf8(Fcheck_time))+1);
  StrMove(record_pointer.check_time, PChar(AnsiToUtf8(Fcheck_time)), Length(AnsiToUtf8(Fcheck_time))+1);

  //Allocate memory for string variable create_options
  if(record_pointer.create_options=nil)then
    GetMem(record_pointer.create_options, Length(AnsiToUtf8(Fcreate_options))+1)
  else
    ReallocMem(record_pointer.create_options, Length(AnsiToUtf8(Fcreate_options))+1);
  StrMove(record_pointer.create_options, PChar(AnsiToUtf8(Fcreate_options)), Length(AnsiToUtf8(Fcreate_options))+1);

  //Allocate memory for string variable comment
  if(record_pointer.comment=nil)then
    GetMem(record_pointer.comment, Length(AnsiToUtf8(Fcomment))+1)
  else
    ReallocMem(record_pointer.comment, Length(AnsiToUtf8(Fcomment))+1);
  StrMove(record_pointer.comment, PChar(AnsiToUtf8(Fcomment)), Length(AnsiToUtf8(Fcomment))+1);

  record_pointer.indexes_num:=indexes.Count;
  //if the number of indexes is now 0, free memory
  if(indexes.Count=0)then
  begin
    if(record_pointer.indexes<>nil)then
    begin
      FreeMem(record_pointer.indexes);
      record_pointer.indexes:=nil;
    end;
  end
  //make sure enough memory is allocated for all indexes in the "array"
  else
  begin
    ReallocMem(record_pointer.indexes,
      indexes.Count*sizeof(MYX_TABLE_INDEX));
  end;

  //Get all records in the "array"
  for i:=0 to indexes.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_TABLE_INDEX(
      Integer(record_pointer.indexes)+sizeof(MYX_TABLE_INDEX)*i
      )^:=(TMYX_TABLE_INDEX(indexes[i]).get_record_pointer)^;
  end;

    record_pointer.columns_num:=columns.Count;
  //if the number of columns is now 0, free memory
  if(columns.Count=0)then
  begin
    if(record_pointer.columns<>nil)then
    begin
      FreeMem(record_pointer.columns);
      record_pointer.columns:=nil;
    end;
  end
  //make sure enough memory is allocated for all columns in the "array"
  else
  begin
    ReallocMem(record_pointer.columns,
      columns.Count*sizeof(MYX_SCHEMA_TABLE_COLUMN));
  end;

  //Get all records in the "array"
  for i:=0 to columns.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_SCHEMA_TABLE_COLUMN(
      Integer(record_pointer.columns)+sizeof(MYX_SCHEMA_TABLE_COLUMN)*i
      )^:=(TMYX_SCHEMA_TABLE_COLUMN(columns[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_TABLE_STATUS.Assign(Source: TPersistent);
begin
  if(Source is TMYX_TABLE_STATUS)then
  begin
    with TMYX_TABLE_STATUS(Source) do
    begin
      self.table_name:=table_name;
      self.table_type:=table_type;
      self.row_format:=row_format;
      self.rows:=rows;
      self.avg_row_length:=avg_row_length;
      self.data_length:=data_length;
      self.max_data_length:=max_data_length;
      self.index_length:=index_length;
      self.data_free:=data_free;
      self.auto_increment:=auto_increment;
      self.create_time:=create_time;
      self.update_time:=update_time;
      self.check_time:=check_time;
      self.create_options:=create_options;
      self.comment:=comment;
      self.indexes.Assign(indexes);
      self.columns.Assign(columns);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_TABLE_STATUS_List.GetItem(Index: Integer): TMYX_TABLE_STATUS;
begin
  Result:=TMYX_TABLE_STATUS(inherited GetItem(Index));
end;

procedure TMYX_TABLE_STATUS_List.SetItem(Index: Integer; AObject: TMYX_TABLE_STATUS);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_TABLE_STATUS_List.Add(AObject: TMYX_TABLE_STATUS): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_TABLE_STATUS_List.Extract(Item: TMYX_TABLE_STATUS): TMYX_TABLE_STATUS;
begin
  Result:=TMYX_TABLE_STATUS(inherited Extract(Item));
end;

function TMYX_TABLE_STATUS_List.Remove(AObject: TMYX_TABLE_STATUS): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_TABLE_STATUS_List.IndexOf(AObject: TMYX_TABLE_STATUS): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_TABLE_STATUS_List.Insert(Index: Integer; AObject: TMYX_TABLE_STATUS);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_TABLE_STATUS_List.First: TMYX_TABLE_STATUS;
begin
  Result:=TMYX_TABLE_STATUS(inherited First);
end;

function TMYX_TABLE_STATUS_List.Last: TMYX_TABLE_STATUS;
begin
  Result:=TMYX_TABLE_STATUS(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_SCHEMA_TABLE_STATUS.create();
begin
  inherited create;

  self.schema_tables:=TMYX_TABLE_STATUS_List.Create;
end;

constructor TMYX_SCHEMA_TABLE_STATUS.create(record_pointer: PMYX_SCHEMA_TABLE_STATUS);
var i: integer;
begin

  create();

  self.record_pointer:=nil;

  for i:=0 to record_pointer.schema_tables_num-1 do
  begin
    schema_tables.Add(TMYX_TABLE_STATUS.Create(
      PMYX_TABLE_STATUS(
        (Integer(record_pointer.schema_tables)+sizeof(MYX_TABLE_STATUS)*i)
        )
      ));
  end;
end;

destructor TMYX_SCHEMA_TABLE_STATUS.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.schema_tables<>nil)then
      FreeMem(record_pointer.schema_tables);

  schema_tables.Free;

  if(record_pointer<>nil)then
  begin
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_SCHEMA_TABLE_STATUS.get_record_pointer: PMYX_SCHEMA_TABLE_STATUS;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_SCHEMA_TABLE_STATUS));
    record_pointer.schema_tables:=nil;
  end;

  record_pointer.schema_tables_num:=schema_tables.Count;
  //if the number of schema_tables is now 0, free memory
  if(schema_tables.Count=0)then
  begin
    if(record_pointer.schema_tables<>nil)then
    begin
      FreeMem(record_pointer.schema_tables);
      record_pointer.schema_tables:=nil;
    end;
  end
  //make sure enough memory is allocated for all schema_tables in the "array"
  else
  begin
    ReallocMem(record_pointer.schema_tables,
      schema_tables.Count*sizeof(MYX_TABLE_STATUS));
  end;

  //Get all records in the "array"
  for i:=0 to schema_tables.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_TABLE_STATUS(
      Integer(record_pointer.schema_tables)+sizeof(MYX_TABLE_STATUS)*i
      )^:=(TMYX_TABLE_STATUS(schema_tables[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_SCHEMA_TABLE_STATUS.Assign(Source: TPersistent);
begin
  if(Source is TMYX_SCHEMA_TABLE_STATUS)then
  begin
    with TMYX_SCHEMA_TABLE_STATUS(Source) do
    begin
      self.schema_tables.Assign(schema_tables);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_SCHEMA_TABLE_STATUS_List.GetItem(Index: Integer): TMYX_SCHEMA_TABLE_STATUS;
begin
  Result:=TMYX_SCHEMA_TABLE_STATUS(inherited GetItem(Index));
end;

procedure TMYX_SCHEMA_TABLE_STATUS_List.SetItem(Index: Integer; AObject: TMYX_SCHEMA_TABLE_STATUS);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_SCHEMA_TABLE_STATUS_List.Add(AObject: TMYX_SCHEMA_TABLE_STATUS): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_SCHEMA_TABLE_STATUS_List.Extract(Item: TMYX_SCHEMA_TABLE_STATUS): TMYX_SCHEMA_TABLE_STATUS;
begin
  Result:=TMYX_SCHEMA_TABLE_STATUS(inherited Extract(Item));
end;

function TMYX_SCHEMA_TABLE_STATUS_List.Remove(AObject: TMYX_SCHEMA_TABLE_STATUS): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_SCHEMA_TABLE_STATUS_List.IndexOf(AObject: TMYX_SCHEMA_TABLE_STATUS): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_SCHEMA_TABLE_STATUS_List.Insert(Index: Integer; AObject: TMYX_SCHEMA_TABLE_STATUS);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_SCHEMA_TABLE_STATUS_List.First: TMYX_SCHEMA_TABLE_STATUS;
begin
  Result:=TMYX_SCHEMA_TABLE_STATUS(inherited First);
end;

function TMYX_SCHEMA_TABLE_STATUS_List.Last: TMYX_SCHEMA_TABLE_STATUS;
begin
  Result:=TMYX_SCHEMA_TABLE_STATUS(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_SCHEMA_TABLE.create(table_name: string);
begin
  inherited create;

  self.Ftable_name:=table_name;
  self.columns:=TMYX_SCHEMA_TABLE_COLUMN_List.Create;
end;

constructor TMYX_SCHEMA_TABLE.create(record_pointer: PMYX_SCHEMA_TABLE);
var i: integer;
begin

  create(Utf8ToAnsi(record_pointer.table_name));

  self.record_pointer:=nil;

  for i:=0 to record_pointer.columns_num-1 do
  begin
    columns.Add(TMYX_SCHEMA_TABLE_COLUMN.Create(
      PMYX_SCHEMA_TABLE_COLUMN(
        (Integer(record_pointer.columns)+sizeof(MYX_SCHEMA_TABLE_COLUMN)*i)
        )
      ));
  end;
end;

destructor TMYX_SCHEMA_TABLE.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.columns<>nil)then
      FreeMem(record_pointer.columns);

  columns.Free;

  if(record_pointer<>nil)then
  begin
    //Free mem for string variable table_name
    if(record_pointer.table_name=nil)then
      FreeMem(record_pointer.table_name);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_SCHEMA_TABLE.get_record_pointer: PMYX_SCHEMA_TABLE;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_SCHEMA_TABLE));
    record_pointer.table_name:=nil;
    record_pointer.columns:=nil;
  end;

  //Allocate memory for string variable table_name
  if(record_pointer.table_name=nil)then
    GetMem(record_pointer.table_name, Length(AnsiToUtf8(Ftable_name))+1)
  else
    ReallocMem(record_pointer.table_name, Length(AnsiToUtf8(Ftable_name))+1);
  StrMove(record_pointer.table_name, PChar(AnsiToUtf8(Ftable_name)), Length(AnsiToUtf8(Ftable_name))+1);

  record_pointer.columns_num:=columns.Count;
  //if the number of columns is now 0, free memory
  if(columns.Count=0)then
  begin
    if(record_pointer.columns<>nil)then
    begin
      FreeMem(record_pointer.columns);
      record_pointer.columns:=nil;
    end;
  end
  //make sure enough memory is allocated for all columns in the "array"
  else
  begin
    ReallocMem(record_pointer.columns,
      columns.Count*sizeof(MYX_SCHEMA_TABLE_COLUMN));
  end;

  //Get all records in the "array"
  for i:=0 to columns.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_SCHEMA_TABLE_COLUMN(
      Integer(record_pointer.columns)+sizeof(MYX_SCHEMA_TABLE_COLUMN)*i
      )^:=(TMYX_SCHEMA_TABLE_COLUMN(columns[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_SCHEMA_TABLE.Assign(Source: TPersistent);
begin
  if(Source is TMYX_SCHEMA_TABLE)then
  begin
    with TMYX_SCHEMA_TABLE(Source) do
    begin
      self.table_name:=table_name;
      self.columns.Assign(columns);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_SCHEMA_TABLE_List.GetItem(Index: Integer): TMYX_SCHEMA_TABLE;
begin
  Result:=TMYX_SCHEMA_TABLE(inherited GetItem(Index));
end;

procedure TMYX_SCHEMA_TABLE_List.SetItem(Index: Integer; AObject: TMYX_SCHEMA_TABLE);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_SCHEMA_TABLE_List.Add(AObject: TMYX_SCHEMA_TABLE): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_SCHEMA_TABLE_List.Extract(Item: TMYX_SCHEMA_TABLE): TMYX_SCHEMA_TABLE;
begin
  Result:=TMYX_SCHEMA_TABLE(inherited Extract(Item));
end;

function TMYX_SCHEMA_TABLE_List.Remove(AObject: TMYX_SCHEMA_TABLE): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_SCHEMA_TABLE_List.IndexOf(AObject: TMYX_SCHEMA_TABLE): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_SCHEMA_TABLE_List.Insert(Index: Integer; AObject: TMYX_SCHEMA_TABLE);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_SCHEMA_TABLE_List.First: TMYX_SCHEMA_TABLE;
begin
  Result:=TMYX_SCHEMA_TABLE(inherited First);
end;

function TMYX_SCHEMA_TABLE_List.Last: TMYX_SCHEMA_TABLE;
begin
  Result:=TMYX_SCHEMA_TABLE(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_SCHEMA_TABLES.create();
begin
  inherited create;

  self.schema_tables:=TMYX_SCHEMA_TABLE_List.Create;
end;

constructor TMYX_SCHEMA_TABLES.create(record_pointer: PMYX_SCHEMA_TABLES);
var i: integer;
begin

  create();

  self.record_pointer:=nil;

  for i:=0 to record_pointer.schema_tables_num-1 do
  begin
    schema_tables.Add(TMYX_SCHEMA_TABLE.Create(
      PMYX_SCHEMA_TABLE(
        (Integer(record_pointer.schema_tables)+sizeof(MYX_SCHEMA_TABLE)*i)
        )
      ));
  end;
end;

destructor TMYX_SCHEMA_TABLES.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.schema_tables<>nil)then
      FreeMem(record_pointer.schema_tables);

  schema_tables.Free;

  if(record_pointer<>nil)then
  begin
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_SCHEMA_TABLES.get_record_pointer: PMYX_SCHEMA_TABLES;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_SCHEMA_TABLES));
    record_pointer.schema_tables:=nil;
  end;

  record_pointer.schema_tables_num:=schema_tables.Count;
  //if the number of schema_tables is now 0, free memory
  if(schema_tables.Count=0)then
  begin
    if(record_pointer.schema_tables<>nil)then
    begin
      FreeMem(record_pointer.schema_tables);
      record_pointer.schema_tables:=nil;
    end;
  end
  //make sure enough memory is allocated for all schema_tables in the "array"
  else
  begin
    ReallocMem(record_pointer.schema_tables,
      schema_tables.Count*sizeof(MYX_SCHEMA_TABLE));
  end;

  //Get all records in the "array"
  for i:=0 to schema_tables.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_SCHEMA_TABLE(
      Integer(record_pointer.schema_tables)+sizeof(MYX_SCHEMA_TABLE)*i
      )^:=(TMYX_SCHEMA_TABLE(schema_tables[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_SCHEMA_TABLES.Assign(Source: TPersistent);
begin
  if(Source is TMYX_SCHEMA_TABLES)then
  begin
    with TMYX_SCHEMA_TABLES(Source) do
    begin
      self.schema_tables.Assign(schema_tables);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_SCHEMA_TABLES_List.GetItem(Index: Integer): TMYX_SCHEMA_TABLES;
begin
  Result:=TMYX_SCHEMA_TABLES(inherited GetItem(Index));
end;

procedure TMYX_SCHEMA_TABLES_List.SetItem(Index: Integer; AObject: TMYX_SCHEMA_TABLES);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_SCHEMA_TABLES_List.Add(AObject: TMYX_SCHEMA_TABLES): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_SCHEMA_TABLES_List.Extract(Item: TMYX_SCHEMA_TABLES): TMYX_SCHEMA_TABLES;
begin
  Result:=TMYX_SCHEMA_TABLES(inherited Extract(Item));
end;

function TMYX_SCHEMA_TABLES_List.Remove(AObject: TMYX_SCHEMA_TABLES): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_SCHEMA_TABLES_List.IndexOf(AObject: TMYX_SCHEMA_TABLES): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_SCHEMA_TABLES_List.Insert(Index: Integer; AObject: TMYX_SCHEMA_TABLES);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_SCHEMA_TABLES_List.First: TMYX_SCHEMA_TABLES;
begin
  Result:=TMYX_SCHEMA_TABLES(inherited First);
end;

function TMYX_SCHEMA_TABLES_List.Last: TMYX_SCHEMA_TABLES;
begin
  Result:=TMYX_SCHEMA_TABLES(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_SCHEMA.create(schema_name: string;catalog_name: string;schema_tables: PMYX_SCHEMA_TABLES);
begin
  inherited create;

  self.Fschema_name:=schema_name;
  self.Fcatalog_name:=catalog_name;
  if(schema_tables<>nil)then
    self.schema_tables:=TMYX_SCHEMA_TABLES.Create(schema_tables)
  else
    self.schema_tables:=nil;
end;

constructor TMYX_SCHEMA.create(record_pointer: PMYX_SCHEMA);
begin

  create(Utf8ToAnsi(record_pointer.schema_name),Utf8ToAnsi(record_pointer.catalog_name),record_pointer.schema_tables);

  self.record_pointer:=nil;

end;

destructor TMYX_SCHEMA.destroy;
begin
  if(schema_tables<>nil)then
    schema_tables.Free;
  if(record_pointer<>nil)then
  begin
    //Free mem for string variable schema_name
    if(record_pointer.schema_name=nil)then
      FreeMem(record_pointer.schema_name);
    //Free mem for string variable catalog_name
    if(record_pointer.catalog_name=nil)then
      FreeMem(record_pointer.catalog_name);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_SCHEMA.get_record_pointer: PMYX_SCHEMA;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_SCHEMA));
    record_pointer.schema_name:=nil;
    record_pointer.catalog_name:=nil;
  end;

  //Allocate memory for string variable schema_name
  if(record_pointer.schema_name=nil)then
    GetMem(record_pointer.schema_name, Length(AnsiToUtf8(Fschema_name))+1)
  else
    ReallocMem(record_pointer.schema_name, Length(AnsiToUtf8(Fschema_name))+1);
  StrMove(record_pointer.schema_name, PChar(AnsiToUtf8(Fschema_name)), Length(AnsiToUtf8(Fschema_name))+1);

  //Allocate memory for string variable catalog_name
  if(record_pointer.catalog_name=nil)then
    GetMem(record_pointer.catalog_name, Length(AnsiToUtf8(Fcatalog_name))+1)
  else
    ReallocMem(record_pointer.catalog_name, Length(AnsiToUtf8(Fcatalog_name))+1);
  StrMove(record_pointer.catalog_name, PChar(AnsiToUtf8(Fcatalog_name)), Length(AnsiToUtf8(Fcatalog_name))+1);

  if(schema_tables<>nil)then
    record_pointer.schema_tables:=schema_tables.get_record_pointer
  else
    record_pointer.schema_tables:=nil;
  Result:=record_pointer;
end;

procedure TMYX_SCHEMA.Assign(Source: TPersistent);
begin
  if(Source is TMYX_SCHEMA)then
  begin
    with TMYX_SCHEMA(Source) do
    begin
      self.schema_name:=schema_name;
      self.catalog_name:=catalog_name;
      if(self.schema_tables<>nil)and(schema_tables<>nil)then
        self.schema_tables.Assign(schema_tables);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_SCHEMA_List.GetItem(Index: Integer): TMYX_SCHEMA;
begin
  Result:=TMYX_SCHEMA(inherited GetItem(Index));
end;

procedure TMYX_SCHEMA_List.SetItem(Index: Integer; AObject: TMYX_SCHEMA);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_SCHEMA_List.Add(AObject: TMYX_SCHEMA): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_SCHEMA_List.Extract(Item: TMYX_SCHEMA): TMYX_SCHEMA;
begin
  Result:=TMYX_SCHEMA(inherited Extract(Item));
end;

function TMYX_SCHEMA_List.Remove(AObject: TMYX_SCHEMA): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_SCHEMA_List.IndexOf(AObject: TMYX_SCHEMA): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_SCHEMA_List.Insert(Index: Integer; AObject: TMYX_SCHEMA);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_SCHEMA_List.First: TMYX_SCHEMA;
begin
  Result:=TMYX_SCHEMA(inherited First);
end;

function TMYX_SCHEMA_List.Last: TMYX_SCHEMA;
begin
  Result:=TMYX_SCHEMA(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_CATALOG.create(catalog_name: string);
begin
  inherited create;

  self.Fcatalog_name:=catalog_name;
  self.schemata:=TMYX_SCHEMA_List.Create;
end;

constructor TMYX_CATALOG.create(record_pointer: PMYX_CATALOG);
var i: integer;
begin

  create(Utf8ToAnsi(record_pointer.catalog_name));

  self.record_pointer:=nil;

  for i:=0 to record_pointer.schemata_num-1 do
  begin
    schemata.Add(TMYX_SCHEMA.Create(
      PMYX_SCHEMA(
        (Integer(record_pointer.schemata)+sizeof(MYX_SCHEMA)*i)
        )
      ));
  end;
end;

destructor TMYX_CATALOG.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.schemata<>nil)then
      FreeMem(record_pointer.schemata);

  schemata.Free;

  if(record_pointer<>nil)then
  begin
    //Free mem for string variable catalog_name
    if(record_pointer.catalog_name=nil)then
      FreeMem(record_pointer.catalog_name);
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_CATALOG.get_record_pointer: PMYX_CATALOG;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_CATALOG));
    record_pointer.catalog_name:=nil;
    record_pointer.schemata:=nil;
  end;

  //Allocate memory for string variable catalog_name
  if(record_pointer.catalog_name=nil)then
    GetMem(record_pointer.catalog_name, Length(AnsiToUtf8(Fcatalog_name))+1)
  else
    ReallocMem(record_pointer.catalog_name, Length(AnsiToUtf8(Fcatalog_name))+1);
  StrMove(record_pointer.catalog_name, PChar(AnsiToUtf8(Fcatalog_name)), Length(AnsiToUtf8(Fcatalog_name))+1);

  record_pointer.schemata_num:=schemata.Count;
  //if the number of schemata is now 0, free memory
  if(schemata.Count=0)then
  begin
    if(record_pointer.schemata<>nil)then
    begin
      FreeMem(record_pointer.schemata);
      record_pointer.schemata:=nil;
    end;
  end
  //make sure enough memory is allocated for all schemata in the "array"
  else
  begin
    ReallocMem(record_pointer.schemata,
      schemata.Count*sizeof(MYX_SCHEMA));
  end;

  //Get all records in the "array"
  for i:=0 to schemata.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_SCHEMA(
      Integer(record_pointer.schemata)+sizeof(MYX_SCHEMA)*i
      )^:=(TMYX_SCHEMA(schemata[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_CATALOG.Assign(Source: TPersistent);
begin
  if(Source is TMYX_CATALOG)then
  begin
    with TMYX_CATALOG(Source) do
    begin
      self.catalog_name:=catalog_name;
      self.schemata.Assign(schemata);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_CATALOG_List.GetItem(Index: Integer): TMYX_CATALOG;
begin
  Result:=TMYX_CATALOG(inherited GetItem(Index));
end;

procedure TMYX_CATALOG_List.SetItem(Index: Integer; AObject: TMYX_CATALOG);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_CATALOG_List.Add(AObject: TMYX_CATALOG): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_CATALOG_List.Extract(Item: TMYX_CATALOG): TMYX_CATALOG;
begin
  Result:=TMYX_CATALOG(inherited Extract(Item));
end;

function TMYX_CATALOG_List.Remove(AObject: TMYX_CATALOG): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_CATALOG_List.IndexOf(AObject: TMYX_CATALOG): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_CATALOG_List.Insert(Index: Integer; AObject: TMYX_CATALOG);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_CATALOG_List.First: TMYX_CATALOG;
begin
  Result:=TMYX_CATALOG(inherited First);
end;

function TMYX_CATALOG_List.Last: TMYX_CATALOG;
begin
  Result:=TMYX_CATALOG(inherited Last);
end;


//------------------------------------------------------------------------------

constructor TMYX_CATALOGS.create();
begin
  inherited create;

  self.catalogs:=TMYX_CATALOG_List.Create;
end;

constructor TMYX_CATALOGS.create(record_pointer: PMYX_CATALOGS);
var i: integer;
begin

  create();

  self.record_pointer:=nil;

  for i:=0 to record_pointer.catalogs_num-1 do
  begin
    catalogs.Add(TMYX_CATALOG.Create(
      PMYX_CATALOG(
        (Integer(record_pointer.catalogs)+sizeof(MYX_CATALOG)*i)
        )
      ));
  end;
end;

destructor TMYX_CATALOGS.destroy;
begin
  if(record_pointer<>nil)then
    if(record_pointer.catalogs<>nil)then
      FreeMem(record_pointer.catalogs);

  catalogs.Free;

  if(record_pointer<>nil)then
  begin
  
    FreeMem(record_pointer);
  end;

  inherited destroy;
end;

function TMYX_CATALOGS.get_record_pointer: PMYX_CATALOGS;
var i: integer;
begin
  if(record_pointer=nil)then
  begin
    GetMem(record_pointer, sizeof(MYX_CATALOGS));
    record_pointer.catalogs:=nil;
  end;

  record_pointer.catalogs_num:=catalogs.Count;
  //if the number of catalogs is now 0, free memory
  if(catalogs.Count=0)then
  begin
    if(record_pointer.catalogs<>nil)then
    begin
      FreeMem(record_pointer.catalogs);
      record_pointer.catalogs:=nil;
    end;
  end
  //make sure enough memory is allocated for all catalogs in the "array"
  else
  begin
    ReallocMem(record_pointer.catalogs,
      catalogs.Count*sizeof(MYX_CATALOG));
  end;

  //Get all records in the "array"
  for i:=0 to catalogs.Count-1 do
  begin
    //Copy the record of the element to the "array"
    PMYX_CATALOG(
      Integer(record_pointer.catalogs)+sizeof(MYX_CATALOG)*i
      )^:=(TMYX_CATALOG(catalogs[i]).get_record_pointer)^;
  end;

    Result:=record_pointer;
end;

procedure TMYX_CATALOGS.Assign(Source: TPersistent);
begin
  if(Source is TMYX_CATALOGS)then
  begin
    with TMYX_CATALOGS(Source) do
    begin
      self.catalogs.Assign(catalogs);
    end;
  end
  else
    inherited Assign(Source);
end;

function TMYX_CATALOGS_List.GetItem(Index: Integer): TMYX_CATALOGS;
begin
  Result:=TMYX_CATALOGS(inherited GetItem(Index));
end;

procedure TMYX_CATALOGS_List.SetItem(Index: Integer; AObject: TMYX_CATALOGS);
begin
  inherited SetItem(Index, AObject);
end;

function TMYX_CATALOGS_List.Add(AObject: TMYX_CATALOGS): Integer;
begin
  Result:=inherited Add(AObject);
end;

function TMYX_CATALOGS_List.Extract(Item: TMYX_CATALOGS): TMYX_CATALOGS;
begin
  Result:=TMYX_CATALOGS(inherited Extract(Item));
end;

function TMYX_CATALOGS_List.Remove(AObject: TMYX_CATALOGS): Integer;
begin
  Result:=inherited Remove(AObject);
end;

function TMYX_CATALOGS_List.IndexOf(AObject: TMYX_CATALOGS): Integer;
begin
  Result:=inherited IndexOf(AObject);
end;

procedure TMYX_CATALOGS_List.Insert(Index: Integer; AObject: TMYX_CATALOGS);
begin
  inherited Insert(Index, AObject);
end;

function TMYX_CATALOGS_List.First: TMYX_CATALOGS;
begin
  Result:=TMYX_CATALOGS(inherited First);
end;

function TMYX_CATALOGS_List.Last: TMYX_CATALOGS;
begin
  Result:=TMYX_CATALOGS(inherited Last);
end;


//------------------------------------------------------------------------------



end.
