/* Copyright (C) 2003 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>

#if !defined(__WIN__) && !defined(_WIN32) && !defined(_WIN64)
#include <unistd.h>
/*typedef unsigned int           uint;*/
typedef unsigned char           uint8;
typedef unsigned char           uchar;
typedef signed char                     int8;                   
typedef unsigned short          uint16;                 
typedef short                           int16;                  
typedef unsigned long           uint32;                 
typedef long                            int32; 
#ifndef __GLIBC__
typedef unsigned long           ulong;
#endif
typedef long long longlong;
typedef unsigned long long ulonglong;
#else
typedef unsigned int           uint;   
typedef unsigned char           uint8;
typedef unsigned char           uchar;
typedef signed char                     int8;                   
typedef unsigned short          uint16;                 
typedef short                           int16;                  
typedef unsigned long           uint32;                 
typedef long                            int32;   
typedef unsigned long           ulong;
typedef __int64 longlong;
typedef unsigned __int64 ulonglong;
#endif

#include "myx_library.h"
#include "myx_network.h"
#include "myx_aux_functions.h"

#include <m_ctype.h>

/*
 * global variables
 */

enum myx_lib_error error_code;

/*
 * internal data
 */

static char *myx_my_cnf_path= NULL;

#define MYSQL_OPTION_OFFSET(option) (&(((struct st_mysql_options*)0)->use_ssl))

static struct {
  const char *name;
  int option;
  unsigned int offset;
  int type; /* 0= unused, 'F'=flag, U=unsigned int*, C=char*, B=bool */
} myx_mysql_options[] = {
  {"CONNECT_TIMEOUT=", MYSQL_OPT_CONNECT_TIMEOUT, 0, 'U'},
  {"COMPRESS=", CLIENT_COMPRESS, 0, 'F'},
  {"USE_SSL=", CLIENT_SSL, 0, 'F'},
  {NULL, 0, 0}
};



/*
 * functions
 */

//Used for debugging only
#ifdef TEST
int main(/*int argc, char **argv*/)
{
  MYX_USER_CONNECTION *myconn;
  MYSQL* mysql;
  MYX_MACHINE_INFO *info;

  myconn= malloc(sizeof(MYX_USER_CONNECTION));
  myconn->username= "root";
  myconn->password= "root";
  myconn->hostname= "localhost";
  myconn->schema= "mysql";
  myconn->port= 3306;

  mysql= myx_mysql_init();

  myx_connect_to_instance(myconn, mysql);

  info= myx_get_server_info(myconn, mysql);

  printf(info->IP);

  myx_free_pc_info(info);
  myx_mysql_close(mysql);
  free(myconn->username);
  free(myconn->password);
  free(myconn->hostname);
  free(myconn->schema);
  free(myconn);

  return 0;
}
#endif // TEST

int myx_get_public_interface_version()
{
  return libmysqlx_PUBLIC_INTERFACE_VERSION;
}


const char *myx_get_my_cnf_path()
{
  return myx_my_cnf_path;
}


int myx_set_my_cnf_path(const char *path)
{
  if (myx_my_cnf_path)
    free(myx_my_cnf_path);
  myx_my_cnf_path= strdup(path);

  return 0;
}


MYSQL* myx_mysql_init()
{
  MYSQL *mysql= mysql_init(NULL);
  
  if (myx_my_cnf_path)
    mysql_options(mysql, MYSQL_READ_DEFAULT_FILE, myx_my_cnf_path);

  return mysql;
}

int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
{
  unsigned int num;
  char *ptr;
  unsigned int i, j;
  int flags= 0;
  char *socket_path= NULL;
  int major_version_number, minor_version_number;
  
  /* read in my.cnf file sections [administrator] and [client] */
//  mysql_options(mysql,MYSQL_READ_DEFAULT_GROUP,"administrator");

  /* setup advanced options */
  for (i= 0; i < user_conn->advanced_options_num; i++)
  {
    char *opvalue= user_conn->advanced_options[i];
    
    /* this is an exception */
    if (str_beginswith(opvalue, "SOCKET_PATH")
        && (ptr= strchr(opvalue, '=')))
    {
      socket_path= ptr+1;
      continue;
    }
    for (j= 0; myx_mysql_options[j].name; j++)
    {
      if (str_beginswith(opvalue, myx_mysql_options[j].name)
          && (ptr= strchr(opvalue, '=')))
      {
        int option= myx_mysql_options[j].option;
        ptr++;
        
        if (option < 0)
        {
          switch (myx_mysql_options[j].type)
          {
          case 'B':
            *(my_bool*)((char*)&mysql->options+myx_mysql_options[j].offset)= atoi(ptr);
            break;
          }
        }
        else
        {
          switch (myx_mysql_options[j].type)
          {
          case 'U':
            num= atoi(ptr);
            mysql_options(mysql, option, (char*)&num);
            break;
          case 'C':
            mysql_options(mysql, option, ptr);
            break;
          case 0:
            mysql_options(mysql, option, 0);
            break;
          case 'F':
            flags |= option;
            break;
          }
        }
      }
    }
  }

  if(!mysql_real_connect(mysql, user_conn->hostname, user_conn->username,
    user_conn->password, user_conn->schema, user_conn->port, socket_path, flags))
  {
    return -1;
  }
  else
  {
    //if connected to a server 4.1.x or higher, set charactersets to UTF8
    major_version_number= myx_get_mysql_major_version(mysql);
    minor_version_number= myx_get_mysql_minor_version(mysql);
    if(((major_version_number==4)&&(minor_version_number>=1))||
      (major_version_number>4))
    {
      if (mysql_query(mysql, "SET NAMES utf8"))
      {
        return -1;
      }
    }

    return 0;
  }
}

int myx_mysql_close(MYSQL *mysql)
{
  mysql_close(mysql);

  return 0;
}

int myx_mysql_errno(MYSQL *mysql)
{
  return mysql_errno(mysql);
}

const char *myx_mysql_error(MYSQL *mysql)
{
  return mysql_error(mysql);
}

int myx_get_mysql_major_version(MYSQL *mysql)
{
  static MYSQL *last_used_mysql= NULL;
  static int major_version;

  if (!last_used_mysql || last_used_mysql != mysql)
  {
    const char *version;

    last_used_mysql= mysql;

    version= mysql_get_server_info(mysql);
    sscanf(version, "%i.%*s", &major_version);
  }

  return major_version;
}

int myx_get_mysql_minor_version(MYSQL *mysql)
{
  static MYSQL *last_used_mysql= NULL;
  static int minor_version= 0;

  if (!last_used_mysql || last_used_mysql != mysql)
  {
    const char *version;

    last_used_mysql= mysql;

    version= mysql_get_server_info(mysql);
    if (sscanf(version, "%*i.%i.%*s", &minor_version)!=1)
      minor_version= 0;
  }

  return minor_version;
}


MYX_MACHINE_INFO * myx_get_server_info(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
{
  MYX_MACHINE_INFO *machine_info= malloc(sizeof(MYX_MACHINE_INFO));
  char *add_host_info;

  machine_info->version= malloc(sizeof(char)*64);
  machine_info->network_name= malloc(sizeof(char)*64);
  machine_info->IP= malloc(sizeof(char)*16);
  machine_info->OS= malloc(sizeof(char)*16);
  machine_info->hardware= malloc(sizeof(char)*200);

  if(mysql->options.protocol==MYSQL_PROTOCOL_MEMORY)
  {
    add_host_info= strdup("via shared memory");
  }
  else if(mysql->options.protocol==MYSQL_PROTOCOL_SOCKET)
  {
    add_host_info= strdup("via socket");  
  }
  else if(mysql->options.protocol==MYSQL_PROTOCOL_PIPE)
  {
    add_host_info= strdup("via named pipe");  
  }
  else if(mysql->options.protocol==MYSQL_PROTOCOL_TCP)
  {
    add_host_info= strdup("via TCP/IP");
  }
  else
  {
    add_host_info= strdup("");
  }

  sprintf(machine_info->version, "MySQL %s %s", (unsigned char *)mysql_get_server_info(mysql), add_host_info);

  free(add_host_info);

  myx_get_ip_as_string(user_conn->hostname, machine_info->IP);
  myx_get_network_name_by_ip(machine_info->IP, machine_info->network_name);

  machine_info->OS= NULL;
  machine_info->hardware= NULL;

  return machine_info;
}

#ifdef __GNUC__
#include <unistd.h>
#endif
MYX_MACHINE_INFO * myx_get_client_info(MYSQL *mysql)
{
  MYX_MACHINE_INFO *machine_info= malloc(sizeof(MYX_MACHINE_INFO));

  machine_info->version= malloc(sizeof(char)*64);
  machine_info->network_name= malloc(sizeof(char)*64);
  machine_info->IP= malloc(sizeof(char)*16);

  sprintf(machine_info->version, "MySQL Client Version %d, Protocol %d", MYSQL_VERSION_ID, mysql_get_proto_info(mysql));

  gethostname(machine_info->network_name, 64);

  myx_get_ip_as_string(machine_info->network_name, machine_info->IP);

  machine_info->OS= get_local_os_name();
 
  machine_info->hardware= get_local_hardware_info();

  return machine_info;
}

int myx_free_pc_info(MYX_MACHINE_INFO *machine_info)
{
  free(machine_info->version);
  free(machine_info->network_name);
  free(machine_info->IP);
  if (machine_info->OS)
      free(machine_info->OS);
  if (machine_info->hardware)
      free(machine_info->hardware);
  free(machine_info);

  return 0;
}



/*
 *----------------------------------------------------------------------
 * myx_match_string
 *
 * SYNOPSIS
 *   Match a string to a search pattern.
 * 
 * DESCRIPTION
 *   Checks whether a text string matches a given search string, 
 * interpreting certain wild characters appropriately.
 * 
 * RETURN VALUE
 * 
 * -1 if the search string is empty
 * 1 if the string matches
 * 0 if the string doesnt match
 *
 * NOTES
 * 
 * Supported wild characters:
 *   %  match anything, any number of chars
 *   _  match a single character
 *
 *----------------------------------------------------------------------
 */
static int dummy(int c)
{
  return c;
}

int myx_match_string(const char *text, const char *search, int case_sensitive)
{
#define MANY '%'
#define SINGLE '_'

#ifdef __GNUC__
  const char *stack[strlen(text)];
#else
  const char **stack= malloc(sizeof(char)*((unsigned int)strlen(text)+1));
#endif
  int stackp= 0;
  int (*normcase)(int);

  if (case_sensitive)
      normcase= dummy;
  else
      normcase= toupper;

  if (!search || !*search)
      return -1;

  while (*search && *text)
  {
    switch (*search)
    {
     case MANY:
      ++search;
      if (*search != '_')
      {
        stack[stackp++]= search;
        while (*text && *search && normcase(*text) != normcase(*search))
            ++text;
        break;
      }

     case SINGLE:
      ++search;
      ++text;
      break;
      
     default:
      if (normcase(*text) != normcase(*search))
      {
        if (stackp==0)
            return 0;
        else
        {
          // backtrack
          search= stack[--stackp];
          while (*text && *search && normcase(*text) != normcase(*search))
              ++text;
        }
      }
      else
      {
        ++search;
        ++text;
      }
      break;
    }
  }

#ifndef __GNUC__
  free((void*)stack);
#endif

  return !*search;
}

const char * myx_get_server_charset_name(MYSQL *mysql)
{
  return mysql->charset->csname;
}


char *myx_get_server_variable(MYSQL *mysql, const char *name)
{
  char *sqlcmd;
  char *value= NULL;
  MYSQL_RES *res;
  MYSQL_ROW row;

  sqlcmd= g_strdup_printf("SHOW VARIABLES LIKE '%s'", name);

  if (!(myx_mysql_query(mysql, sqlcmd)))
  {
    if ((res= mysql_store_result(mysql)))
    {
      if ((row= mysql_fetch_row(res)))
      {
        value= g_strdup(row[1]);
      }

      mysql_free_result(res);
    }
  }

  g_free(sqlcmd);

  return value;
}

char *myx_convert_dbstr_utf8(MYSQL *mysql, const char* str)
{
  GError *error= NULL;
  gchar *utf8= NULL;
  gsize bytes_read= 0;
  char *charset = NULL;

  //If string is NULL return NULL
  if(!str)
    return NULL;

  //Check if string already in UTF8
	if ( g_utf8_validate(str, -1, NULL) == TRUE )
		return g_strdup(str);

  //Get server charset
  charset= g_strdup(myx_get_server_charset_name(mysql));

  utf8= g_convert(str, -1, "UTF-8", charset, &bytes_read, NULL, &error);

  g_free(charset);

	//check if it worked
	if (error == NULL)
		return utf8;

	g_free(utf8);

	//try western european into UTF8

  utf8= g_convert(str, -1, "UTF-8", "ISO-8859-1", &bytes_read, NULL, &error);

	//check if it worked
	if (error == NULL)
		return utf8;

	g_free(utf8);

  return NULL;

	/*
  // try locale into UTF8

  if (!charset)
		(void) g_get_charset(&charset);

	utf8 = g_locale_to_utf8 (str, -1, &bytes_read, NULL, &error);

	//check if it worked
	if (error == NULL)
		return utf8;

	g_free(utf8);
	g_error_free(error);
	error = NULL;

	// LAST: if all that doesn't work, convert from locale and replace
	//       problematic charactes with code                           

	g_print("FIXME: in %s: charset conversion error in g_convert().\n"
	        "\tError message: %s\n\tFirst problematic character: %s\n",
	        __FUNCTION__, error->message, str + bytes_read);


	utf8= g_convert_with_fallback (str, -1, "UTF-8", charset, NULL, NULL, NULL, NULL);

	g_error_free(error);

	return utf8;*/
}

char *myx_convert_utf8_dbstr(MYSQL *mysql, const char* str)
{
  GError *error= NULL;
  gchar *converted_str= NULL;
  gsize bytes_read= 0;
  char *charset = NULL;
  int major_version_number= myx_get_mysql_major_version(mysql);
  int minor_version_number= myx_get_mysql_minor_version(mysql);

  //If string is NULL return NULL
  if(!str)
    return NULL;

  //if connected to a server 4.1.x or higher, return UTF8
  if(((major_version_number==4)&&(minor_version_number>=1))||
    (major_version_number>4))
  {
    return g_strdup(str);
  }

  //Get server charset
  charset= g_strdup(myx_get_server_charset_name(mysql));

  converted_str= g_convert(str, -1, charset, "UTF-8", &bytes_read, NULL, &error);

  g_free(charset);

	//check if it worked
	if (error == NULL)
		return converted_str;

  g_free(converted_str);

  return NULL;
}

void myx_free_lib_str(char *str)
{
  g_free(str);
}

int myx_mysql_query(MYSQL *mysql, const char *cmd)
{
  char *conv_cmd= myx_convert_utf8_dbstr(mysql, cmd);
  int res;

  res= mysql_query(mysql, conv_cmd);

  g_free(conv_cmd);

  return res;
}

int myx_mysql_real_query(MYSQL *mysql, const char *cmd, unsigned int len)
{
  char *conv_cmd;
  int res;
  int major_version_number= myx_get_mysql_major_version(mysql);
  int minor_version_number= myx_get_mysql_minor_version(mysql);

  //if connected to a server 4.1.x or higher, execute UTF8
  if(((major_version_number==4)&&(minor_version_number>=1))||
    (major_version_number>4))
  {
    res= mysql_real_query(mysql, cmd, len);
  }
  //if connected to a server 4.0.x or lower, convert cmd to server's charset
  else
  {
    conv_cmd= myx_convert_utf8_dbstr(mysql, cmd);

    res= mysql_real_query(mysql, conv_cmd, len);

    g_free(conv_cmd);
  }

  return res;
}
