/* Copyright (C) 2003 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>

#include <myx_library.h>

// Windows includes
#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
#include <windows.h>
#include <winsock.h>
// Include icmp headers for ping
#include <ipexport.h>
#include <icmpapi.h>
#else

#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/socket.h>

#endif


/*
 * functions
 */

struct hostent *get_host_info(const char *hostname)
{
   /* get host information by host's IP number */
   if (isdigit(hostname[0]))
   {
      unsigned long hostnum = inet_addr(hostname);
      return gethostbyaddr ((char *)&hostnum, sizeof (unsigned long), AF_INET);
   }
   /* get host information by host's computer name */
   else
   {
      return gethostbyname ((char *) hostname);
   }
}

unsigned long myx_resolve_network_name(const char *hostname)
{
  unsigned long ulIP = 0;
  struct hostent* phostent;

  phostent = gethostbyname(hostname);

  if (phostent == NULL)
    return 0;

#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
  ulIP = *(DWORD*)(*phostent->h_addr_list);
#else
  ulIP = *(unsigned long*)(*phostent->h_addr_list);
#endif

  return ulIP;
}

int myx_get_ip_as_string(const char *hostname, char *ip)
{
#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
  struct in_addr in;
#endif
  struct hostent *P;

  P= gethostbyname(hostname);

#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
  in.S_un.S_un_b.s_b1= P->h_addr_list[0][0];
  in.S_un.S_un_b.s_b2= P->h_addr_list[0][1];
  in.S_un.S_un_b.s_b3= P->h_addr_list[0][2];
  in.S_un.S_un_b.s_b4= P->h_addr_list[0][3];

  sprintf(ip, "%s", inet_ntoa(in));
#else
  strcpy(ip, inet_ntoa(*(struct in_addr*)P->h_addr_list[0]));
#endif

  return 0;
}

void ip_to_str(struct hostent *P, unsigned int nr, char *ip)
{
#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
  struct in_addr in;
  in.S_un.S_un_b.s_b1= P->h_addr_list[nr][0];
  in.S_un.S_un_b.s_b2= P->h_addr_list[nr][1];
  in.S_un.S_un_b.s_b3= P->h_addr_list[nr][2];
  in.S_un.S_un_b.s_b4= P->h_addr_list[nr][3];

  sprintf(ip, "%s", inet_ntoa(in));
#else
  strcpy(ip, inet_ntoa(*(struct in_addr*)P->h_addr_list[0]));
#endif
}

int myx_get_network_name_by_ip(const char *ip, char *hostname)
{
  struct hostent *hostptr;

  hostptr= get_host_info(ip);
  if (hostptr == NULL)
  {
    hostname[0]= 0;
    return -1;
  }
  
  strncpy(hostname, hostptr->h_name, 63);
  hostname[63]= 0;

  return 0;
}

/*int myx_get_network_name_by_ip(const char *ip, char *hostname)
{
  struct hostent *P;
  unsigned long ulip = myx_resolve_network_name(ip);

  P= gethostbyaddr((char *)&ulip, 4, AF_INET);
  if(P)
  {
    strncpy(hostname, P->h_name, 63);
    hostname[63]= 0;
    return 0;
  }
  else
  {
    strcpy(hostname, "");
    return -1;
  }
}*/

int myx_ping_host(unsigned long ip, int ping_timeout, MYX_PING_RESULT *ping_result)
{
  int res = -1;
#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
  unsigned long ulip = ip;
  char reply[sizeof(struct icmp_echo_reply)+8];
  struct icmp_echo_reply* iep=(struct icmp_echo_reply*)&reply;

  HANDLE icmphandle = IcmpCreateFile();

  iep->RoundTripTime = 0xffffffff;

  if(IcmpSendEcho(icmphandle, ulip, 0, 0, NULL, reply, sizeof(struct icmp_echo_reply)+8, ping_timeout)) 
  {
    ping_result->round_trip_time= iep->RoundTripTime;
    ping_result->ttl= iep->Options.Ttl;

    res= 0;
  }
  
  if(iep->Status>0)
  {
    res= iep->Status-11000+1;
  }

  IcmpCloseHandle(icmphandle);
#else
  char buf[32];
  char result[128];
  pid_t pid;
  FILE *f;
  char *argv[8];
  
  strcpy(buf, inet_ntoa(*(struct in_addr*)&ip));
  
  argv[0]= "/bin/ping";
  argv[1]= buf;
  argv[2]= "-c";
  argv[3]= "1";
  argv[4]= NULL;

  /*
   * exec() external /bin/ping since doing it ourselves would require
   * the program being ran as root.
   */
  
  ping_result->round_trip_time= 0xffffffff;

  f= myx_popen(argv, &pid);
  if (f) 
  {
    while (1)
    {
      if (myx_read_timeout(f, ping_timeout, result, sizeof(result)) > 0)
      {
        float rtime;
        int ttl;

        char *p= strstr(result, "ttl=");
        if (p && sscanf(p, "ttl=%i time=%f", &ttl, &rtime) == 2) 
        {
          ping_result->round_trip_time= rtime;
          ping_result->ttl= ttl;
          res= 0;
          break;
        }
      } 
      else 
      {
        break;
      }
    }
    myx_pclose(f, pid);
  }
#endif
  return res;
}


int myx_is_localhost(const char *hostname)
{
  char *ip= malloc(sizeof(char)*(15+1));
  char *ip2= malloc(sizeof(char)*(15+1));
  char *network_name= malloc(sizeof(char)*(65));
  struct hostent *P;
  int i, is_local= 0;

  //Get host IP as string
  myx_get_ip_as_string(hostname, ip);

  if(strcmp(ip, "127.0.0.1")!=0)
  {
    //Get localhost IPs
    gethostname(network_name, 64);

    P= gethostbyname(network_name);

    i= 0;
    while(P->h_addr_list[i])
    {
      ip_to_str(P, i, ip2);
      if(strcmp(ip, ip2)==0)
      {
        is_local= 1;
        break;
      }

      i++;
    }
  }
  else
  {
    is_local= 1;
  }

  free(network_name);
  free(ip2);
  free(ip);

  return is_local;
}
