/* Copyright (C) 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#include "MGGladeXML.h"




MGGladeXML::MGGladeXML(const std::string &file, const Glib::ustring &root)
  : _xml(0)
{
  static bool initialized= false;
  
  if (!initialized)
  {
    initialized= true;
    glade_init();
  }

  _xml= glade_xml_new(file.c_str(), root.empty()?NULL:root.c_str(), NULL);
  
  if (!_xml)
  {
    throw Error("could not open glade file "+Glib::filename_to_utf8(file));
  }
}


Gtk::Widget *MGGladeXML::get_widget(const Glib::ustring &name) const
{
  GtkWidget *w= glade_xml_get_widget(_xml, name.c_str());
  if (!w)
    return 0;
  
  return Glib::wrap(w);
}


Gtk::Entry *MGGladeXML::get_entry(const Glib::ustring &name) const
{
  return (Gtk::Entry*)get_widget(name);
}


Gtk::ToggleButton *MGGladeXML::get_toggle(const Glib::ustring &name) const
{
  return (Gtk::ToggleButton*)get_widget(name);
}


Gtk::TreeView *MGGladeXML::get_tree(const Glib::ustring &name) const
{
  return (Gtk::TreeView*)get_widget(name);
}


Gtk::Button *MGGladeXML::get_button(const Glib::ustring &name) const
{
  return (Gtk::Button*)get_widget(name);
}


Gtk::Label *MGGladeXML::get_label(const Glib::ustring &name) const
{
  return (Gtk::Label*)get_widget(name);
}

Gtk::SpinButton *MGGladeXML::get_spin(const Glib::ustring &name) const
{
  return (Gtk::SpinButton*)get_widget(name);
}

Gtk::TextView *MGGladeXML::get_text(const Glib::ustring &name) const
{
  return (Gtk::TextView*)get_widget(name);
}

Gtk::OptionMenu *MGGladeXML::get_option(const Glib::ustring &name) const
{
  return (Gtk::OptionMenu*)get_widget(name);
}

