unit OptionsEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, ImgList, AuxFuncs,
  ApplicationDataModule, myx_public_interface, Grids, ValEdit,
  Options, PNGImage, MyxError;

type
  TOptionsForm = class(TForm)
    HeaderPnl: TPanel;
    Image1: TImage;
    TitleLbl: TLabel;
    OptionTreeImageList: TImageList;
    OptionPageControl: TPageControl;
    GeneralTabSheet: TTabSheet;
    ConnectionsTabSheet: TTabSheet;
    GroupBox1: TGroupBox;
    ShowTipOfDayCBox: TCheckBox;
    StoreWindowsPosCBox: TCheckBox;
    GroupBox2: TGroupBox;
    CoverPnl: TPanel;
    GroupBox3: TGroupBox;
    Label3: TLabel;
    LanguageCBox: TComboBox;
    CloseBtn: TButton;
    Label2: TLabel;
    FontEd: TEdit;
    ChooseFont: TButton;
    CharsetCBox: TComboBox;
    Label4: TLabel;
    ConnsTreeView: TTreeView;
    ConnectionPageControl: TPageControl;
    ConnGeneralTabSheet: TTabSheet;
    ConnAdvTabSheet: TTabSheet;
    ConnectionLbl: TLabel;
    UsernameLbl: TLabel;
    PasswordLbl: TLabel;
    PortLbl: TLabel;
    HostnameLbl: TLabel;
    TypeLbl: TLabel;
    SchemataEdLbl: TLabel;
    PasswordEd: TEdit;
    PortEd: TEdit;
    TypeCBox: TComboBox;
    SchemataEd: TEdit;
    UsernameEd: TEdit;
    ConnectionEd: TEdit;
    HostnameEd: TEdit;
    Label1: TLabel;
    ConnMemo: TMemo;
    ApplyChangesBtn: TButton;
    DiscardChangesBtn: TButton;
    ConnValueListEditor: TValueListEditor;
    Label5: TLabel;
    GroupBox5: TGroupBox;
    StorePasswordCBox: TCheckBox;
    PasswordStorageTypeLbl: TLabel;
    PasswordStorageTypeCBox: TComboBox;
    AddConnectionBtn: TButton;
    DeleteConnectionBtn: TButton;
    SidebarScrollBox: TScrollBox;
    Panel1: TPanel;

    constructor Create(AOwner: TComponent;
      ApplOptionForm: TApplicationOptionsForm); reintroduce;
    destructor Destroy; override;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure ShowOptionPage(PageNr: integer);
    procedure OptionImgClick(Sender: TObject);
    procedure AddOptionPage(IconName: string; TopPos: integer;
      PageNr: integer; PageTitle: string);
    procedure BuildOptionSideBar;

    procedure CloseBtnClick(Sender: TObject);
    procedure OptionTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure OptionPageControlChange(Sender: TObject);
    procedure ConnsTreeViewChange(Sender: TObject; Node: TTreeNode);

    procedure LoadPageData(PageNr: integer);
    procedure DoPageContentChanged(Sender: TObject);
    procedure ApplyChanges;
    function DiscardChanges(ConfirmDiscard: Boolean = True): Boolean;

    procedure OptionTreeViewChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    procedure ApplyChangesBtnClick(Sender: TObject);
    procedure DiscardChangesBtnClick(Sender: TObject);
    procedure ConnsTreeViewChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    procedure DeleteConnectionBtnClick(Sender: TObject);

    procedure AddConnection(UserConn: TMYX_USER_CONNECTION = nil);
    procedure AddConnectionBtnClick(Sender: TObject);

    procedure StorePasswordCBoxClick(Sender: TObject);

    procedure DisableEnableConnectionPages(Enable: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    PageContentChanged,
    InitControls: Boolean;

    NewConnection: TMYX_USER_CONNECTION;

    DiscardingChanges: Boolean;

    OptionsImgs: TList;

    ApplOptionForm: TApplicationOptionsForm;
  public
    { Public declarations }
    stored_conns: TMYX_USER_CONNECTIONS;
    last_connection: integer;
  end;

const
  ConnectionsPage = -1;

implementation

{$R *.dfm}

constructor TOptionsForm.Create(AOwner: TComponent;
  ApplOptionForm: TApplicationOptionsForm);
begin
  inherited Create(AOwner);

  self.ApplOptionForm:=ApplOptionForm;
end;

destructor TOptionsForm.Destroy;
begin
  ApplOptionForm.Free;

  inherited Destroy;
end;

procedure TOptionsForm.FormCreate(Sender: TObject);
begin
  stored_conns:=nil;
  PageContentChanged:=False;
  InitControls:=False;

  NewConnection:=nil;
  last_connection:=-1;

  OptionPageControl.ActivePageIndex:=0;
  ConnectionPageControl.ActivePageIndex:=0;

  OptionPageControl.Top:=_s(20);
  OptionPageControl.Height:=_s(329);
  HeaderPnl.BringToFront;

  Width:=_s(710);

  BuildOptionSideBar;

  ShowOptionPage(0);

  DisableEnableConnectionPages(False);
end;

procedure TOptionsForm.FormDestroy(Sender: TObject);
var i: integer;
begin
  for i:=0 to OptionsImgs.Count-1 do
    TPNGObject(OptionsImgs[i]).Free;

  OptionsImgs.Free;
end;

procedure TOptionsForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if(Not(DiscardChanges))then
    Abort;
end;

procedure TOptionsForm.ShowOptionPage(PageNr: integer);
var i: integer;
begin
  if(PageNr<0)then
    PageNr:=OptionPageControl.PageCount+PageNr;

  OptionPageControl.ActivePageIndex:=PageNr;
  TitleLbl.Caption:=OptionPageControl.ActivePage.Caption;

  for i:=0 to SidebarScrollBox.ComponentCount-1 do
    if(TComponent(SidebarScrollBox.Components[i]) is TLabel)then
    begin
      if(TComponent(SidebarScrollBox.Components[i]).Tag=PageNr)then
      begin
        TLabel(SidebarScrollBox.Components[i]).Color:=$009F9F9F;
        TLabel(SidebarScrollBox.Components[i]).Transparent:=False;
      end
      else
      begin
        TLabel(SidebarScrollBox.Components[i]).Transparent:=True;
      end;
    end;

  OptionPageControlChange(self);
end;

procedure TOptionsForm.OptionImgClick(Sender: TObject);
begin
  ShowOptionPage(TComponent(Sender).Tag);
end;

procedure TOptionsForm.AddOptionPage(IconName: string; TopPos: integer;
  PageNr: integer; PageTitle: string);
var Img: TImage;
  Lbl: TLabel;
  PNGImg: TPNGObject;
begin
  Img:=TImage.Create(SidebarScrollBox);
  Img.Parent:=SidebarScrollBox;
  Img.Width:=48;
  Img.Height:=48;
  Img.Left:=(SidebarScrollBox.Width-Img.Width-2) div 2;
  Img.Top:=TopPos;
  PNGImg:=LoadPNGImageFromResource(IconName, Img);
  OptionsImgs.Add(PNGImg);
  Img.Tag:=PageNr;
  Img.OnClick:=OptionImgClick;

  Lbl:=TLabel.Create(SidebarScrollBox);
  Lbl.Parent:=SidebarScrollBox;
  Lbl.Caption:=' '+PageTitle+' ';
  Lbl.Left:=(SidebarScrollBox.Width-Lbl.Width-2) div 2;;
  Lbl.Top:=TopPos+48+4;
  Lbl.Alignment:=taCenter;
  Lbl.AutoSize:=False;
  Lbl.Tag:=PageNr;
  Lbl.Transparent:=True;
  Lbl.OnClick:=OptionImgClick;
end;

procedure TOptionsForm.BuildOptionSideBar;
var Img: TImage;
  ypos, i: integer;
  TabSheet: TTabSheet;
  ApplPagesCount: integer;
begin
  OptionsImgs:=TList.Create;

  //Create Background Img
  if(ApplOptionForm<>nil)then
  begin
    Img:=TImage.Create(self);
    Img.Parent:=SidebarScrollBox;
    Img.Width:=111;
    Img.Height:=166;
    Img.Left:=0;
    Img.Top:=35+48+13;
    OptionsImgs.Add(LoadPNGImageFromResource('options_bg', Img));
  end;

  //Start at 25
  ypos:=25;
  ApplPagesCount:=0;
  if(ApplOptionForm<>nil)then
  begin
    ApplOptionForm.DoPageContentChanged:=DoPageContentChanged;

    if(ApplOptionForm.DockOptionPageControl<>nil)then
    begin
      ApplPagesCount:=ApplOptionForm.DockOptionPageControl.PageCount;

      for i:=0 to ApplPagesCount-1 do
      begin
        TabSheet:=ApplOptionForm.DockOptionPageControl.Pages[0];
        TabSheet.PageControl:=OptionPageControl;
        TabSheet.PageIndex:=i;

        if(i<ApplOptionForm.OptionsImgNames.Count)then
          AddOptionPage(ApplOptionForm.OptionsImgNames[i],
            ypos, i, TabSheet.Caption);

        ypos:=ypos+84;
      end;
    end;
  end;

  AddOptionPage('options_general', ypos,
    ApplPagesCount, 'General Options');

  ypos:=ypos+84;
  AddOptionPage('options_connections', ypos,
    ApplPagesCount+1, 'Connections');
end;

procedure TOptionsForm.DisableEnableConnectionPages(Enable: Boolean);
var i: integer;
begin
  //Enable/Disable Controls on pages
  for i:=0 to ConnectionPageControl.PageCount-1 do
    DisableEnableControls(ConnectionPageControl.Pages[i], Enable);
end;

procedure TOptionsForm.CloseBtnClick(Sender: TObject);
begin
  ModalResult:=mrOK;
end;

procedure TOptionsForm.OptionTreeViewChange(Sender: TObject;
  Node: TTreeNode);
begin
  if(Node<>nil)then
    if(Node.Data<>nil)then
      if(TObject(Node.Data) is TTabSheet)then
      begin
        OptionPageControl.ActivePage:=Node.Data;
        TitleLbl.Caption:=Node.Text;

        OptionPageControlChange(self);
      end;
end;

procedure TOptionsForm.OptionPageControlChange(Sender: TObject);
begin
  LoadPageData(OptionPageControl.ActivePageIndex);
end;

procedure TOptionsForm.ConnsTreeViewChange(Sender: TObject;
  Node: TTreeNode);
begin
  DeleteConnectionBtn.Enabled:=False;

  if(Node<>nil)then
    if(Node.Data<>nil)then
    begin
      InitControls:=True;
      try
        ConnectionEd.Text:=TMYX_USER_CONNECTION(Node.Data).connection_name;
        UsernameEd.Text:=TMYX_USER_CONNECTION(Node.Data).username;
        PasswordEd.Text:=TMYX_USER_CONNECTION(Node.Data).password;
        HostnameEd.Text:=TMYX_USER_CONNECTION(Node.Data).hostname;
        PortEd.Text:=IntToStr(TMYX_USER_CONNECTION(Node.Data).port);
        TypeCBox.ItemIndex:=Ord(TMYX_USER_CONNECTION(Node.Data).connection_type);
        SchemataEd.Text:=TMYX_USER_CONNECTION(Node.Data).schema;
        ConnMemo.Text:=TMYX_USER_CONNECTION(Node.Data).notes;

        ConnValueListEditor.Strings.Assign(
          TMYX_USER_CONNECTION(Node.Data).advanced_options);

        //Enable Controls on pages
        DisableEnableConnectionPages(True);
      finally
        InitControls:=False;
      end;


      //Set last_connection
      last_connection:=stored_conns.user_connections.IndexOf(TMYX_USER_CONNECTION(Node.Data));

      if(Visible)and(ConnectionEd.Enabled)and
        (ConnectionPageControl.ActivePage=ConnGeneralTabSheet)then
        ConnectionEd.SetFocus;

      DeleteConnectionBtn.Enabled:=True;
    end
    else
    begin
      InitControls:=True;
      try
        ConnectionEd.Text:='';
        UsernameEd.Text:='';
        PasswordEd.Text:='';
        HostnameEd.Text:='';
        PortEd.Text:='';
        TypeCBox.ItemIndex:=-1;
        SchemataEd.Text:='';
        ConnMemo.Text:='';

        ConnValueListEditor.Strings.Clear;

        //Disable Controls on pages
        DisableEnableConnectionPages(False);
      finally
        InitControls:=False;
      end;
    end;
end;

procedure TOptionsForm.DoPageContentChanged(Sender: TObject);
begin
  if(not(InitControls))then
  begin
    PageContentChanged:=True;

    ApplyChangesBtn.Enabled:=True;
    DiscardChangesBtn.Enabled:=True;
  end;
end;

function TOptionsForm.DiscardChanges(ConfirmDiscard: Boolean): Boolean;
var res: integer;
  pos: integer;
begin
  DiscardChanges:=True;

  //Set DiscardingChanges so there will be no recursive calls
  DiscardingChanges:=True;
  try
    if(PageContentChanged)then
    begin
      if(ConfirmDiscard)then
      begin
        res:=ShowModalDialog('Save changes?',
          'Some of the settings have changed. Do you want to apply these '+
          'changes permanently?', myx_mtConfirmation,
          'Yes'#13#10'No'#13#10'Abort');

        if(res=1)then
          ApplyChanges
        else if(res=3)then
        begin
          DiscardChanges:=False;
          Exit;
        end;
      end;

      if(OptionPageControl.ActivePage=ConnectionsTabSheet)then
      begin
        if(NewConnection<>nil)then
        begin
          try
            pos:=stored_conns.user_connections.IndexOf(NewConnection);
            if(pos>=0)then
              stored_conns.user_connections.Delete(pos);

            ConnsTreeView.Items.Delete(ConnsTreeView.Selected);
          finally
            NewConnection:=nil;
          end;
        end
        else
          //Reset Connection Edits
          ConnsTreeViewChange(self, ConnsTreeView.Selected);
      end
      else
        LoadPageData(OptionPageControl.ActivePageIndex);

      PageContentChanged:=False;
      ApplyChangesBtn.Enabled:=False;
      DiscardChangesBtn.Enabled:=False;
    end;
  finally
    DiscardingChanges:=False;
  end;
end;

procedure TOptionsForm.OptionTreeViewChanging(Sender: TObject;
  Node: TTreeNode; var AllowChange: Boolean);
begin
  DiscardChanges;
end;

procedure TOptionsForm.ApplyChangesBtnClick(Sender: TObject);
begin
  ApplyChanges;
end;

procedure TOptionsForm.DiscardChangesBtnClick(Sender: TObject);
begin
  DiscardChanges(False);
end;

procedure TOptionsForm.ConnsTreeViewChanging(Sender: TObject;
  Node: TTreeNode; var AllowChange: Boolean);
begin
  if(not(DiscardingChanges))then
    AllowChange:=DiscardChanges
  else
    AllowChange:=True;
end;

procedure TOptionsForm.DeleteConnectionBtnClick(Sender: TObject);
var pos: integer;
begin
  if(ConnsTreeView.Selected<>nil)and(stored_conns<>nil)then
    if(ConnsTreeView.Selected.Data<>nil)then
    begin
      pos:=stored_conns.user_connections.IndexOf(
        TMYX_USER_CONNECTION(ConnsTreeView.Selected.Data));

      if(pos<>-1)then
      begin
        if(ShowModalDialog('Delete connection?',
          'Are you sure you want to delete the connection '+
          TMYX_USER_CONNECTION(ConnsTreeView.Selected.Data).connection_name+'?',
          myx_mtConfirmation, 'Yes'#13#10'No')=1)then
        begin
          PageContentChanged:=False;
          ApplyChangesBtn.Enabled:=False;
          DiscardChangesBtn.Enabled:=False;

          stored_conns.user_connections.Delete(pos);
          ConnsTreeView.Items.Delete(ConnsTreeView.Selected);

          ConnsTreeView.Selected:=ConnsTreeView.Items[0];

          {myx_store_user_connections(stored_conns.get_record_pointer,
            ApplicationDM.Options.PasswordStorageType,
            PChar(AnsiToUtf8(ApplicationDM.Options.ConnectionsFileName)));}
        end;
      end;
    end;
end;

procedure TOptionsForm.AddConnection(UserConn: TMYX_USER_CONNECTION = nil);
var Node: TTreeNode;
  FreeConnection: Boolean;
begin
  if(UserConn=nil)then
  begin
    UserConn:=TMYX_USER_CONNECTION.Create('New Connection',
      '', '', '',
      3306, '',
      '', '',
      MYX_MYSQL_CONN, MYX_FAVORITE_USER_CONNECTION);

    FreeConnection:=True;
  end
  else
    FreeConnection:=False;

  try
    Node:=AddTreeViewChildNode(ConnsTreeView, ConnsTreeView.Items[0],
      'New Connection', 6,
      UserConn);

    stored_conns.user_connections.Add(UserConn);
    stored_conns.last_connection:=stored_conns.user_connections.Count-1;

    ConnsTreeView.Selected:=Node;

    ConnectionEd.SelectAll;

    NewConnection:=UserConn;

    DoPageContentChanged(self);
  except
    if(FreeConnection)then
      UserConn.Free;

    raise;
  end;
end;

procedure TOptionsForm.AddConnectionBtnClick(Sender: TObject);
begin
  AddConnection;
end;

procedure TOptionsForm.StorePasswordCBoxClick(Sender: TObject);
begin
  PasswordStorageTypeLbl.Enabled:=StorePasswordCBox.Checked;
  PasswordStorageTypeCBox.Enabled:=StorePasswordCBox.Checked;

  if(StorePasswordCBox.Checked)then
    PasswordStorageTypeCBox.ItemIndex:=0;

  DoPageContentChanged(Sender);
end;

procedure TOptionsForm.LoadPageData(PageNr: integer);
var i: integer;
  user_conns: PMYX_USER_CONNECTIONS;
  error: MYX_LIB_ERROR;
  FavoritesNode,
  HistoryNode: TTreeNode;
  StrList: TStringList;
begin
  InitControls:=True;
  try
    if(PageNr=OptionPageControl.PageCount-2)then
    begin
      //ShowTipOfDayCBox.Checked:=ApplicationDM.Options.ShowTipOfDay;
      StoreWindowsPosCBox.Checked:=ApplicationDM.Options.StoreWindowsPositions;

      LanguageCBox.Items.Text:=MYXCommonOptions.AvailableLanguages;

      StrList:=TStringList.Create;
      try
        StrList.Text:=MYXCommonOptions.AvailableLanguageCodes;
        LanguageCBox.ItemIndex:=StrList.IndexOf(MYXCommonOptions.Language);
      finally
        StrList.Free;
      end;

      StorePasswordCBox.Checked:=Not(
        (MYXCommonOptions.PasswordStorageType=MYX_PASSWORD_NOT_STORED));
      StorePasswordCBoxClick(self);

      PasswordStorageTypeCBox.Items.Text:=MYXCommonOptions.AvailablePasswordStorageTypes;
      PasswordStorageTypeCBox.ItemIndex:=Ord(MYXCommonOptions.PasswordStorageType)-2;
    end
    else if(PageNr=OptionPageControl.PageCount-1)then
    begin
      if(stored_conns=nil)then
      begin
        ConnsTreeView.Items.Clear;

        //Fetch connections
        user_conns:=myx_load_user_connections(
          PChar(AnsiToUtf8(MYXCommonOptions.UserXMLDir+'mysqlx_user_connections.xml')), @error);
        if(error<>MYX_NO_ERROR)then
          raise EMyxError.Create('Error while loading stored connections.'+
            'Error Nr.:'+IntToStr(Ord(error)));
        try
          stored_conns:=TMYX_USER_CONNECTIONS.create(user_conns);
        finally
          myx_free_user_connections(user_conns);
        end;

        FavoritesNode:=AddTreeViewChildNode(ConnsTreeView, nil,
          'Connections', 5, nil);
        HistoryNode:=AddTreeViewChildNode(ConnsTreeView, nil,
          'History', 5, nil);

        for i:=0 to stored_conns.user_connections.Count-1 do
          if(stored_conns.user_connections[i].storage_type=
            MYX_FAVORITE_USER_CONNECTION)then
          begin
            //if(TMYX_USER_CONNECTION(stored_conns.user_connections[i]).storage_path='')then
              AddTreeViewChildNode(ConnsTreeView, FavoritesNode,
                stored_conns.user_connections[i].connection_name, 6,
                stored_conns.user_connections[i]);
          end
          else
          begin
            AddTreeViewChildNode(ConnsTreeView, HistoryNode,
              stored_conns.user_connections[i].username+'@'+
                stored_conns.user_connections[i].hostname, 6,
              stored_conns.user_connections[i]);
          end;

        FavoritesNode.Expand(True);
      end;
    end
    else if(ApplOptionForm<>nil)then
      ApplOptionForm.SetControls(PageNr);
  finally
    InitControls:=False;
  end;
end;

procedure TOptionsForm.ApplyChanges;
var StrList: TStringList;
begin
  if(OptionPageControl.ActivePage=GeneralTabSheet)then
  begin
    //ApplicationDM.Options.ShowTipOfDay:=ShowTipOfDayCBox.Checked;
    ApplicationDM.Options.StoreWindowsPositions:=StoreWindowsPosCBox.Checked;

    StrList:=TStringList.Create;
    try
      StrList.Text:=MYXCommonOptions.AvailableLanguageCodes;
      MYXCommonOptions.Language:=StrList[LanguageCBox.ItemIndex];
    finally
      StrList.Free;
    end;

    if(Not(StorePasswordCBox.Checked))then
      MYXCommonOptions.PasswordStorageType:=MYX_PASSWORD_NOT_STORED
    else
    begin
      MYXCommonOptions.PasswordStorageType:=
        MYX_PASSWORD_STORAGE_TYPE(PasswordStorageTypeCBox.ItemIndex+2);
    end;

    ApplicationDM.Options.StoreOptions;
  end
  else if(OptionPageControl.ActivePage=ConnectionsTabSheet)then
  begin
    NewConnection:=nil;

    if(ConnsTreeView.Selected<>nil)then
      if(ConnsTreeView.Selected.Data<>nil)then
      begin
        with TMYX_USER_CONNECTION(ConnsTreeView.Selected.Data) do
        begin
          connection_name:=ConnectionEd.Text;
          username:=UsernameEd.Text;
          password:=PasswordEd.Text;
          hostname:=HostnameEd.Text;
          port:=StrToIntDef(PortEd.Text, 3306);
          connection_type:=MYX_USER_CONNECTION_TYPE(TypeCBox.ItemIndex);
          schema:=SchemataEd.Text;
          notes:=ConnMemo.Text;

          advanced_options.Assign(ConnValueListEditor.Strings);
        end;

        ConnsTreeView.Selected.Text:=ConnectionEd.Text;

        myx_store_user_connections(stored_conns.get_record_pointer,
          MYXCommonOptions.PasswordStorageType,
          PChar(AnsiToUtf8(MYXCommonOptions.UserXMLDir+'mysqlx_user_connections.xml')));
      end;
  end
  else if(ApplOptionForm<>nil)then
    ApplOptionForm.ApplyChanges(OptionPageControl.ActivePageIndex);

  PageContentChanged:=False;
  ApplyChangesBtn.Enabled:=False;
  DiscardChangesBtn.Enabled:=False;

  MessageToAllForms(WM_OptionsChanged, OptionPageControl.ActivePageIndex, 0);
end;

procedure TOptionsForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if(Key=VK_F1)then
    ShowHelp;
end;

end.
