unit Sections;

interface

uses Classes, Forms, Controls, ExtCtrls, Types, Graphics, Messages, Menus,
  ComCtrls, AuxFuncs, ApplicationDataModule, PNGImage;


type
  TSectionForm = class;

  TCreateSectionFormFunction =
    function (AOwner: TComponent; SidebarSectionType: integer): TSectionForm of object;

  //All individual Forms (which are triggered by the SidebarSections)
  //descend from this class so they can be docked to a TSectionPanel from the
  //MainForm more easily
  TSectionForm = class(TForm)
    constructor Create(AOwner: TComponent; StatusBar: TStatusBar = nil); reintroduce;
  private
    pDockedPanel: TPanel;
    pSubTreePanel: TPanel;

    pStatusBar: TStatusBar;

    PInitControls: Boolean;

    procedure SetDockedPanel(DockedPanel: TPanel);
    function GetDockedPanel: TPanel;

    procedure SetSubTreePanel(SubTreePnl: TPanel);
    function GetSubTreePanel: TPanel;

    procedure SetStatusBar(StatusBar: TStatusBar);
    function GetStatusBar: TStatusBar;
  public
    property InitControls: Boolean Read PInitControls Write PInitControls;
    property DockedPanel: TPanel Read GetDockedPanel Write SetDockedPanel;
    property SubTreePanel: TPanel Read GetSubTreePanel Write SetSubTreePanel;
    property StatusBar: TStatusBar Read GetStatusBar Write SetStatusBar;

    procedure InitializeControls; virtual;

    procedure SetStatusText(StatusText: string);
  end;

  //The Panel which will holds the docked SectionForms
  TSectionPanel = class(TPanel)
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
    procedure DockWinControl(WinControl: TWinControl);
    //procedure WndProc(var Msg: TMessage); override;
  private
    XPStyleEnabled: Boolean;
  end;

  TSidebarSection = class(TObject)
    constructor Create(SidebarSectionType: integer;
      IconNr: integer; DisabledIconNr: integer;
      NeedsConnection: Boolean; NeedsLocalhostConnection: Boolean;
      MenuItem: TMenuItem = nil;
      ShowSubTree: Boolean = False;
      SectionForm: TSectionForm = nil;
      SectionObject: TObject = nil);
  public
    SidebarSectionType: integer;
    SectionForm: TSectionForm;
    ShowSubTree: Boolean;
    SectionObject: TObject;
    MenuItem: TMenuItem;

    IconNr: integer;
    DisabledIconNr: integer;
    NeedsConnection: Boolean;
    NeedsLocalhostConnection: Boolean;
  end;

  TSectionControls = class(TObject)
    constructor Create(AOwner: TComponent; CreateSectionForm: TCreateSectionFormFunction;
      ViewMenuItem: TMenuItem; SectionSidebarHidden: Boolean;
      SectionSidebarWidth: integer; TreeviewImages: TImageList;
      SectionCount: integer);

    destructor Destroy; override;

    procedure AdminTreeViewChange(Sender: TObject; Node: TTreeNode);

    procedure HSplitPnlMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure HSplitPnlMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure HSplitPnlMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure HSplitPnlResize(Sender: TObject);

    function AddViewMenuItem(Name, Caption: string; Tag: integer): TMenuItem;

    procedure DoViewMenuClick(Sender: TObject);

    procedure ShowSidebarSection(Section: TSidebarSection);

    procedure HideSubTreePnl(Section: TSidebarSection = nil);
    procedure ShowSubTreePnl(Section: TSidebarSection);

    procedure RefreshSidebarIcons(Connected: Boolean;
      ConnectedToLocalhost: Boolean);
  public
    Owner: TComponent;

    CurrentSidebarSection: TSidebarSection;

    ViewMenuItem: TMenuItem;

    LeftPnl,
    SidebarPnl: TPanel;
    AdminTreeView: TTreeView;
    SubTreeSplitter: TSplitter;
    SubTreePnl: TPanel;

    HSplitPnl: TPanel;
    HSplitImg: TImage;
    HSplitMX, HSplitSize: integer;
    HSplitLeftPNGImg, HSplitRightPNGImg: TPNGObject;
    SectionSidebarHidden: Boolean;
    SectionSidebarWidth: integer;

    DockPnl: TSectionPanel;

    CreateSectionForm: TCreateSectionFormFunction;

    SectionCount: integer;
  end;

implementation

//------------------------------------------------------------------------------

constructor TSectionForm.Create(AOwner: TComponent; StatusBar: TStatusBar);
begin
  inherited Create(AOwner);

  InitForm(self);

  DockedPanel:=nil;
  SubTreePanel:=nil;
  self.StatusBar:=StatusBar;
end;

procedure TSectionForm.SetDockedPanel(DockedPanel: TPanel);
begin
  self.pDockedPanel:=DockedPanel;
end;

function TSectionForm.GetDockedPanel: TPanel;
begin
  GetDockedPanel:=pDockedPanel;
end;

procedure TSectionForm.SetSubTreePanel(SubTreePnl: TPanel);
begin
  self.pSubTreePanel:=SubTreePnl;
end;

function TSectionForm.GetSubTreePanel: TPanel;
begin
  GetSubTreePanel:=pSubTreePanel;
end;

procedure TSectionForm.SetStatusBar(StatusBar: TStatusBar);
begin
  self.pStatusBar:=StatusBar;
end;

function TSectionForm.GetStatusBar: TStatusBar;
begin
  GetStatusBar:=pStatusBar;
end;

procedure TSectionForm.SetStatusText(StatusText: string);
begin
  if(StatusBar<>nil)then
  begin
    if(StatusBar.SimplePanel)then
      StatusBar.SimpleText:=StatusText
    else if(StatusBar.Panels.Count>1)then
      StatusBar.Panels[1].Text:=StatusText
    else if(StatusBar.Panels.Count>0)then
      StatusBar.Panels[0].Text:=StatusText;
  end;
end;

procedure TSectionForm.InitializeControls;
begin
  //
end;

//------------------------------------------------------------------------------

constructor TSectionPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  XPStyleEnabled:=getXPStyleEnabled;

  if(Not(XPStyleEnabled))then
  begin
    BorderStyle:=bsSingle;
    BevelOuter:=bvNone;
  end;

  FullRepaint:=False;
end;

procedure TSectionPanel.Paint;
var Rect: TRect;
begin
  Rect := GetClientRect;

  with Canvas do
  begin
    if(XPStyleEnabled)then
    begin
      Pen.Color:=clHotLight;
      Brush.Style:=bsClear;
      Rectangle(Rect);

      {ThemeServices.DrawElement(Canvas.Handle,
        ThemeServices.GetElementDetails(ttTabItemHot), Rect);}
    end
    else
      inherited;
  end;
end;

procedure TSectionPanel.DockWinControl(WinControl: TWinControl);
var SectionForm: TSectionForm;
begin
  Visible:=False;

  //Go back to Parent of Docked WinControl and if it's a TSectionForm
  //set InitControls
  if(WinControl.Parent.InheritsFrom(TSectionForm))then
  begin
    SectionForm:=TSectionForm(WinControl.Parent);
    SectionForm.InitControls:=True;
  end
  else
    SectionForm:=nil;

  try
    WinControl.Parent:=self;
    //Get new Handles for all docked controls
    InitHandles(WinControl);
    //Resize WinControl
    WinControl.Perform(WM_SIZE, 0, 0);
  finally
    if(SectionForm<>nil)then
      SectionForm.InitControls:=False;
  end;

  Visible:=True;
end;

{procedure TSectionPanel.WndProc(var Msg: TMessage);
begin
  if(Msg.Msg=WM_ERASEBKGND)then
    Msg.Result:=0
  else
    inherited WndProc(Msg);
end;}

//------------------------------------------------------------------------------

constructor TSidebarSection.Create(SidebarSectionType: integer;
  IconNr: integer; DisabledIconNr: integer;
  NeedsConnection: Boolean; NeedsLocalhostConnection: Boolean;
  MenuItem: TMenuItem = nil;
  ShowSubTree: Boolean = False;
  SectionForm: TSectionForm = nil;
  SectionObject: TObject = nil);
begin
  inherited Create;

  self.SidebarSectionType:=SidebarSectionType;
  self.SectionForm:=SectionForm;
  self.ShowSubTree:=ShowSubTree;
  self.SectionObject:=SectionObject;
  self.MenuItem:=MenuItem;
  self.IconNr:=IconNr;
  self.DisabledIconNr:=DisabledIconNr;
  self.NeedsConnection:=NeedsConnection;
  self.NeedsLocalhostConnection:=NeedsLocalhostConnection;
end;

//------------------------------------------------------------------------------

constructor TSectionControls.Create(AOwner: TComponent; CreateSectionForm: TCreateSectionFormFunction;
  ViewMenuItem: TMenuItem; SectionSidebarHidden: Boolean;
  SectionSidebarWidth: integer; TreeviewImages: TImageList;
  SectionCount: integer);
begin
  inherited Create;

  self.Owner:=AOwner;
  self.CreateSectionForm:=CreateSectionForm;
  self.ViewMenuItem:=ViewMenuItem;
  self.SectionSidebarHidden:=SectionSidebarHidden;
  self.SectionSidebarWidth:=SectionSidebarWidth;
  self.SectionCount:=SectionCount;

  HSplitLeftPNGImg:=LoadPNGImageFromResource('sizer_h_left');
  HSplitRightPNGImg:=LoadPNGImageFromResource('sizer_h_right');

  CurrentSidebarSection:=nil;

  LeftPnl:=TPanel.Create(Owner);
  LeftPnl.Parent:=TWinControl(Owner);
  LeftPnl.BevelOuter:=bvNone;
  LeftPnl.Name:='SidebarLeftPnl';
  LeftPnl.Caption:='';
  LeftPnl.Align:=alLeft;
  LeftPnl.Width:=3;

  SidebarPnl:=TPanel.Create(Owner);
  SidebarPnl.Parent:=TWinControl(Owner);
  SidebarPnl.BevelOuter:=bvNone;
  SidebarPnl.Name:='SidebarPnl';
  SidebarPnl.Caption:='';
  SidebarPnl.Align:=alLeft;
  SidebarPnl.Width:=185;

  AdminTreeView:=TTreeView.Create(Owner);
  AdminTreeView.Parent:=SidebarPnl;
  AdminTreeView.Name:='AdminTreeView';
  AdminTreeView.Align:=alClient;
  AdminTreeView.Height:=247;
  AdminTreeView.Images:=TreeviewImages;
  AdminTreeView.HideSelection:=False;
  AdminTreeView.ReadOnly:=True;
  AdminTreeView.RightClickSelect:=True;
  AdminTreeView.ShowLines:=False;
  AdminTreeView.ShowRoot:=False;
  AdminTreeView.OnChange:=AdminTreeViewChange;

  SubTreeSplitter:=TSplitter.Create(Owner);
  SubTreeSplitter.Parent:=SidebarPnl;
  SubTreeSplitter.Name:='SubTreeSplitter';
  SubTreeSplitter.Align:=alTop;
  SubTreeSplitter.Height:=3;
  SubTreeSplitter.Visible:=False;

  SubTreePnl:=TPanel.Create(Owner);
  SubTreePnl.Parent:=SidebarPnl;
  SubTreePnl.Name:='SubTreePnl';
  SubTreePnl.Caption:='';
  SubTreePnl.Align:=alClient;
  SubTreePnl.BevelOuter:=bvNone;
  SubTreePnl.Visible:=False;

  HSplitPnl:=TPanel.Create(Owner);
  HSplitPnl.Parent:=TWinControl(Owner);
  HSplitPnl.Name:='HSplitPnl';
  HSplitPnl.Caption:='';
  HSplitPnl.Align:=alLeft;
  HSplitPnl.BevelOuter:=bvNone;
  HSplitPnl.Cursor:=crHSplit;
  HSplitPnl.Width:=7;
  HSplitPnl.OnResize:=HSplitPnlResize;
  HSplitPnl.OnMouseDown:=HSplitPnlMouseDown;
  HSplitPnl.OnMouseMove:=HSplitPnlMouseMove;
  HSplitPnl.OnMouseUp:=HSplitPnlMouseUp;

  HSplitImg:=TImage.Create(Owner);
  HSplitImg.Parent:=HSplitPnl;
  HSplitImg.Name:='HSplitImg';
  HSplitImg.Height:=41;
  HSplitImg.Left:=1;
  HSplitImg.Width:=5;
  HSplitImg.Top:=214;
  HSplitImg.OnMouseDown:=HSplitPnlMouseDown;
  HSplitImg.OnMouseMove:=HSplitPnlMouseMove;
  HSplitImg.OnMouseUp:=HSplitPnlMouseUp;

  SidebarPnl.Left:=0;
  LeftPnl.Left:=0;

  DockPnl:=TSectionPanel.Create(Owner);
  DockPnl.Parent:=TWinControl(Owner);
  DockPnl.Name:='DockPnl';
  DockPnl.Caption:='';
  DockPnl.Align:=alClient;

  if(Not(SectionSidebarHidden))then
  begin
    HSplitImg.Picture.Graphic:=HSplitLeftPNGImg;
    SidebarPnl.Width:=SectionSidebarWidth;
  end
  else
  begin
    HSplitImg.Picture.Graphic:=HSplitRightPNGImg;
    LeftPnl.Width:=0;
    SidebarPnl.Width:=0;
  end;
end;

destructor TSectionControls.Destroy;
begin
  HSplitLeftPNGImg.Free;
  HSplitRightPNGImg.Free;

  inherited Destroy;
end;

procedure TSectionControls.AdminTreeViewChange(Sender: TObject; Node: TTreeNode);
begin
  if(Assigned(Node))then
    if(Assigned(Node.Data))then
      ShowSidebarSection(TSidebarSection(Node.Data));
end;

procedure TSectionControls.HSplitPnlMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  HSplitMX:=Mouse.CursorPos.X;
  HSplitSize:=SidebarPnl.Width;
end;

procedure TSectionControls.HSplitPnlMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var newWidth: integer;
begin
  if(ssLeft in Shift)then
  begin
    newWidth:=HSplitSize+(Mouse.CursorPos.X-HSplitMX);

    if(newWidth<250)then
    begin
      if(newWidth>20)then
      begin
        LeftPnl.Width:=3;
        SidebarPnl.Width:=newWidth;
        SectionSidebarHidden:=False;
        HSplitImg.Picture.Graphic:=HSplitLeftPNGImg;
      end
      else
      begin
        LeftPnl.Width:=0;
        SidebarPnl.Width:=0;
        SectionSidebarHidden:=True;
        HSplitImg.Picture.Graphic:=HSplitRightPNGImg;
      end;
    end;
  end;
end;

procedure TSectionControls.HSplitPnlMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if(HSplitMX=Mouse.CursorPos.X)then
  begin
    if(SidebarPnl.Width=0)then
    begin
      LeftPnl.Width:=3;
      SidebarPnl.Width:=SectionSidebarWidth;
      SectionSidebarHidden:=False;
      HSplitImg.Picture.Graphic:=HSplitLeftPNGImg;
    end
    else
    begin
      LeftPnl.Width:=0;
      SidebarPnl.Width:=0;
      SectionSidebarHidden:=True;
      HSplitImg.Picture.Graphic:=HSplitRightPNGImg;
    end;
  end
  else
  begin
    if(SidebarPnl.Width>0)then
      SectionSidebarWidth:=SidebarPnl.Width;
  end;
end;

procedure TSectionControls.HSplitPnlResize(Sender: TObject);
begin
  HSplitImg.Top:=(HSplitPnl.Height-HSplitImg.Height) div 2;
end;

function TSectionControls.AddViewMenuItem(Name, Caption: string; Tag: integer): TMenuItem;
var MenuItem: TMenuItem;
begin
  MenuItem:=TMenuItem.Create(Owner);
  MenuItem.Name:=Name;
  MenuItem.Caption:=Caption;
  MenuItem.Tag:=Tag;
  MenuItem.OnClick:=DoViewMenuClick;
  ViewMenuItem.Add(MenuItem);

  Result:=MenuItem;
end;

procedure TSectionControls.DoViewMenuClick(Sender: TObject);
begin
  if(TMenuItem(Sender).Tag<AdminTreeView.Items.Count)then
    AdminTreeView.Selected:=AdminTreeView.Items[TMenuItem(Sender).Tag];
end;

procedure TSectionControls.ShowSidebarSection(Section: TSidebarSection);
begin
  //hide current
  if(Assigned(CurrentSidebarSection))then
  begin
    CurrentSidebarSection.SectionForm.DockedPanel.Hide;
    if(CurrentSidebarSection.SectionForm.SubTreePanel<>nil)then
      HideSubTreePnl(CurrentSidebarSection);
  end;

  if(Assigned(Section.SectionForm))then
  begin
    Section.SectionForm.DockedPanel.Show;
    if(Section.ShowSubTree)then
      ShowSubTreePnl(Section);
  end
  else
  begin
    Section.SectionForm:=
      CreateSectionForm(Owner, Section.SidebarSectionType);

    if(Section.SectionForm=nil)then
      Exit;

    DockPnl.DockWinControl(Section.SectionForm.DockedPanel);

    if(Assigned(Section.SectionForm.SubTreePanel))then
    begin
      Section.SectionForm.InitControls:=True;
      try
        Section.SectionForm.SubTreePanel.Parent:=SubTreePnl;
        Section.SectionForm.SubTreePanel.Align:=alClient;
      finally
        Section.SectionForm.InitControls:=False;
      end;
    end;

    //Dock panels from SectionForm
    if(Section.ShowSubTree)then
      ShowSubTreePnl(Section)
    else
      HideSubTreePnl;

    Section.SectionForm.InitializeControls;
  end;

  //Uncheck last View MenuItem
  if(CurrentSidebarSection<>nil)then
    if(CurrentSidebarSection.MenuItem<>nil)then
      CurrentSidebarSection.MenuItem.Checked:=False;
  //Check new current View MenuItem
  if(Section.MenuItem<>nil)then
    Section.MenuItem.Checked:=True;

  CurrentSidebarSection:=Section;
end;

procedure TSectionControls.HideSubTreePnl(Section: TSidebarSection);
begin
  if(Assigned(Section))then
    if(Section.SectionForm.SubTreePanel<>nil)then
      Section.SectionForm.SubTreePanel.Hide;

  SubTreePnl.Hide;
  SubTreeSplitter.Hide;
  AdminTreeView.Align:=alClient;
end;

procedure TSectionControls.ShowSubTreePnl(Section: TSidebarSection);
begin
  if(Assigned(Section))then
    if(Section.SectionForm.SubTreePanel<>nil)then
    begin
      AdminTreeView.Align:=alTop;
      AdminTreeView.Height:=(SectionCount+1)*24;//250+24;
      SubTreePnl.Height:=192;
      SubTreePnl.Visible:=True;
      SubTreeSplitter.Top:=1000;
      SubTreeSplitter.Show;

      Section.SectionForm.SubTreePanel.Show;
      Section.SectionForm.SubTreePanel.Align:=alClient;
    end;
end;

procedure TSectionControls.RefreshSidebarIcons(Connected: Boolean;
  ConnectedToLocalhost: Boolean);
var i: integer;
begin
  for i:=0 to AdminTreeView.Items.Count-1 do
  begin
    if(AdminTreeView.Items[i].Data<>nil)then
    begin
      if(((TSidebarSection(AdminTreeView.Items[i].Data).NeedsConnection)and
          (Not(Connected)))or
        ((TSidebarSection(AdminTreeView.Items[i].Data).NeedsLocalhostConnection)and
          (Not(ConnectedToLocalhost))))then
      begin
        AdminTreeView.Items[i].ImageIndex:=
          TSidebarSection(AdminTreeView.Items[i].Data).DisabledIconNr;
        AdminTreeView.Items[i].SelectedIndex:=
          TSidebarSection(AdminTreeView.Items[i].Data).DisabledIconNr;
      end
      else
      begin
        AdminTreeView.Items[i].ImageIndex:=
          TSidebarSection(AdminTreeView.Items[i].Data).IconNr;
        AdminTreeView.Items[i].SelectedIndex:=
          TSidebarSection(AdminTreeView.Items[i].Data).IconNr;
      end;
    end;
  end;

  AdminTreeView.Refresh;
end;


end.
